#!/usr/bin/env python

##################################
## MoDIL (seunghak@cs.toronto.edu)

from __future__ import division
import os
import sys

#################################
## directories should be changed
MODIL_HOME='/home/seunghak/structural_variations/mrfstructvar'
#################################

MODULE_DIR = MODIL_HOME + '/modules'
sys.path.append(MODIL_HOME)
sys.path.append(MODULE_DIR)
import datetime
from sets import Set
import math
import MPProperties
import Statistics
import copy
import random
import gzip
from random import Random
from time   import *

# GLOBAL CONSTANTS
db = None

mean_list={}
std_list={}
chi_sig_val={}

alpha={}
getprob={}
WIN=10
CLUS_WIN=1

##########################
# probability distribution
exp_dist={}
g_cum_prob_dist={}
probdist={}
probdist_keys={}

#---------
props = {}

################################
## YOU MAY CHANGE THE FOLLOWING 
#MINIMUM_NUM_OF_MATEPAIRS=5  # 5
MAXIMUM_NUM_OF_MATEPAIRS=100 # 200
MAX_ITER=5
MAX_ITER_INIT=5 # 9
PVAL_THRESHOLD=0.025
SMOOTH_WINDOW=2
#PVAL_THRESHOLD=0.001


###########################################
## PLEASE DO NOT CHANGE THE FOLLOWING CODE
HOMOZYGOUS=0
HETEROZYGOUS=1
MIXED=2
TAB="\t"
LARGE_INDEL_SIZE=1000
#MAX_DEL_SIZE=10000
FILTER_KS_STAT_ON = 0
RESPONSIBILITY_CUT_OFF = 0.7
file_prefix=""

## penalty for not having observation
## small value favors deletion and large value favors insertion
## set between 2.1 and 2.5
## NO_OBS_PANALTY=10**-(2.8)

def initialize_global_vars():
    # set the properties hash
    global props
    # read the properties hash
    props = MPProperties.read_config()

## KL divergence
def compute_KL(ins_size, exp_dist, mu_diff, bin_set,CHI_BIN):
    #for i in range(len(P)):
    #    entry_of_P = P[i]
    #    entry_of_Q = Q[i]
    #if len(P)!=len(Q):
    #    return -1
    COST=0
    #for j in range(len(P)):
    #for j in probdist[ins_size].keys():
    for j in range(0, 2*ins_size,CHI_BIN):
        bin=int((j-mu_diff)/CHI_BIN)*CHI_BIN
        if bin in bin_set:
            O=exp_dist[bin]
        else:
            O=0

        #if j in probdist_keys[ins_size]:
        E=probdist[ins_size][j]
        #else:
        #E=10**-5
        #    print j

        if O>0 and E>0:
            COST+=(E*math.log(E/O))
        #    tmp=(E*math.log(E/O))
        elif O==0 and E>0:
            COST+=(E*math.log(E/NO_OBS_PANALTY))

        #if (mu_diff==0 or mu_diff==40) and O>0:
        #    print "***************** ", tmp, mu_diff, j, E, O, ins_size
        #elif O!=0 and probdist[ins_size][j]==0:
        #    COST+=probdist[ins_size][j]*math.log(TINY/O)

    return COST


def fast_EM_mog(x_list, mean_list_clus, CHI_BIN):
    #[mu_new, iter, r_num, ks_stats]=fast_EM_mog(m_list, size_list, CHI_BIN)
    # find 0 0 
    #MEDIAN_DEL_SIZE = 500

    min_density = 0.0001
    
    cnt = 0
    ndist_list = []
    max_ins_size = mean_list_clus[0]
    max_std_size = std_list[mean_list_clus[0]]
    probdist_keys_set = {}
    mset = Set()
    for x in x_list:
        ndist = mean_list_clus[cnt]-x
        ndist_list.append([x, mean_list_clus[cnt]])
        max_ins_size = max(max_ins_size,mean_list_clus[cnt])
        max_std_size = max(max_std_size,std_list[mean_list_clus[cnt]])
        if mean_list_clus[cnt] not in mset:
            probdist_keys_set[mean_list_clus[cnt]] = Set(probdist[mean_list_clus[cnt]].keys())
            mset.add(mean_list_clus[cnt])
            
        cnt=cnt+1
    
    MEDIAN_DEL_SIZE = max_std_size * 5
    # fix one distribution    

    #print probdist[208]
    indel_size = max_ins_size
       

    # hetero
    likeli_list=[] 
    while 1:
        likeli = 0
        den2_cnt = 0
        r_cnt = [0,0]
        r_new = []
        r_std = [[],[]]
        for d in ndist_list:
            if d[0] in  probdist_keys_set[d[1]]:
                den1 = probdist[d[1]][d[0]]
            else:
                den1 = min_density
            if d[0]+indel_size in  probdist_keys_set[d[1]]:
                den2 = probdist[d[1]][d[0]+indel_size]
            else:
                den2 = min_density
            if den1>den2:
                likeli += -math.log(den1)
                r_cnt[0]+=1
                r_new.append([1,0])
                r_std[0].append(d[0])
            else:
                likeli += -math.log(den2)
                den2_cnt +=1
                r_cnt[1]+=1
                r_new.append([0,1])
                r_std[1].append(d[0]+indel_size)

        if sum(r_cnt)!=0: # heterozygous 
            ratio = min(r_cnt[0],r_cnt[1])/sum(r_cnt)  # 0-0.5
        else:
            ratio = 0

        indel_size = indel_size-1
        if -indel_size>MEDIAN_DEL_SIZE:
            break
        likeli_list.append([likeli,[0,indel_size],ratio,r_cnt,r_new,r_std])
        
    # homozygous, move both
    indel_size = max_ins_size
    while 1:
        likeli = 0
        den2_cnt = 0
        r_new=[]
        r_std=[[],[]]
        for d in ndist_list:
            if d[0]+indel_size in  probdist_keys_set[d[1]]:
                den2 = probdist[d[1]][d[0]+indel_size]
                den2_cnt+=1
            else:
                den2 = min_density 
                den2_cnt +=1
            likeli += -math.log(den2)
            r_new.append([0,1])
            r_std[1].append(d[0]+indel_size)

        indel_size = indel_size-1
        if -indel_size>MEDIAN_DEL_SIZE:
            break
        likeli_list.append([likeli,[indel_size,indel_size],1,[0,len(ndist_list)],r_new,r_std])


    # for large indels
    x_list = []
    r_cnt = [0,0] 
    r_new=[]
    for d in ndist_list:
        if d[0] > (d[1] + 3*max_std_size) or d[0] < (d[1] - 3*max_std_size):
            x_list.append(d[1]-d[0])
            r_cnt[1] +=1
            r_new.append([0,1])
        else:
            r_cnt[0] += 1
            r_new.append([1,0])
    if len(x_list) == 0:
        indel_size = 0
    else:
        indel_size = sum(x_list)/len(x_list)
        
    likeli = 0
    den2_cnt = 0
    r_std=[[],[]]
    for d in ndist_list:
        if d[0] in  probdist_keys_set[d[1]]:
            den1 = probdist[d[1]][d[0]]
        else:
            den1 = min_density
            
        if d[0]+indel_size in  probdist_keys_set[d[1]]:
            den2 = probdist[d[1]][d[0]+indel_size]
        else:
            den2 = min_density
        if den1>den2:
            likeli += -math.log(den1)
            r_std[0].append(d[0]) 
        else:
            likeli += -math.log(den2)
            den2_cnt +=1
            r_std[1].append(d[0]+indel_size) 
    if sum(r_cnt)!=0:
        ratio = min(r_cnt[0],r_cnt[1])/sum(r_cnt)  # 0-0.5
    else:
        ratio = 0
    if r_cnt[0]!=0: # heterozygous 
        likeli_list.append([likeli,[0,indel_size],ratio,r_cnt,r_new,r_std])
    else: # homozygous 
        likeli_list.append([likeli,[indel_size,indel_size],ratio,r_cnt,r_new,r_std])

    likeli_list.sort()
    best_result = likeli_list[0]
   
    # compute std for each distribution
    best_r_std = likeli_list[0][5]  

    best_mean = [-1,-1]
    for dn in [0,1]:
        if len(best_r_std[dn]) != 0:
            best_mean[dn] = sum(best_r_std[dn])/len(best_r_std[dn])

    b_std = [0,0]
    for dn in [0,1]:
        for b in best_r_std[dn]:
            b_std[dn] += (b-best_mean[dn])**2
        if len(best_r_std[dn])!=0:
            b_std[dn] = b_std[dn]/len(best_r_std[dn])
            b_std[dn] = b_std[dn] ** 0.5
        else:
            b_std[dn] = -1


    # compute heterozygosity
    # likelihood with heterozygous model

    #indel_size = best_result[1][1]
    #het_likeli = 0
    #for d in ndist_list:
    #    if d[0] in  probdist_keys_set[d[1]]:
    #        den1 = probdist[d[1]][d[0]]
    #    else:
    #        den1 = min_density
    #    if d[0]+indel_size in  probdist_keys_set[d[1]]:
    #        den2 = probdist[d[1]][d[0]+indel_size]
    #    else:
    #        den2 = min_density
#
#        if den1>den2 and b_std[0]>0:
#            het_likeli += -math.log(den1)
#            #het_likeli += (d[0]-d[1])**2 / 2*(b_std[0]**2)
#        elif b_std[1]>0:
#            het_likeli += -math.log(den2)
#            #het_likeli += (d[0]+indel_size-d[1])**2 / 2*(b_std[1]**2)
#
#
#    # likelihood with homozygous model
#    hom_likeli = 0
#    for d in ndist_list:
#        if d[0]+indel_size in  probdist_keys_set[d[1]]:
#            den2 = probdist[d[1]][d[0]+indel_size]
#        else:
#            den2 = min_density
#        hom_likeli += -math.log(den2)
        #if b_std[1]>0:
        #    hom_likeli += (d[0]+indel_size-d[1])**2 / (2*b_std[1]**2)
         
    #het_prob = 0 
    #D = 0
    #if hom_likeli != 0:
    #    D = 2*hom_likeli - 2*het_likeli
    #print D, hom_likeli, het_likeli

    #[mu_new, iter, r_num, ks_stats]
    ret = [best_result[1],best_result[2],best_result[3],best_result[0],best_result[4],b_std] 
    # indelsize, hom(1)/het(0), number of mp in each dist, likeli, membership

    return ret



def EM_mog(loglikeli_init, x_list, mu, sigma, r, mean_list_clus, CHI_BIN):
    STEP_SIZE=1
    #print 
    #print 
    #print 
    #print 
    #print 
    #print '>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>> INPUT DATA : ', x_list
    #print 

    #print "EM to optimize mixture of gaussians"
    N=len(x_list)
    ret_param=[[0,0], [300,300], r, loglikeli_init, 0, [1,1],[0],[300,300],-1]
    #print "less than minimum matepairs"
    #if N<MINIMUM_NUM_OF_MATEPAIRS:
    if N<5:
        print '!!!!!!!!!!!!!!!!! less than the minimum num of matepairs !!!!!!!!!!!!!!!!!!!'
        return ret_param

    ins_size_set=Set()
    for i in range(N):
        ins_size=mean_list_clus[i]
        ins_size_set.add(ins_size)

    ##########################################3

    tmp_uni=0
    for i in range(N):
        tmp_uni=tmp_uni+(mean_list[mean_list_clus[i]]-x_list[i])
    mu_uni=tmp_uni/N

    if abs(mu_uni)>20000: ## if it's too large, skip
        print '!!!!!!!!!!!!!!!!! larger than the maximum indel size !!!!!!!!!!!!!!!!!!!'
	return ret_param

    probdist_keys_alternate={}
    for ins_size in ins_size_set:
        probdist_keys[ins_size]=probdist[ins_size].keys()
        probdist_keys[ins_size].sort()
        probdist_keys_alternate[ins_size]=[]
	tmp_lst=[]
        for pkey in probdist_keys[ins_size]:
	    tmp_lst.append([-probdist[ins_size][pkey], pkey])
        tmp_lst.sort()
	for tmp in tmp_lst:
	    probdist_keys_alternate[ins_size].append(tmp[1])
	
    #print 'alternate : ', probdist_keys_alternate

    ### p(Y) dist
    weight_ins_sz={}
    cum_prob_dist={}
    for ins_size in ins_size_set:
        cum_prob_dist[ins_size]={}
        weight_ins_sz[ins_size]=0

    for ins_size in ins_size_set:
        cum_E=0
        for ebin in probdist_keys[ins_size]:
            cum_E+=probdist[ins_size][ebin]
            cum_prob_dist[ins_size][ebin]=cum_E

    #print 'cum prob dist : ', cum_prob_dist
    ##################################
    ## make experimental distribution
    bin_list={}
    cum_exp_dist_tot={}
    max_exp_tot={}
    min_exp_tot={}
    ins_size_=0
    for ins_size in ins_size_set:
        cum_exp_dist_tot[ins_size]={}
        bin_list[ins_size]=[]
        max_exp_tot[ins_size]=-1
        min_exp_tot[ins_size]=10**10
	ins_size_=ins_size

    #################################
    ## insert size
    aaa=mean_list[ins_size_]-min(x_list)
    bbb=mean_list[ins_size_]-max(x_list)
    RANGE_SEARCH=[int(round(bbb)),int(round(aaa))]
    for i in range(N):
        sz=mean_list_clus[i]
        bin=int(x_list[i]/CHI_BIN)*CHI_BIN
        bin_list[sz].append(bin)
        weight_ins_sz[sz]+=1
	max_exp_tot[sz]=max(max_exp_tot[sz],bin)
        min_exp_tot[sz]=min(min_exp_tot[sz],bin)

    for ins_size in ins_size_set:
        bin_list[ins_size].sort()

	#print 'global', max_exp_tot[ins_size], min_exp_tot[ins_size]
	if abs(max_exp_tot[ins_size]-min_exp_tot[ins_size])>LARGE_INDEL_SIZE:
	    ind=0
	    for bin in bin_list[ins_size]:
	        ind+=1
                cum_exp_dist_tot[ins_size][bin]=ind/len(bin_list[ins_size])
	
	else:
            for ebin in range(min_exp_tot[ins_size],max_exp_tot[ins_size]+STEP_SIZE,STEP_SIZE):
                ind=0
                for bin in bin_list[ins_size]:
                    if bin<=ebin:
                        ind+=1
                    else:
                        break
                cum_exp_dist_tot[ins_size][ebin]=ind/len(bin_list[ins_size])

    #########################################
    ## Using EM, maximize the log likelihood

    iter_init=0
    best_score=10**30
    loglikeli_new=10**30
    #chi_fitness=[best_score,best_score]
    pval=[1,1]

    while(1): ## multiple initial starting point
	model_type=iter_init ## odd: hetero, even homo
        if iter_init>=MAX_ITER_INIT:
            break
        #print iter_init, loglikeli_new, best_score
        if iter_init==0:
            mu=[0,0]
        else:
            if iter_init==1:  ## heterozygous
                mu=[int(round(mu_uni*2)),0]
	    elif iter_init==2:  ## homozygous
                mu=[int(round(mu_uni)),int(round(mu_uni))]
	    elif iter_init==3:
                mu=[-int(round(mu_uni)),int(round(mu_uni))]
		
            else: #if iter_init==2: #iter_init==2:
                #if bbb<0 and aaa>0:
                #print bbb,aaa
		tmp_mu=random.randrange(bbb-1, aaa+1)
		if model_type % 3 == HOMOZYGOUS: ## homozygous
	            mu=[tmp_mu, tmp_mu]
		elif model_type % 3 == HETEROZYGOUS: ## hetero
	            mu=[tmp_mu, 0]
		elif model_type % 3 == MIXED: ## hetero
	            mu=[tmp_mu, 0]

        ## prior
        pi={}
	for ins_size in ins_size_set:
	    if model_type % 3 == HOMOZYGOUS: 
	        pi[ins_size]=[1,0]
                #print '--  HOMOZYGOUS'
                #print 
            elif model_type % 3 == HETEROZYGOUS:
	        pi[ins_size]=[0.5,0.5] # second distribution is fixed to mean of 0
                #print '--  HETEROZYGOUS'
                #print
            elif model_type % 3 == MIXED:
	        pi[ins_size]=[0.5,0.5]
                #print '--  MIXED'
                #print

        loglikeli_old=10**30
        loglikeli_best=10**30
        iter=0
        stop=0
        mu_prev=[0,0]
	mu_prev_second=0

        #print 'starting mu: ', mu
        while(iter<MAX_ITER):
            #print mu
            #####################################
            ## sort by the order of g(l1)-g(l2)
            r={} ## responsibility
            out_range_cnt=0
            for i in range(N):
                ins_size=mean_list_clus[i]
                bin1=round(x_list[i]+mu[0])
                bin2=round(x_list[i]+mu[1])

                if bin1 in probdist_keys[ins_size]:
                    g1 = probdist[ins_size][bin1]
                else:
                    g1 = 0
                if bin2 in probdist_keys[ins_size]:
                    g2 = probdist[ins_size][bin2]
                else:
                    g2 = 0
                r_0=pi[ins_size][0]*g1
                r_1=pi[ins_size][1]*g2
                #print r_0, r_1, pi 
                #######################################################
                ## we do not maximize pi so cannot involve prior here
                #r_0=g1
                #r_1=g2
                #print r_0, r_1
                p_fn=r_0+r_1
		#if x_list[i]<=185:
		#    print x_list[i], r_0, r_1, mu, bin1, bin2, [r_0/p_fn, r_1/p_fn]
                if r_0==0 and r_1==0:
                    #print "both zero", bin1, bin2
                    dif1=abs(mean_list[ins_size]-(x_list[i]+mu[0]))
                    dif2=abs(mean_list[ins_size]-(x_list[i]+mu[1]))
                    if dif1>dif2:
                        #r[i]=[0.0001, 0.9999]
                        r[i]=[0, 1]
                    else:
                        #r[i]=[0.9999, 0.0001]
                        r[i]=[1, 0]
                    out_range_cnt+=1
                else:
                    r[i]=[r_0/p_fn, r_1/p_fn]
                #print pi[ins_size][0], pi[ins_size][1], bin1, bin2, g1, g2, r[i]
            clus_size_list={}
            for ins_size in ins_size_set:
                clus_size_list[ins_size]=[0,0]

            #print 'prior: ', pi
            #print 'means : ', mu
            #print 'list : ', x_list
            #print r
            #N_k=[0,0]
            for i in range(N):
                ins_size=mean_list_clus[i]
                #N_k[0]+=r[i][0]
                #N_k[1]+=r[i][1]
                clus_size_list[ins_size][0]+=r[i][0]
                clus_size_list[ins_size][1]+=r[i][1]


            for ins_size in ins_size_set:
                pi1=clus_size_list[ins_size][0]
                pi2=clus_size_list[ins_size][1]
                pi[ins_size]=[pi1/(pi1+pi2),pi2/(pi1+pi2)]

                #norm_f+=pi1+pi2
            #print pi, N_k, N,r
            #member_weight=[N_k[0]/N,N_k[1]/N]

            #########################################
            ## 1st MAXIMIZATION STEP
	    #loglikeli_new=0
	    #mu=[0,0]
	    if model_type%3==HETEROZYGOUS or model_type%3==HOMOZYGOUS:
	        [loglikeli_new, mu, ks]=first_M_step(ins_size_set,r,pi,x_list,mean_list_clus,mu_prev,clus_size_list,N,cum_prob_dist,probdist_keys_alternate, model_type)
	    elif model_type%3==MIXED:
	        [loglikeli_new, mu, ks]=first_M_step(ins_size_set,r,pi,x_list,mean_list_clus,mu_prev,clus_size_list,N,cum_prob_dist,probdist_keys_alternate, model_type)
	        [loglikeli_new, mu, mu_prev_second]=second_M_step(mu_prev_second, mu, ins_size_set, RANGE_SEARCH,loglikeli_new,cum_exp_dist_tot,min_exp_tot,max_exp_tot)
		
	    ########################
	    ## set prior
	    if model_type%3==HETEROZYGOUS:
		loglikeli_new*=1 ## prior 1/3, 2/3, 1/9
	    elif model_type%3==HOMOZYGOUS:
		loglikeli_new*=1.5 ## prior 1/3, 2/3, 1/9
	    elif model_type%3==MIXED:
		loglikeli_new*=2 ## prior 

            
            if loglikeli_new<=loglikeli_best or iter==0:
                loglikeli_best=loglikeli_new
                ks_best=copy.copy(ks)
                mu_best=copy.copy(mu)
                r_best=copy.copy(r)
                pi_best=copy.copy(pi)

            if (loglikeli_new>=loglikeli_old and iter>1) or (loglikeli_new+10**-4>loglikeli_old and iter>3):
                stop+=1
            mu_prev=copy.copy(mu)
 
            if stop>=1:
                break

            loglikeli_old=loglikeli_new
            iter=iter+1
        
        ###########################
        iter_init=iter_init+1

        if best_score>=loglikeli_best or iter_init==1:
       	    best_score=loglikeli_best
            best_ks_stats=ks_best
	    mu_best_=copy.copy(mu_best)
	    pi_best_=copy.copy(pi_best)
	    r_num=[0,0]  	 
            for ins_size in ins_size_set:
	        r_num[0]=len(x_list)*pi_best_[ins_size][0]
	        r_num[1]=len(x_list)*pi_best_[ins_size][1]

	    r_={}
	    randomgen=Random(time())
	    for i in range(N):
	        ins_size=mean_list_clus[i]
	        ## bug fix - Mar. 21th
	        ratio=r_best[i][0]
		cast_dice=randomgen.uniform(0, 1)
	        #cast_dice=random.random() ## determine membership
	        #print ratio, cast_dice, r_best[i]
	        if cast_dice>ratio:
	            r_[i]=[0,1]
	    	else:# cast_dice<ratio:
		    r_[i]=[1,0]
	    ret_param=[mu_best_, -1, r_, best_score, iter, pval, r_num, best_ks_stats, pi_best_]
        #print best_ks_stats

    return ret_param

   
def second_M_step(mu_prev_second, mu, ins_size_set,RANGE_SEARCH,loglikeli_new,cum_exp_dist_tot,min_exp_tot,max_exp_tot):
    ################################
    ## second optimization: fix mu
    if abs(mu[0])<abs(mu[1]):
	second_mu=mu[1]-mu[0]
    else:
	second_mu=mu[0]-mu[1]

    prob_dist_tot={}
    prob_dist_tot_set={}
    cum_prob_dist_tot={}
    probdist_totkey_alternate={}
    for ins_size in ins_size_set:
	prob_dist_tot[ins_size]={}
	prob_dist_tot_set[ins_size]=Set()
	cum_prob_dist_tot[ins_size]={}
	probdist_totkey_alternate[ins_size]={}
	
    cum_dist_keys=cum_exp_dist_tot[ins_size].keys()
    cum_dist_keys.sort()
    #print mu_

    for ins_size in ins_size_set:
	for ebin in probdist_keys[ins_size]:
	    prob_dist_tot[ins_size][ebin]=0

	for ebin in probdist_keys[ins_size]:
	    prob_dist_tot[ins_size][ebin]+=probdist[ins_size][ebin]/2
	    prob_dist_tot_set[ins_size].add(ebin)

	    ###################
	    ## second mu
	    bin=ebin-second_mu
	    if bin in prob_dist_tot_set[ins_size]:
		prob_dist_tot[ins_size][bin]+=probdist[ins_size][ebin]/2
	    else:
		prob_dist_tot[ins_size][bin]=probdist[ins_size][ebin]/2
		prob_dist_tot_set[ins_size].add(bin)

        probdist_tot_keys=prob_dist_tot_set[ins_size]
	min_max_probdist_totkey=[min(prob_dist_tot_set[ins_size]),max(prob_dist_tot_set[ins_size])]

	tmp_lst=[]
        for pkey in range(min_max_probdist_totkey[0],min_max_probdist_totkey[1]+1,1):
	    if pkey in probdist_tot_keys:
	        tmp_lst.append([-prob_dist_tot[ins_size][pkey], pkey])
	    else:
	        tmp_lst=[[0, pkey]]
        tmp_lst.sort()
        probdist_totkey_alternate[ins_size]=[]
	for tmp in tmp_lst:
	    probdist_totkey_alternate[ins_size].append(tmp[1])
	cum_E=0
	#print probdist_totkey_alternate[ins_size]
	#print
	#print range(min(prob_dist_tot_set[ins_size]),max(prob_dist_tot_set[ins_size])+1,1)
	for ebin in range(min_max_probdist_totkey[0],min_max_probdist_totkey[1]+1,1):
	    if ebin in probdist_tot_keys:
		cum_E+=prob_dist_tot[ins_size][ebin]
	    cum_prob_dist_tot[ins_size][ebin]=cum_E
	#print min_max_probdist_totkey

    #for ins_size in ins_size_set:
    #    print "---"
#	for ebin in range(min_max_probdist_totkey[0],min_max_probdist_totkey[1]+1,1):
#            print ebin, cum_prob_dist_tot[ins_size][ebin]

    #######################
    ### second optimization
    sup_val={}
    for ins_size in ins_size_set:
	sup_val[ins_size]=-1

    fitness_best=10**30
    
    RANGE_SEARCH_=[mu_prev_second,0]
    RANGE_SEARCH_+=range(RANGE_SEARCH[0],RANGE_SEARCH[1])
    if len(RANGE_SEARCH_)>1000:
	mean_all=sum(mu)/2
        RANGE_SEARCH_=[mu_prev_second,0]
        RANGE_SEARCH_+=range(int(mean_all-50), int(mean_all+50)) ## TODO: std_dev

    fixed_mu=0
    for mu_moving in RANGE_SEARCH_:
	fitness=0
	for ins_size in ins_size_set:
	    filter_bin=mean_list[ins_size]-mu_moving
	    if filter_bin<min_exp_tot[ins_size] or filter_bin>max_exp_tot[ins_size]:
		fitness=10**10
		continue

	    elif abs(max_exp_tot[ins_size]-min_exp_tot[ins_size])<=LARGE_INDEL_SIZE:
	        if cum_exp_dist_tot[ins_size][filter_bin]<0.05 or cum_exp_dist_tot[ins_size][filter_bin]>0.95:
	            fitness=10**10
	            continue
	    filter_bin=mean_list[ins_size]-second_mu-mu_moving
	    if filter_bin<min_exp_tot[ins_size] or filter_bin>max_exp_tot[ins_size]:
		fitness=10**10
		continue
	    elif abs(max_exp_tot[ins_size]-min_exp_tot[ins_size])<=LARGE_INDEL_SIZE:
	        if cum_exp_dist_tot[ins_size][filter_bin]<0.05 or cum_exp_dist_tot[ins_size][filter_bin]>0.95:
	            fitness=10**10
	            continue

	    smooth_ks={}
	    for sks_idx in range(SMOOTH_WINDOW):
	        smooth_ks[sks_idx]=0

	    sup_val[ins_size]=-1
	    #for bin in range(min_max_probdist_totkey[0],min_max_probdist_totkey[1]+1,1):
	    idx=0
	    for bin in probdist_totkey_alternate[ins_size]:
	    #for bin in range(min(prob_dist_tot_set[ins_size]),max(prob_dist_tot_set[ins_size])+1,1):
	    #for bin in range(min_max_probdist_totkey[0],min_max_probdist_totkey[1]+1,1):
	    #for bin in prob_dist_tot_set[ins_size]:
		cum_E=cum_prob_dist_tot[ins_size][bin]
		bin2=bin-mu_moving
		if (abs(max_exp_tot[ins_size]-min_exp_tot[ins_size])>LARGE_INDEL_SIZE) and  (bin2 not in cum_dist_keys):
		    tmp_pre=bin2
		    for tmp in cum_dist_keys:
                        if tmp>bin2:
                            bin2=tmp_pre
                            break
                        else:
                            tmp_pre=tmp
	        	
		if bin2<=max_exp_tot[ins_size] and bin2>=min_exp_tot[ins_size]:
		    cum_O=cum_exp_dist_tot[ins_size][bin2]
		elif bin2<min_exp_tot[ins_size]:
		    cum_O=0
		else:
		    cum_O=1

		smooth_ks[idx%SMOOTH_WINDOW]=abs(cum_O-cum_E)
		idx+=1
		if idx<SMOOTH_WINDOW:
		    ks=float((sum(smooth_ks.values()))/idx)
		else:
		    ks=float((sum(smooth_ks.values()))/SMOOTH_WINDOW)
		sup_val[ins_size]=max(sup_val[ins_size], ks)	
		#sup_val[ins_size]=max(sup_val[ins_size], abs(cum_O-cum_E))
		if sup_val[ins_size]>fitness_best:
		    fitness=10**10
		    break	
	    #fitness+=(weight_ins_sz[ins_size]/N)*sup_val[ins_size]
	    fitness+=sup_val[ins_size]
	    
	if fitness_best>fitness:
	    fitness_best=fitness
	    fixed_mu=mu_moving
    #print loglikeli_new, fitness_best
    #print "second opti!", loglikeli_new, fitness_best

    if loglikeli_new>fitness_best:
        if abs(mu[0])<abs(mu[1]):
            mu=[fixed_mu, second_mu+fixed_mu]
        else:
            mu=[second_mu+fixed_mu, fixed_mu]
        loglikeli_new=fitness_best

    return [loglikeli_new, mu, fixed_mu]
 
def first_M_step(ins_size_set,r,pi,x_list,mean_list_clus,mu_prev,clus_size_list,N,cum_prob_dist,probdist_keys_alternate, model_type):
    loglikeli_new=0
    mu=[0,0]
    ks_stats=[0,0]
    for member in [0,1]:
	##################################
	## make experimental distribution
	bin_list={}
	sup_val={}
	cum_exp_dist={}
	norm_factor=0
	max_bin={}
	min_bin={}
	bin_idx_keys={}	

	for ins_size in ins_size_set:
	    cum_exp_dist[ins_size]={}
	    bin_list[ins_size]=[]
	    sup_val[ins_size]=-1
	    norm_factor+=clus_size_list[ins_size][member]
	    max_bin[ins_size]=-1
	    min_bin[ins_size]=10**10
	    bin_idx_keys[ins_size]=[]

	if norm_factor==0:
	    continue
	mean_member=0
	for i in range(N):
	    sz=mean_list_clus[i]
	    if r[i][member]>0:
		bin=int(x_list[i])
		bin_list[sz].append([bin, r[i][member]])
		max_bin[sz]=max(max_bin[sz], bin)
		min_bin[sz]=min(min_bin[sz], bin)
		mean_member+=(sz-bin)
	if len(bin_list[sz])!=0:
	    mean_member/=len(bin_list[sz])

	for ins_size in ins_size_set:
	    if len(bin_list[ins_size])==0:
		continue
	    bin_list[ins_size].sort()
	    #print bin_list[ins_size]
	    #for ebin in range(min_bin[ins_size],max_bin[ins_size]+1,1):
            #    ind=0
            #    for bin_item in bin_list[ins_size]:
            #        bin=bin_item[0]
            #        respon=bin_item[1]
            #        if bin<=ebin:
            #            ind+=respon
            #        else:
            #            break
            #    cum_exp_dist[ins_size][ebin]=ind/clus_size_list[ins_size][member]
	    #print 'local ', max_bin[ins_size], min_bin[ins_size]
	    if abs(max_bin[ins_size]-min_bin[ins_size])>LARGE_INDEL_SIZE:
                for tmp in bin_list[ins_size]:
                    bin_idx_keys[ins_size].append(tmp[0])
                ind=0
                for bin_idx_list in bin_list[ins_size]:
                    bin=bin_idx_list[0]
                    ind+=bin_idx_list[1]
                    cum_exp_dist[ins_size][bin]=ind/clus_size_list[ins_size][member]

	    else:

                ind=0
                bin_idx=0
                for ebin in range(min_bin[ins_size],max_bin[ins_size]+1,1):
                    while(1):
                        if bin_idx>=len(bin_list[ins_size]):
                            break
                        bin_item=bin_list[ins_size][bin_idx]
                        if bin_item[0]>ebin:
                            break
                        ind+=bin_item[1]
                        bin_idx+=1
                    cum_exp_dist[ins_size][ebin]=ind/clus_size_list[ins_size][member]


	    ins_size_=ins_size

        #print 'cum_exp_dist : ', cum_exp_dist
	if (model_type%3== HOMOZYGOUS or model_type%3==HETEROZYGOUS) and member==1: # hetero
	    RANGE_SEARCH_=[0] 
	else:
	    RANGE_SEARCH_=[mu_prev[member],0]
            RANGE_SEARCH_+=range(mean_list[ins_size_]-max_bin[ins_size_],mean_list[ins_size_]-min_bin[ins_size_])
	    if len(RANGE_SEARCH_)>1000:
	        RANGE_SEARCH_=[mu_prev[member],0]
                RANGE_SEARCH_+=range(int(mean_member-50), int(mean_member+50)) ## TODO: std_dev change

        #print
        #print 'member: ', member
        #print 'SEARCH RANGE: ', RANGE_SEARCH_, max_bin, min_bin
        #print


        fitness_best=10**10
	for mu_moving in RANGE_SEARCH_:
	    fitness=0
	    for ins_size in ins_size_set:
		if len(bin_list[ins_size])==0:
		    continue

		###################################
		## filtering step
		filter_bin=mean_list[ins_size]-mu_moving
                #print 'filter bin', filter_bin
		if filter_bin<min_bin[ins_size] or filter_bin>max_bin[ins_size]:
		    fitness=10**10
		    continue
		else:
		    if (abs(max_bin[ins_size]-min_bin[ins_size])>LARGE_INDEL_SIZE) and (filter_bin not in bin_idx_keys[ins_size]):
                        #print 'starting detecting of key', bin2
			tmp_pre=filter_bin
                        for tmp in bin_idx_keys[ins_size]:
                            if tmp>filter_bin:
                                filter_bin=tmp_pre
                                break
                            else:
                                tmp_pre=tmp
		        if cum_exp_dist[ins_size][filter_bin]<0.05 or cum_exp_dist[ins_size][filter_bin]>0.95:
		            fitness=10**10
		            continue
		    else:
		        if cum_exp_dist[ins_size][filter_bin]<0.05 or cum_exp_dist[ins_size][filter_bin]>0.95:
		            fitness=10**10
		            continue
		
		sup_val[ins_size]=-1
		smooth_ks={}
		for sks_idx in range(SMOOTH_WINDOW):
		    smooth_ks[sks_idx]=0

		idx=0
		
                #print probdist_keys_alternate

                #####################################
                ## Base: experimental distribution

		for bin in probdist_keys_alternate[ins_size]:
		    cum_E=cum_prob_dist[ins_size][bin]
		    bin2=bin-mu_moving
                    #print 'bin2:', bin2

		    if (abs(max_bin[ins_size]-min_bin[ins_size])>LARGE_INDEL_SIZE) and (bin2 not in bin_idx_keys[ins_size]):
                        #print 'starting detecting of key', bin2
			tmp_pre=bin2
                        for tmp in bin_idx_keys[ins_size]:
                            if tmp>bin2:
                                bin2=tmp_pre
                                break
                            else:
                                tmp_pre=tmp


		    if bin2<=max_bin[ins_size] and bin2>=min_bin[ins_size]:
			cum_O=cum_exp_dist[ins_size][bin2]
		    elif bin2<min_bin[ins_size]:
			cum_O=0
		    else:
			cum_O=1
		    #if (mu_[member]<mu_[1-member] and bin<(sum(mu_)/2)) or (mu_[member]>mu_[1-member] and bin>(sum(mu_)/2)) or (mu_[member]==mu_[1-member]):
		    smooth_ks[idx%SMOOTH_WINDOW]=abs(cum_O-cum_E)
		    idx+=1
		    if idx<SMOOTH_WINDOW:
			ks=float((sum(smooth_ks.values()))/idx)
		    else:
			ks=float((sum(smooth_ks.values()))/SMOOTH_WINDOW)
		    sup_val[ins_size]=max(sup_val[ins_size], ks)	
		    if sup_val[ins_size]>fitness_best:
		        fitness=10**10
			break	
		    #if (member==0 and mu_moving==-6) or (member==1 and mu_moving==0):	
		    #    print mu_moving, bin, cum_E, cum_O, abs(cum_O-cum_E), sup_val[ins_size], mu
		#if norm_factor>0:
		    #fitness+=(clus_size_list[ins_size][member]/norm_factor)*sup_val[ins_size]
	        fitness+=sup_val[ins_size]

	    if fitness_best>fitness:
		fitness_best=fitness
		mu[member]=mu_moving
	    #print mu_moving, 'fitness:', fitness, 'fitness_best:', fitness_best

	for ins_size in ins_size_set:
	    loglikeli_new+=(pi[ins_size][member]*fitness_best)
            ks_stats[member]=fitness_best

	#print "first maxi:", loglikeli_new, member

    return [loglikeli_new, mu, ks_stats]

def Kolm_montecarlo(sample_size):
    MEAN_INSERT_SIZE = int(props["MEAN_INSERT_SIZE"])
    ALL_DATA_SOURCES_INSERT_SIZE=[MEAN_INSERT_SIZE]
    KOLM_DIST={}
    KOLM_set={}
    for i in ALL_DATA_SOURCES_INSERT_SIZE:
        KOLM_DIST[i]={}
        KOLM_set[i]=Set() 

    bin_set={}
    sup_val={}
    cum_exp_dist={}
    cum_prob_dist={}

    for ins_size in ALL_DATA_SOURCES_INSERT_SIZE:
        cum_exp_dist[ins_size]={}
        cum_prob_dist[ins_size]={}
        bin_set[ins_size]=[]
        sup_val[ins_size]=-1

    ## reject sampling 
    for ins_size in ALL_DATA_SOURCES_INSERT_SIZE:
        #max_den=max(probdist[ins_size].keys())
        max_den=max(probdist[ins_size].values())

        while(1):
            for bin in probdist[ins_size].keys():
                rannum=random.random()*max_den
                if rannum<probdist[ins_size][bin]:
                    bin_set[ins_size].append(bin)

            if len(bin_set[ins_size])>=sample_size:
                break

    for ins_size in ALL_DATA_SOURCES_INSERT_SIZE:
        probkeys=probdist[ins_size].keys()
        probkeys.sort()
        cum_E=0
        for ebin in probkeys:
            ind=0
            for bin in bin_set[ins_size]:
                if bin<=ebin:
                    ind+=1
            cum_exp_dist[ins_size][ebin]=ind/len(bin_set[ins_size])

            cum_E+=probdist[ins_size][ebin]
            cum_prob_dist[ins_size][ebin]=cum_E


    for ins_size in ALL_DATA_SOURCES_INSERT_SIZE:
        for bin in probdist[ins_size].keys():
            cum_E=cum_prob_dist[ins_size][bin]#*len(soft_cluster_dic[ins_size])
            cum_O=cum_exp_dist[ins_size][bin]
            sup_val[ins_size]=max(sup_val[ins_size], abs(cum_O-cum_E))
   
    for ins_size in ALL_DATA_SOURCES_INSERT_SIZE:
        sup_bin=int(sup_val[ins_size]/0.0001)*0.0001
        if sup_bin in KOLM_set[ins_size]:
            KOLM_DIST[ins_size][sup_bin]+=1
        else:
            KOLM_DIST[ins_size][sup_bin]=1
            KOLM_set[ins_size].add(sup_bin)

    #klkeys=cum_exp_dist[ins_size].keys()
    #klkeys.sort()
    #for i in klkeys:
    #    print "cum: ", i, cum_exp_dist[ins_size][i]
    
    ########################################
    ## find threshold 
    kolmo_thres={}
    for ins_size in ALL_DATA_SOURCES_INSERT_SIZE:
        kolmo_thres[ins_size]=0
    for ins_size in ALL_DATA_SOURCES_INSERT_SIZE:
        klkeys=KOLM_DIST[ins_size].keys()
        if len(klkeys)>0:
            klkeys.sort()
            norm=0
            for i in klkeys:
                norm+=KOLM_DIST[ins_size][i]

            tmp=0
            for i in klkeys:
                tmp+=(KOLM_DIST[ins_size][i]/norm)
                if tmp>=0.95:
                    kolmo_thres[ins_size]=i
                    break
    return kolmo_thres 

def montecarlo_run():
    ########################
    ## SIMULATION
    initialize_global_vars()
    MEAN_INSERT_SIZE = int(props["MEAN_INSERT_SIZE"])
    STD_INSERT_SIZE = int(props["STD_INSERT_SIZE"])
    EXE_DIR = props["EXE_DIR"]

    probdist[MEAN_INSERT_SIZE]={}
    FILE_l=open(EXE_DIR+"/data/PofY.txt","r")
    illu_line=FILE_l.readline()
    while illu_line:
        tmp=illu_line.split()
        probdist[MEAN_INSERT_SIZE][int(tmp[0])]=float(tmp[1])
        illu_line=FILE_l.readline() 

    FILE_l.close()
    mean_list[MEAN_INSERT_SIZE]=MEAN_INSERT_SIZE ## change if nessasary
    std_list[MEAN_INSERT_SIZE]=STD_INSERT_SIZE


    for ins_size in mean_list.keys():
        probdist_keys[ins_size]=probdist[ins_size].keys()
        probdist_keys[ins_size].sort()



    #############################################
    ## monte carlo, find threshold for rejection

    num_of_sampling=10
    range_sample_size=[20,401]
    src_dir=props["INPUT_DATA_DIR"]
    illumina_usage=3
    ALL_DATA_SOURCES_INSERT_SIZE=[MEAN_INSERT_SIZE]
    if illumina_usage==1:
        file_prefix="Illumina_"
    elif illumina_usage==2:
        file_prefix="Simul_"
    else:
        file_prefix=""

    fname_monte=EXE_DIR+"/data/"+file_prefix+"montecarlo_20_400.threshold"
    FILE_monte=open(fname_monte,"w")
    kolms={} 
    for sample_sz in range(range_sample_size[0], range_sample_size[1], 1):
        kolms[sample_sz]={}
        for ins_size in ALL_DATA_SOURCES_INSERT_SIZE:
            kolms[sample_sz][ins_size]=0
    print "... start monte carlo ...", num_of_sampling, range_sample_size
    for i in range(num_of_sampling): 
        print 
        for sample_sz in range(range_sample_size[0], range_sample_size[1], 1):
            tmp=Kolm_montecarlo(sample_sz)
            for ins_size in ALL_DATA_SOURCES_INSERT_SIZE:
                kolms[sample_sz][ins_size]+=tmp[ins_size]/num_of_sampling
                #print "monte:", sample_sz, tmp[ins_size]
            print "---- progress ---- ", str(i+1)+"-th sample out of "+str(num_of_sampling), ", sample size:", sample_sz

    for sz in kolms.keys():
        for ins_size in ALL_DATA_SOURCES_INSERT_SIZE:
            FILE_monte.write(str(sz)+'\t'+str(ins_size)+'\t'+str(kolms[sz][ins_size])+'\n')

    FILE_monte.close()


#######################################################
## It runs on each chromosome
## Make sure you filter out repeats using repeat masker

def soft_clustering_indel(chr, step, step_size):
    print_message_and_current_time("Begin time: ")
    
    EXE_DIR=props["EXE_DIR"]
    RESULTS_DIR=props["RESULTS_DIR"]
    INPUT_DATA_DIR=props["INPUT_DATA_DIR"]
    MAX_DEL_SIZE=int(props["MAX_DEL_SIZE"])
    MINIMUM_NUM_OF_MATEPAIRS = int(props["MINIMUM_NUM_OF_MATEPAIRS"])

    CHI_BIN=1

    MEAN_INSERT_SIZE=int(props["MEAN_INSERT_SIZE"])
    STD_INSERT_SIZE=int(props["STD_INSERT_SIZE"])
    
    #file_prefix="mod_"
    cluster_list=[]
    ##################################3
    ## load data
    print "... START LOAD DATA ... ", chr

    dbname="%s/%s" % (INPUT_DATA_DIR,chr)
    db = open(dbname, "r")
    
    curr_clusidx=0
    line=db.readline()
    print 'from', (step*step_size)-MEAN_INSERT_SIZE, 'to', (step+1)*step_size+MEAN_INSERT_SIZE
    ## already sorted
    minmax_start_pos=[10**10,-1]
    
    while line:
        if curr_clusidx>=(step*step_size)-MEAN_INSERT_SIZE and curr_clusidx<(step+1)*step_size+MEAN_INSERT_SIZE:
            tmp=line.split()
            start_pos=int(tmp[0])
            end_pos=int(tmp[1])
            #print start_pos
            minmax_start_pos[0]=min(minmax_start_pos[0],start_pos)
            minmax_start_pos[1]=max(minmax_start_pos[1],start_pos)
            map_id=tmp[2]
            ndist=float(tmp[3])
            insert_size=MEAN_INSERT_SIZE
            template_id=tmp[5]
            if abs(ndist)<MAX_DEL_SIZE:
                cluster_list.append([start_pos, end_pos, map_id, ndist, insert_size, template_id])
        curr_clusidx+=1
        line=db.readline()
    db.close()
    print "min & max starting position:", minmax_start_pos

    #########################################################################################
    ## CLUSTERING
    print "... start creating clusters ..."

    fname_indel=RESULTS_DIR+"/CLUSTERING_RESULTS/"+chr+"/"+file_prefix+"clus_"+chr+".candidates."+str(step)+'.gz'
    FILE_indel = gzip.open(fname_indel,"wb")
    fname_log=EXE_DIR+"/log/"+file_prefix+chr+"."+str(step)
    FILE_log = open(fname_log,"w")

    KOR_DIST={}
    KOR_set={}
    cum_exp_dist={}
    cum_prob_dist={}
    prob_keys={}

    KOR_DIST[MEAN_INSERT_SIZE]={}
    KOR_set[MEAN_INSERT_SIZE]=Set() 
    cum_exp_dist[MEAN_INSERT_SIZE]={}
    cum_prob_dist[MEAN_INSERT_SIZE]={}
    prob_keys[MEAN_INSERT_SIZE]={}

    sample_num=0
    prog=-10**10
    reject_num=0
    accept_num=0
    levycnt=0


    prob_keys[MEAN_INSERT_SIZE]=probdist[MEAN_INSERT_SIZE].keys()
    prob_keys[MEAN_INSERT_SIZE].sort()
    cum_E=0
    for ebin in prob_keys[MEAN_INSERT_SIZE]:
        cum_E+=probdist[MEAN_INSERT_SIZE][ebin]
        cum_prob_dist[MEAN_INSERT_SIZE][ebin]=cum_E

    progress=0
    #step_size=int(len(cluster_list)/101.0)

    #print "step_size", step_size
    prev_mp_list=[]
    cluster_id=step_size*step

    for curr_clusidx in range(len(cluster_list)):
        #if curr_clusidx>=step*step_size and curr_clusidx<(step+1)*step_size:
            ## step -1: the rest
        #    stay=1
        #else:
        #    continue         
        progress+=1

        if progress % 4000 == 0:
            print "progress: ", progress, step_size, progress/step_size, chr
            print "reject: ", reject_num, "accept: ", accept_num
            print

        ## find all matepairs overlapping with the current one
        curr_cluster=cluster_list[curr_clusidx]

	#loc_range_list=[]
	#for cr in range(curr_cluster[0]+1, curr_cluster[1]-11, 10):
        #    loc_range_list.append(cr)
        #loc_range_list=[curr_cluster[0]+1, curr_cluster[1]-1]
        loc_range_list=[curr_cluster[0]+1]

        ####################################################################
        ## use previously found matepairs after certain number of iterations
        chk_reg=[19219522,19219536]
        if max(curr_cluster[0], chk_reg[0])<=min(curr_cluster[1], chk_reg[1]):
            FILE_log.write(str(curr_cluster[0])+'\t'+str(curr_cluster[1])+'\n')
	#print loc_range_list
            
        if progress>100:
            #print prev_mp_list
	    for loc_range in loc_range_list: 
                neighbor_mp_list=copy.copy(prev_mp_list)
                for prev_mp in prev_mp_list:
                    if cluster_list[prev_mp[1]][1]<loc_range or cluster_list[prev_mp[1]][0]>loc_range:
                        neighbor_mp_list.remove(prev_mp)
			#print 'DELETE', loc_range, cluster_list[prev_mp[1]][0], cluster_list[prev_mp[1]][1]
                #elif cluster_list[prev_mp[1]][0]>loc_range:
                #    break

                if neighbor_mp_list==[]:
                    neighbor_mp_list=[[0, curr_clusidx]]
           
                #######################################################
                ## find all overlapping matepairs: after previous set
                neighbor_clusidx=neighbor_mp_list[len(neighbor_mp_list)-1][1]
		#print 'same? ', loc_range, cluster_list[neighbor_clusidx][0]+1
                while(1):
                    neighbor_clusidx+=1
                    if neighbor_clusidx>=len(cluster_list):
                        break 
                    else: 
                        neighbor_cluster=cluster_list[neighbor_clusidx] 
                
	            #if loc_range==202264175+1:
	            #    print '1 ------------->', loc_range, neighbor_cluster[0], neighbor_cluster[1], cluster_list[neighbor_mp_list[1][1]][0], cluster_list[neighbor_mp_list[1][1]][1]
                    if (loc_range<neighbor_cluster[0] or loc_range>neighbor_cluster[1]) and neighbor_clusidx>=curr_clusidx: ## no overlapping
                        break
		    elif max(loc_range,neighbor_cluster[0])<=min(loc_range,neighbor_cluster[1]): # overlapping
                        neighbor_mp_list.append([0, neighbor_clusidx])
                #elif abs(curr_cluster[4]-neighbor_cluster[4])<4*resolution:
                #neighbor_mp_list.append([abs(neighbor_cluster[3]-curr_cluster[3]), neighbor_clusidx])


	        neighbor_mp_list.sort()
                if max(curr_cluster[0], chk_reg[0])<=min(curr_cluster[1], chk_reg[1]):
                    FILE_log.write("LENGTH: "+str(len(neighbor_mp_list))+"\n")

                if prev_mp_list!=neighbor_mp_list and len(neighbor_mp_list)>MINIMUM_NUM_OF_MATEPAIRS: # exactly the same
		    prev_cluster_id=cluster_id
	            cluster_id=create_a_cluster(neighbor_mp_list, FILE_indel, loc_range, cluster_id, cluster_list, CHI_BIN, chr, curr_cluster[2])
		    if prev_cluster_id==cluster_id:
			accept_num+=1
                        if max(curr_cluster[0], chk_reg[0])<=min(curr_cluster[1], chk_reg[1]):
                            FILE_log.write("accept\n")
		    else:
			reject_num+=1
                        if max(curr_cluster[0], chk_reg[0])<=min(curr_cluster[1], chk_reg[1]):
                            FILE_log.write("reject\n")
		prev_mp_list=copy.copy(neighbor_mp_list)
                
        else:
	    for loc_range in loc_range_list: 
                neighbor_mp_list=[[0, curr_clusidx]]
                #############################################
                ## find all overlapping matepairs: forwards
                neighbor_clusidx=curr_clusidx
                while(1):
                    neighbor_clusidx-=1
                    if neighbor_clusidx<0:
                        break 
                    else:
                        neighbor_cluster=cluster_list[neighbor_clusidx] 

                    if loc_range>neighbor_cluster[1] or loc_range<neighbor_cluster[0]: ## no overlapping
		        continue

                    #neighbor_mp_list.append([abs(neighbor_cluster[3]-curr_cluster[3]), neighbor_clusidx])
                    neighbor_mp_list.append([0, neighbor_clusidx])

                #############################################
                ## find all overlapping matepairs: backwards
                neighbor_clusidx=curr_clusidx
                while(1):
                    neighbor_clusidx=neighbor_clusidx+1
                    if neighbor_clusidx>=len(cluster_list):
                        break 
                    else: 
                        neighbor_cluster=cluster_list[neighbor_clusidx] 
                
                    if loc_range<neighbor_cluster[0] or loc_range>neighbor_cluster[1]: ## no overlapping
                        break

                    neighbor_mp_list.append([0, neighbor_clusidx])

                if max(curr_cluster[0], chk_reg[0])<=min(curr_cluster[1], chk_reg[1]):
                    FILE_log.write("LENGTH: "+str(len(neighbor_mp_list))+"\n")

	        neighbor_mp_list.sort()

	        #if loc_range_list[0]==202264175+1:
	        #    print '2 ------------->', neighbor_mp_list

	        #if max(loc_range, 202264175) <=min(loc_range, 202264252):
	    	#    print 
		#    print '1. found ', neighbor_mp_list
		#    print 'loc_range ', loc_range
		#    print 

                if prev_mp_list!=neighbor_mp_list and len(neighbor_mp_list)>MINIMUM_NUM_OF_MATEPAIRS: # exactly the same
		    prev_cluster_id=cluster_id
	            cluster_id=create_a_cluster(neighbor_mp_list, FILE_indel, loc_range, cluster_id, cluster_list, CHI_BIN, chr, curr_cluster[2])
		    if prev_cluster_id==cluster_id:
			accept_num+=1
                        if max(curr_cluster[0], chk_reg[0])<=min(curr_cluster[1], chk_reg[1]):
                            FILE_log.write("accept\n")
		    else:
			reject_num+=1
                        if max(curr_cluster[0], chk_reg[0])<=min(curr_cluster[1], chk_reg[1]):
                            FILE_log.write("reject\n")
	        prev_mp_list=copy.copy(neighbor_mp_list)
		         


    print "accept:", accept_num, "reject:", reject_num
    FILE_indel.close()
    FILE_log.close()

def create_a_cluster(neighbor_mp_list, FILE_indel, loc_range, cluster_id, cluster_list, CHI_BIN, chrom, rep_map):
    ins_size_set=Set()
    soft_cluster_dic={}

    mapped_ndist_list=[]
    clusidx_list=[]
    for mp in neighbor_mp_list:
        clus_=cluster_list[mp[1]]
        #if loc_range>clus_[0] and loc_range<clus_[1]:
        if clus_[4] in ins_size_set:
            soft_cluster_dic[clus_[4]].append(clus_[2])
        else:
            soft_cluster_dic[clus_[4]]=[clus_[2]]
            ins_size_set.add(clus_[4])
        mapped_ndist_list.append([clus_[3], clus_[4]]) 
        clusidx_list.append(mp[1])
        
    cardi=len(mapped_ndist_list)
    #for ins_size in ins_size_set:
    #    cardi+=len(soft_cluster_dic[ins_size])

    #if cardi<MINIMUM_NUM_OF_MATEPAIRS:
#	return 0

    mu_dist=0
    r_dist={}
    r_idx=0
    d_std=[]
    for mdl in mapped_ndist_list:
        bin=int(mdl[0]/CHI_BIN)*CHI_BIN
        #bin_set[mdl[1]].append(bin)
        mu_dist+=bin
        if r_idx<(cardi/2):	# take half of matepairs
            r_dist[r_idx]=[1,0]
	    r_idx+=1
	    d_std.append(std_list[mdl[1]])

    mu_dist=(mu_dist/cardi)*2
    pval=get_pvalue([mu_dist,0], d_std, [len(d_std),0])
    
    EM_GOTO=1

    if pval[0]>=PVAL_THRESHOLD: ## filtering
        #accept_num+=1
        EM_GOTO=0

    #if max(loc_range, 202264175) <=min(loc_range, 202264252):
#	print 
#	print '2. found == pvalue', pval, EM_GOTO
#	print '2. cardi', cardi 
#	print 
	
    if EM_GOTO>=1:
        #reject_num+=1
        for j in range(cardi):
            ctmp=cluster_list[clusidx_list[j]]
            ## clus_id,  map_id, chrom, start_pos, end_pos, ndist, insert_size, template_id, base_location, base_map_id
            string=str(cluster_id)+TAB+str(ctmp[2])+TAB+chrom+TAB+str(ctmp[0])+TAB
            string+=str(ctmp[1])+TAB+str(ctmp[3])+TAB+str(ctmp[4])+TAB+str(ctmp[5])+TAB
            string+=str(loc_range)+TAB+str(rep_map)+"\n"
            FILE_indel.write(string)

            #FILE_dist.write(str(ctmp[1]-ctmp[0])+'\t'+str(ctmp[0])+':'+str(ctmp[1])+'\t'+str(r_new[j])+'\n')
        cluster_id+=1
            #FILE_dist.write('\n-------------------------- '+str(pval)+':'+str(pval_h)+":"+str(mu_new)+'\n')
    #else:
        #accept_num+=1
    #print "TOTAL number of clusters:", cluster_id, "reject:", reject_num, "accept:", accept_num, "f_step", step

    return cluster_id



class Searcher_repeat:
    def __init__(self, filename):
        self.f = open(filename, 'rb')
        self.f.seek(0,2)
        self.length = self.f.tell()

    def find(self, chr, start, end):
        low = 0
        high = self.length
        while low < high:
            mid = (low+high)/2
            p = mid
            while p >= 0:
                #print "seek", p
                self.f.seek(p)
                test=self.f.read(1)
                #print "mmm", test, p
                if (test == '\n'):
                    #self.f.seek(0)
                    break
                elif p==0:
                    self.f.seek(0)
                    break
                p -= 1
            line = self.f.readline()
            tmp=line.split()
            if tmp==[]:
                low = mid+1
                continue
            chr_=tmp[1]
            start_=int(tmp[2])
            if chr_<chr:
                low = mid+1
            elif chr_>chr:
                high = mid
            elif start_<start:
                low = mid+1
            else:
                high = mid
        p = low
        while p > 0:
            self.f.seek(p)
            if self.f.read(1) == '\n':
                break
            p -= 1
        if p>0: p-=1
        while p > 0:
            self.f.seek(p)
            if self.f.read(1) == '\n':
                break
            p -= 1

	result = []
        while True:
            line = self.f.readline()
            tmp=line.split()
            if tmp==[]:
                break
            chr_=tmp[1]
            start_=int(tmp[2])
            end_=int(tmp[3])
            span_o=min(end_,end)-max(start_,start)
            if span_o>0 and (end-start)-span_o<15:
		return 1
            if not line or chr_!=chr or start_>end:
                break

            result.append(chr_+"-"+tmp[2]+":"+tmp[3])
        return 0
    
#------------------------------------------
# find the correct location in the genome
# 2012.1.16 this version is mainly for speed up!

def inference(chr, f_in):
    initialize_global_vars()
    CHI_BIN=1
    src_dir=props["RESULTS_DIR"]
    EXE_DIR=props["EXE_DIR"]
    MEAN_INSERT_SIZE = int(props["MEAN_INSERT_SIZE"])
    STD_INSERT_SIZE = int(props["STD_INSERT_SIZE"])
    MINIMUM_NUM_OF_MATEPAIRS = int(props["MINIMUM_NUM_OF_MATEPAIRS"])

    monte_thres_file=EXE_DIR+"/data/montecarlo_20_400.threshold"

    ################################################
    # this is for filtering false positives
    #FILE_monte_thres=open(monte_thres_file, "r")
    #monte_thres_dic={}
    #monte_thres_line=FILE_monte_thres.readline()
    #while monte_thres_line:
    #    tmp=monte_thres_line.split()
    #    clus_size=int(tmp[0])
    #    monte_thres_dic[clus_size]=float(tmp[2])
    #    monte_thres_line=FILE_monte_thres.readline()
    #FILE_monte_thres.close()

    ########################
    ## make distribution
    probdist[MEAN_INSERT_SIZE]={}
    FILE_l=open(EXE_DIR+"/data/PofY.txt","r")
    illu_line=FILE_l.readline()
    while illu_line:
        tmp=illu_line.split()
        probdist[MEAN_INSERT_SIZE][int(tmp[0])]=float(tmp[1])
        illu_line=FILE_l.readline()

    #probdist[208]={}
    #probdist[208][208]=1
    FILE_l.close()
    mean_list[MEAN_INSERT_SIZE]=MEAN_INSERT_SIZE ## change if nessasary
    std_list[MEAN_INSERT_SIZE]=STD_INSERT_SIZE


    for ins_size in mean_list.keys():
        probdist_keys[ins_size]=probdist[ins_size].keys()
        probdist_keys[ins_size].sort()

    print "!!!!!!!!!!!!!!! INFERENCE START !!!!!!!!!!!!!!!!!!!!!!!!!"

    ###################################
    ## put template_id in Set
    print "!!!DATA READ START!!!"

    ##################
    ## READ DATA
    f_in_name=src_dir+"/CLUSTERING_RESULTS/"+chr+"/"+file_prefix+"clus_"+chr+".candidates."+str(f_in)+'.gz'
    file_p = gzip.open(f_in_name, "rb")

    cluster_list={}
    matepair_list={}
    map_id_list={}
    template_id_set=Set()
    clusid_set=Set()
    tempid_set=Set()
    map_id_set=Set()
    map_id_mapped_dist={}
    line=file_p.readline()
    linecnt=0
    while line:
        if linecnt % 10000 == 100:
            print "read progress", linecnt
        linecnt+=1

        tmp=line.split()
        clus_id=int(tmp[0])
        map_id=tmp[1]
        chr_=tmp[2]
        start_pos=int(tmp[3])
        end_pos=int(tmp[4])
        ndist=float(tmp[5])
        insert_size=int(tmp[6])
        template_id=tmp[7]

	loc_range=tmp[8]


	#ret=search_rep.find(chr_, start_pos, end_pos)
        #if ret>0:
        #    line=file_p.readline()
        #    continue
        ###########################
        ## cluster -> matepair
        if clus_id in clusid_set:
            cluster_list[clus_id].append([map_id,start_pos,end_pos,ndist,insert_size,template_id,loc_range])
        else:
            cluster_list[clus_id]=[[map_id,start_pos,end_pos,ndist,insert_size,template_id,loc_range]]
            clusid_set.add(clus_id)

        #####################################
        ## include matepairs not in this list
        if template_id in tempid_set:
            matepair_list[template_id].add(map_id)
        else:
            matepair_list[template_id]=Set([map_id])
            tempid_set.add(template_id)
        map_id_mapped_dist[map_id]=[ndist, insert_size]

        ##########################
        ## map_id -> cluster_id
        if map_id in map_id_set:
            map_id_list[map_id].add(clus_id)
        else:
            map_id_list[map_id]=Set([clus_id])
            map_id_set.add(map_id)

        template_id_set.add(template_id)
        #print clus_id,map_id,start_pos,end_pos,ndist,insert_size, template_id
        line=file_p.readline()
    file_p.close()

    tot_clusid_num=len(clusid_set)
    ####### free memory
    tempid_set.clear()
    map_id_set.clear()
    clusid_set.clear()

    print "!!!DATA READ END!!!"
    print "!!!INFERENCE START!!!"
    #########################################################
    ## INFERENCE, ITERATED CONDITIONAL MODES
    iter=0
    template_id_to_map_id={}

    no_use_matepairs=Set()
    INFER_BIN=5
    while(1):
        #determined_map_id=Set()
        if iter>=1:
            break
        progress=0
        #################################
        ## per matepair
        for template_id in template_id_set:
            if progress % 1000 == 0:
                print "progress:", progress, len(template_id_set), progress/len(template_id_set)
                print 
            progress+=1

            ##############################
            ## uniquely mapped matepair
            if len(matepair_list[template_id])==1:
                for map_id in matepair_list[template_id]:
                    template_id_to_map_id[template_id]=map_id
	        continue

            ###################################################
            ## find all clusters associated with this matepair: 
            ## templated_id -> map_id -> clus_id -> all map_ids associated w/ clusid
            chi_list=[]
            for map_id in matepair_list[template_id]:
                best=10**30
                exist_set=Set()
                exp_dist={}
                parti_fn=0

                exist_set.clear()
                for clus_id in map_id_list[map_id]: # all clusters associated w/ this map_id
                    #####################################################
                    ## make experimental distribution with this cluster
                    for i in range(len(cluster_list[clus_id])):
                        if cluster_list[clus_id][i][0]==map_id: # exclude the current map_id
                            continue
                        ins_size=cluster_list[clus_id][i][4]
                        if ins_size!=map_id_mapped_dist[map_id][1]:
                            continue
                        mapped_d=mean_list[ins_size]-cluster_list[clus_id][i][3]
                        bin=int(mapped_d/INFER_BIN)*INFER_BIN
                        if bin in exist_set:
                            exp_dist[bin]+=1
                        else:
                            exp_dist[bin]=1
                            exist_set.add(bin)
                        parti_fn+=1
		    ####################################################333
		    ## bug fix: 2009/04/07
                    bi=int((mean_list[ins_size]-map_id_mapped_dist[map_id][0])/INFER_BIN)*INFER_BIN
                    if bi in exist_set:
                        den=math.log(exp_dist[bi]/parti_fn)
                    else:
                        den=-1000*abs(mean_list[ins_size]-bi)
                    
                    best=min(best,-den)
                chi_list.append([best, map_id])

            ## choose best map_id
            chi_list.sort()
            template_id_to_map_id[template_id]=chi_list[0][1]
        
        iter+=1

    #################
    ## free memory
    template_id_set.clear()
    map_id_list=[]
    matepair_list=[]

    ###########################################
    ## FINAL: inference w/ chosen matepairs
    print "!!!EM start and print results!!!"

    r_f=f_in_name.split("/")
    r_f_n=r_f[len(r_f)-1]
    fname_res=src_dir+"/INFERENCE_RESULTS/"+chr+"/inference_"+r_f_n
    FILE_res = gzip.open(fname_res,"wb")

    chk_cnta=0
    chk_cntb=0
    progress=0

    precision=0.01
    for clus_id in cluster_list.keys():
        if progress % 100 == 0:
            print "FINAL PROGRESS: ", progress, tot_clusid_num
        progress+=1

        m_list=[]
        size_list=[]
        avail_list=[]
        for clus_ in cluster_list[clus_id]:
            if template_id_to_map_id[clus_[5]]==clus_[0]: ## map_id
                m_list.append(mean_list[clus_[4]]-float(clus_[3]))
                size_list.append(clus_[4])
                avail_list.append(clus_)

                if len(m_list)>MAXIMUM_NUM_OF_MATEPAIRS:
                    break
            
        if len(m_list)< MINIMUM_NUM_OF_MATEPAIRS:
            continue

        #<--------------------EM-------------------->#
        #[mu_new, sigma_new, r_new, score, iter, pval_chi, r_num, ks_stats, pi_new]=EM_mog(0, m_list, [0,0], [300,300], {}, size_list, CHI_BIN)
        # indelsize, dummy, number of mp in each dist, likeli, membership
        [mu_new, hetero_flag,r_num,ks_stats,r_new,r_std]=fast_EM_mog(m_list, size_list, CHI_BIN)
        if sum(r_num) == 0:
            continue
        else:
            pi_new = [r_num[0]/sum(r_num), r_num[1]/sum(r_num)]

        #print mu_new, r_num
        #print "*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-", mu_new, score

        ##############################
        ## no results after EM
        #if len(r_new)==0: 
        #    continue

        c_std_list=[]
        for sz in size_list:
            c_std_list.append(std_list[sz])

        # actually final p-value and p-het are re-computed in post-processinng step
        
        #pval=get_pvalue(mu_new, c_std_list, r_num) 
        #pval=get_pvalue2(mu_new, r_num) 
        #print mu_new, r_num, r_std


        ###########################################################
        ## filtering with ks-stat

        if FILTER_KS_STAT_ON == 1:
            for mem in [0,1]:
                if int(r_num[mem])<=400 and int(r_num[mem])>=20:
                    if monte_thres_dic[int(r_num[mem])]<ks_stats[mem]:
                        pval[mem]=-1
                else:
                    if 0.2<ks_stats[mem]:
                        pval[mem]=-1

        score=abs(0.5-min(r_num)/sum(r_num))


        ####################################################
        ## indel start & end position

	region_sorted=[]
	region_sorted2=[]
        sv_pos_pval=[-1,10**10]
        sv_pos_pval2=[-1,10**10]

        for ctmp in avail_list:
            resp=responsibilities_of_matepair(mean_list[ctmp[4]]-float(ctmp[3]), mu_new, ctmp[4],pi_new)
	    region_sorted.append([-resp[0], ctmp])
	    region_sorted2.append([-resp[1], ctmp])
	region_sorted.sort()
	region_sorted2.sort()

	for rs in region_sorted:
            tmp1=max(sv_pos_pval[0], rs[1][1])
            tmp2=min(sv_pos_pval[1], rs[1][2])
	    if mu_new[0]<0 and abs(tmp2-tmp1)<abs(mu_new[0]):
		continue
            if abs(rs[0])>RESPONSIBILITY_CUT_OFF: #0.9
                #print '0 : ', rs[1][3], rs[1][1:3]
                sv_pos_pval[0]=tmp1 #max(sv_pos_pval[0], rs[1][1])
                sv_pos_pval[1]=tmp2 #min(sv_pos_pval[1], rs[1][2])
	    else:
		break

        num_avail_mp = 0
	for rs in region_sorted2:
            tmp1=max(sv_pos_pval2[0], rs[1][1])
            tmp2=min(sv_pos_pval2[1], rs[1][2])
	    if mu_new[1]<0 and abs(tmp2-tmp1)<abs(mu_new[1]):
		continue
            if abs(rs[0])>RESPONSIBILITY_CUT_OFF: #0.9
                #print '1 : ', rs[1][3], rs[1][1:3]
                sv_pos_pval2[0]=tmp1 #max(sv_pos_pval2[0], rs[1][1])
                sv_pos_pval2[1]=tmp2 #min(sv_pos_pval2[1], rs[1][2])
                num_avail_mp+=1
	    else:
	  	break

        # determine the heterogygosity
        # 1. select all matepairs having indel regision       
        # 2. compute p that these matepairs occured by chance 
        concordant_mp = []
        j=0
        for ctmp in avail_list:
            resp=responsibilities_of_matepair(mean_list[ctmp[4]]-float(ctmp[3]), mu_new, ctmp[4],pi_new)

            #print sv_pos_pval2[1], ctmp[2], ctmp
            #print min(sv_pos_pval2[1],ctmp[2])
            #print max(sv_pos_pval2[0],ctmp[1])
            overlap  = min(sv_pos_pval2[1],ctmp[2])-max(sv_pos_pval2[0],ctmp[1])
            if mu_new[1]>0:
                overlap_c = 0
            else:
                overlap_c = abs(mu_new[1])
            if overlap>=overlap_c and r_new[j][1] == 0 and abs(float(ctmp[3])) < 2*STD_INSERT_SIZE and resp[0]>0.9:
                concordant_mp.append(float(ctmp[3]))
            j+=1

        ######################################################################
        pval=get_pvalue_i(mu_new, [r_num[0], num_avail_mp], r_std) 
        # if you want to check the results regardless of p-values, comment it
        if (pval[0]>10**(-15) and pval[1]>10**(-15)) or (pval[1]<0):
            continue

        #pval_h=get_phat_i(mu_new, c_std_list, r_num)
        pval_h = get_phat_i(mu_new[1], concordant_mp)
        pval_h = math.exp(pval_h)

        #print concordant_mp, len(concordant_mp), pval_h
        #print mu_new, pval, pval_h, sv_pos_pval2
 
        ##############################################################
        ## PRINT RESULTS

        if sv_pos_pval2[0]<0:
            continue
        j=0
        for ctmp in avail_list:
            string=str(clus_id)+TAB
            string+=str(ctmp[0])+TAB
            string+=chr+TAB
            string+=str(ctmp[1])+TAB
            string+=str(ctmp[2])+TAB
            string+=str(ctmp[3])+TAB
            string+=str(ctmp[5])+TAB

            # location of the indel
            if r_new[j]==[1,0]:
                string+=str(sv_pos_pval[0])+TAB+str(sv_pos_pval[1])+TAB
            else:
                string+=str(sv_pos_pval2[0])+TAB+str(sv_pos_pval2[1])+TAB

            string+=str(r_new[j][1])+TAB  # indicate where the maptepair belongs to 
            string+=str(pval[0])+TAB
            string+=str(pval[1])+TAB
            string+=str(mu_new[0])+TAB
            string+=str(mu_new[1])+TAB
            string+=str(pval_h)+TAB
            string+=str(r_num[0]) + TAB # supporting no indel
            string+=str(r_num[1]) + "\n" # supporting indel
            FILE_res.write(string)
            j+=1

    FILE_res.close()
    print_message_and_current_time("End time: ")



def test_mog():
    clon_coverage=100
    CHI_BIN=1

    real_f=0
    if real_f==1:
        fname_dist="/home/brudno/structural_variations/file_repository/INDEL_RESULTS/sample/chr19/"
        fname_dist+="file"
        
        FILE_dist = open(fname_dist,"r")
        #################################
        # put template_id in Set
        line=FILE_dist.readline()
        m_list=[]
        size_list=[]
        while line:
            tmp=line.split()
            #print tmp, mean_list[600]-float(tmp[0])
            m_list.append(mean_list[int(tmp[1])]-float(tmp[0]))
            size_list.append(int(tmp[1]))
            line=FILE_dist.readline()
        FILE_dist.close()

    if real_f==0:
        m_list=[]
        size_clus=clon_coverage*10
        size_list=[]
        ins_size_list=[208]
        ins_size_list=[208]
        indel=200
        indel2=0
        cardi=0
        for ins_size in ins_size_list:
            tot_size=size_clus/len(ins_size_list)
            prob_keys=probdist[ins_size].keys()
            prob_keys.sort() 
	    alternate=0
            for i in prob_keys:
                clusnum=int(round(probdist[ins_size][i]*tot_size))
                cardi+=clusnum
		clusnum_a=0
		clusnum_b=0
		
                for j in range(clusnum):
                    #x=random.randrange(i, i+CHI_BIN)
                    x=i
                    #if j<clusnum*0.5:
		    #if alternate==0:
		    if random.random()<0.5:
                        m_list.append(x+indel)
			clusnum_a+=1
			alternate=1
	            else: #if alternate==1:
                        m_list.append(x+indel2)
		        clusnum_b+=1
	  	        alternate=0
		    #elif clusnum_a<=clusnum_b:
                    #    m_list.append(x+indel)
		#	clusnum_a+=1
                    #else:
                #        m_list.append(x)
		#	clusnum_b+=1
                    size_list.append(ins_size)
		#print i, clusnum_a, clusnum_b, clusnum

    size_list_input=random.sample(size_list,clon_coverage)
    m_list_input=random.sample(m_list,clon_coverage)

    print m_list_input
    [mu_new, sigma_new, r_new, score, iter, pval_chi, r_num, null, pi_new]=EM_mog(0, m_list_input, [0,0], [300,300], {}, size_list_input, CHI_BIN)
    print "*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-", mu_new, score, null
    print
    ##############################
    ## check p-value
    #r_num=[0,0]
    #for i in range(cardi):
    #    if r_new[i]==[1,0]:
    #        r_num[0]=r_num[0]+1
    #    else:
    #        r_num[1]=r_num[1]+1
    c_std_list=[]
    for sz in size_list:
        c_std_list.append(std_list[sz])

    #pval=get_pvalue(mu_new, c_std_list, r_num)
    pval=get_pvalue2(mu_new, r_num)
    pval_h=get_phat(mu_new, c_std_list, r_num)
    print "pval:", pval, "phat:", pval_h, "r_num", r_num, "pi_new", pi_new, "mu_new", mu_new
    #for cnt in range(len(m_list)):
    #    print r_new[cnt], m_list[cnt]
#	cnt+=1
    #print "responsibiligy: ", responsibilities_of_matepair(250, mu_new, 208, pi_new)   

    '''
    sv_pos_pval=[-1,10**10]
    sv_pos_pval2=[-1,10**10]
    region_sorted=[]
    region_sorted2=[]
    for ctmp in avail_list:
        resp=responsibilities_of_matepair(mean_list[ctmp[4]]-float(ctmp[3]), mu_new, ctmp[4],pi_new)
        region_sorted.append([-resp[0], ctmp])
        region_sorted2.append([-resp[1], ctmp])
    region_sorted.sort()
    region_sorted2.sort()

    for rs in region_sorted:
        if abs(sv_pos_pval[1]-sv_pos_pval[0])<mu_new[0]:
	    continue
        if abs(rs[0])>0.8:
    	    sv_pos_pval[0]=max(sv_pos_pval[0], rs[1][1])
	    sv_pos_pval[1]=min(sv_pos_pval[1], rs[1][2])
    for rs in region_sorted2:
        if abs(sv_pos_pval2[1]-sv_pos_pval2[0])<mu_new[1]:
	    continue
        if abs(rs[1])>0.8:
	    sv_pos_pval2[0]=max(sv_pos_pval2[0], rs[1][1])
	    sv_pos_pval2[1]=min(sv_pos_pval2[1], rs[1][2])
    ''' 

def responsibilities_of_matepair(x, mu, ins_size, pi_new):

    CHI_BIN = 1
    bin1=int((x+mu[0])/CHI_BIN)*CHI_BIN
    bin2=int((x+mu[1])/CHI_BIN)*CHI_BIN
    r=[]

    if bin1 in probdist_keys[ins_size]:
        g1 = probdist[ins_size][bin1]
    else:
        g1 = 0
    if bin2 in probdist_keys[ins_size]:
        g2 = probdist[ins_size][bin2]
    else:
        g2 = 0

    if pi_new[0]>0.99:
        if g1>10**-3:
            return [1,0]
        else:
            return [0,0]
    elif pi_new[1]>0.99:
        if g2>10**-3:
            return [0,1]
        else:
            return [0,0]

    #r_0=pi_new[ins_size][0]*g1
    #r_1=pi_new[ins_size][1]*g2
    r_0=g1
    r_1=g2
    p_fn=r_0+r_1
    if g1<10**-3 and g2<10**-3:
        r=[0,0]
    else:
        r=[r_0/p_fn, r_1/p_fn]

    return r

def gaussian(x,mu, sigma):
    coeffi=1/(sigma*((2*math.pi)**(0.5)))
    exponent=math.exp(-((x-mu)**2)/((2*(sigma**2))))
    return coeffi*exponent


def get_pvalue_i(mu, r_num, std_cluster):
    STD_INSERT_SIZE = int(props["STD_INSERT_SIZE"])

    ## find sigma
    pval_res=[0.5,0.5]
    std_cluster=[0,0]
    r_num[0]=min(50, int(r_num[0]))
    r_num[1]=min(50, int(r_num[1]))
    
    for i in [0,1]:
        if round(r_num[i])>=2: # at least minimum number of matepairs!
            std_cluster[i]=STD_INSERT_SIZE/(round(r_num[i])**0.5)
        elif round(r_num[i])==-1: # used for phat
            std_cluster[i]=STD_INSERT_SIZE
        else:
            pval_res[i]=-1
    #print std_cluster
    ## use pval: 0.05
    ###
    ## find the decision boundary


    #######################
    ## assign p-value
    unit=0.1
    for k in [0,1]:
        if std_cluster[k]<=0:
            continue
        sum_err=0
        #i=db[k]
        i=0
        while(1):
            val=gaussian(i, mu[k], std_cluster[k])*unit
            #print mu[k],std_cluster[k], i, val
            sum_err+=val
            if mu[k]<0:
                i+=unit
            else:
                i-=unit
            if val<10**-10:
                #sum_err+=10**-15
                break
        pval_res[k]=min(sum_err,1)
    return pval_res


def get_pvalue(mu, std_cluster_list, r_num):
    ###
    ## find sigma
    pval_res=[0,0]
    std_cluster=[0,0]
    avail_dist=[]
    for i in [0,1]:
        if round(r_num[i])>=2: # at least minimum number of matepairs!
            std_cluster[i]=std_cluster_list[0]/(round(r_num[i])**0.5)
	    avail_dist.append(i)
        else:
            pval_res[i]=-1

    ## use pval: 0.05
    ###
    ## find the decision boundary
    db=[0,0]
    #######################
    ## assign p-value
    unit=0.1
    for k in avail_dist:

        sum_err=0
        i=db[k]
        while(1):
            val=Statistics.gaussian(i, mu[k], std_cluster[k])*unit
            #print mu[k],std_cluster[k], i, val
            sum_err+=val
            if mu[k]<0:
                i+=unit
            else:
                i-=unit
            if val<10**-10:
		#sum_err+=10**-9
                break
        pval_res[k]=min(sum_err,1)
#
    return pval_res

# p-value of heterozygous event; the smaller the more heterozygous
def get_phat_i(mu,c_mp):
    MEAN_INSERT_SIZE = int(props["MEAN_INSERT_SIZE"])
    STD_INSERT_SIZE = int(props["STD_INSERT_SIZE"])
    if len(c_mp)<=1:
        return 0 # homozygous

    #joint_prob = 0
    #for c in c_mp:
    #    z = abs(c)/STD_INSERT_SIZE
    #    prob=0
    #    if z<0.1:
    #        prob = 0.46
    #    elif z<0.25:
    #        prob = 0.4
    #    elif z<0.5:
    #        prob = 0.3
    #    elif z<1: 
    #        prob = 0.16
    #    elif z<1.5:
    #        prob = 0.07
    #    elif z<2:
    #        prob = 0.02
    #    elif z<2.5:
    #        prob = 0.01
    #    elif z<3:
    #        prob = 0.001
    #    elif z<3.5:
    #        prob = 0.0001
    #    else:
    #        prob = 0.00001
    #    joint_prob += math.log(prob)
    mu_c = sum(c_mp)/len(c_mp)
    #print mu, mu_c, len(c_mp)
    phat=0
    for c in c_mp:
        tmp = get_pvalue_i([0,(mu-mu_c)], [0, -1], 0)
        phat += math.log(tmp[1])

    return phat

############################################
## pvalue for heterozygosity 
def get_phat_p(mu, std_cluster_list, r_num):

    # likelihood test
    # homozygous model

    # heterozygous model

    if r_num[0]<2 or r_num[1]<2:
        return 1
    if mu[0]==mu[1]:
	return 1
    ###################
    ## find sigma
    std_cluster=[0,0]
    for i in [0,1]:
        if round(r_num[i])>0:
            std_cluster[i]=std_cluster_list[0]/(round(r_num[i])**0.5)
        else:
            pval_res[i]=-1

    #i=sum(mu)/2
    unit=0.1
    db=min(mu)

    prev_val=10**10
    i=db
    iter=0
    while(1):
        val_0=Statistics.gaussian(i, mu[0], std_cluster[0])*unit
        val_1=Statistics.gaussian(i, mu[1], std_cluster[1])*unit
        if val_0+val_1<10**-10:
            db=sum(mu)/2
            break

        if abs(val_0-val_1)<10**-10:
            db=i
            break
        #prev_i=i
        i+=unit
        iter+=1
    if min(mu)>i or max(mu)<i:
        db=sum(mu)/2

    #if mu[0]==mu[1]:
    #    print "CHECK........................ ", mu, db
    ## assign p-value
    if min(mu[0],mu[1])==mu[0]:
        mu_0=mu[0]
        std_0=std_cluster[0]
        mu_1=mu[1]
        std_1=std_cluster[1]

    elif min(mu[0],mu[1])==mu[1]:
        mu_0=mu[1]
        std_0=std_cluster[1]
        mu_1=mu[0]
        std_1=std_cluster[0]

    sum_err=0
    i=db
    iter=0
    while(1):
        val=Statistics.gaussian(i, mu_0, std_0)*unit
        sum_err+=val
        i+=unit
        if val<10**-10 and iter>1000:
            break
        iter+=1
    #if mu[0]==mu[1]:
    #    print "CHECK........................ sum1",  sum_err
    ## gaussian
    i=db
    iter=0
    while(1):
        val=Statistics.gaussian(i, mu_1, std_1)*unit
        sum_err+=val
        i-=unit
        if val<10**-10 and iter>100:
            break
        iter+=1

    #if mu[0]==mu[1]:
    #    print "CHECK........................ sum2",  sum_err

    return min(sum_err,1)

def get_sys_command_result(command):
    result = []
    command_result_pipe = os.popen(command)
    while True:
        line = command_result_pipe.readline()
        if not line:
            break
        line = line.strip()
        result.append(line)
    command_result_pipe.close()
    return result

#---------

def print_message_and_current_time(message):
    curr_time = datetime.datetime.now().ctime()
    curr_time_msg = message + curr_time
    
    # print message and current time
    print curr_time_msg
    print

#---------


# given a (one-liner) SQL statement, return the command needed
# at the command prompt to execute this on this system

def get_db_cli_command(sql_statement):
#    return "ssh %s \"%s \\\"%s\\\"\"" % (DB_SERVER, MYSQL_ONE_LINER_PREFIX, sql_statement)
    return "%s \"%s\"" % (MYSQL_ONE_LINER_PREFIX, sql_statement)

#---------

def initialize_cluster_field():
    cluster_table_name=props["cluster_table"]
    table_name=props["interesting_matepair_table"]
    table_name="interesting_matepairs_AB_CLARA1_chr10"

    cursor = db.cursor()

    #print "set null to ins_clusid"
    #update_sql_stmt="UPDATE %s SET ins_clusid=NULL;" % (table_name)
    #cursor.execute(update_sql_stmt)

    print "delete cluster table"
    delete_sql_stmt="DELETE FROM %s WHERE type=0;" % (cluster_table_name)
    cursor.execute(delete_sql_stmt)
    cursor.close()

    #command = "ls %s | grep \"%s\" | awk '{print \"rm -f %s/\"$0;}' | sh" % (CLUSTER_RESULTS_DIR, table_name, CLUSTER_RESULTS_DIR)
    #print command
    #os.system(command)


def kolmogorov_cum(x):
    if x==0:
        return 0
    coeffi=((2*math.pi)**(0.5))/x
    i=1
    res=0
    while(1):
        tmp=math.exp(-(((2*i)-1)**2*(math.pi**2))/(8*(x**2)))
        res+=tmp
        if tmp<10**-100:
            break
        i+=1

    return res*coeffi
    
def main(argv):
    #handle_command_line_args(argv)
    initialize_global_vars()
    chr = sys.argv[1]
    step = int(sys.argv[2])
    step_size = int(sys.argv[3])

    CHI_BIN = 1
    EXE_DIR=props["EXE_DIR"]

    MEAN_INSERT_SIZE = int(props["MEAN_INSERT_SIZE"])
    STD_INSERT_SIZE = int(props["STD_INSERT_SIZE"])

    ########################
    ## SIMULATION
    probdist[MEAN_INSERT_SIZE]={}
    FILE_l=open(EXE_DIR+"/data/PofY.txt","r")
    illu_line=FILE_l.readline()
    while illu_line:
        tmp=illu_line.split()
        probdist[MEAN_INSERT_SIZE][int(tmp[0])]=float(tmp[1])
        illu_line=FILE_l.readline() 

    FILE_l.close()
    mean_list[MEAN_INSERT_SIZE]=MEAN_INSERT_SIZE ## change if nessasary
    std_list[MEAN_INSERT_SIZE]=STD_INSERT_SIZE


    for ins_size in mean_list.keys():
        probdist_keys[ins_size]=probdist[ins_size].keys()
        probdist_keys[ins_size].sort()

    for insert_size in mean_list.keys():
        if insert_size==MEAN_INSERT_SIZE:
            keys= probdist[insert_size].keys()
            keys.sort()
            n=0
            #for k in keys:
            #    print k, probdist[insert_size][k]
    #            n+=probdist[insert_size][k]
    #            
    #        print n

    #print probdist_keys
    #for i in mean_list:
    #    print i, mean_list[i]
    #    print i, std_list[i]

    #print "chi", chi_sig_val
    #initialize_cluster_field()

    # kol:  0.52 0.0503056926705
    # kol:  1.36 0.950514123245
    # reject if root n Dn > 1.36
    #for x in range(200):
    #    x=x/100
    #    print "kol: ", x, kolmogorov_cum(x)
    #print  get_phat([420,420], [200,200,200,150,150], [3,2], [[1,0],[1,0],[1,0],[0,1],[0,1]])
    #input_pofy_db()
    #test_pofy_db()
    #print Kolm_montecarlo(40)
    #montecarlo_run()

    soft_clustering_indel(chr,step,step_size)

    #inference(chr, f_n)
    #repmaskfile = "./repeatMasker.txt"
    #search_rep=Searcher_repeat(repmaskfile)
    #ret=search_rep.find('chr3', 123892670, 123892695)
    #ret=remove_repeats_file.repeatmasker_overlap('chr1',1700,2000)
    #print "repeat", ret
    #profile.run('test_mog()')
    #test_mog()

    #std_list_=[13,13,13,13,13]
    #pval=get_pvalue([-1985400,0], std_list_, [10,0])
    
    #pval_h=get_phat([-3,-1], std_list_, [100,134], [[1,0]])
    #print "pval :", pval
    #print "phat :", pval_h

#---------

if __name__=="__main__":
    main(sys.argv[1:])

