#!/usr/bin/python
from __future__ import division
from sets import Set
import os
import sys
from math import sqrt
import MPProperties
import math

props = {}
diff_density=20
UNAVAILABLE=-100

def initialize_global_vars():
    # set the properties hash
    global props
    # read the properties hash
    props = MPProperties.read_config()

def initialize_statistics(db, mean_list, std_list):
    initialize_global_vars()
    good_matepair_table=props["good_matepair_table"]

    cursor = db.cursor()
    sql_stmt = "SELECT distinct insert_size FROM %s" % (good_matepair_table)
    cursor.execute(sql_stmt)
    result_set = cursor.fetchall()
    insert_size_list=[]
    for result_set_row in result_set:
        insert_size_list.append(result_set_row[0])

    for insert_size in insert_size_list:
        sql_statement = "SELECT mapped_dist, count FROM %s where mapped_dist<%d and insert_size=%d" %(good_matepair_table, insert_size*2, insert_size)
        cursor.execute(sql_statement)
        result_set = cursor.fetchall()

        square_weighted_dist=0
        weighted_dist=0
        tot_cnt=0
        for result_set_row in result_set:
            weighted_dist += result_set_row[0] * result_set_row[1]
            square_weighted_dist += (result_set_row[0]**2) * result_set_row[1]
            tot_cnt += result_set_row[1]
        avg = weighted_dist/tot_cnt
        square_avg = square_weighted_dist/tot_cnt
        std_dev = sqrt(square_avg - avg**2)
        mean_list[insert_size]=avg
        std_list[insert_size]=std_dev
    #return mean_list

#########################
## area: prob
## k: degree of freedom

def chi_table_val(area, df):
    k=int(df/2)*2 ## change degree of freedom nearest 2
    pval=0
    x=0
    step=0.1
    #if area==0.05 and df>400:
    #    print "error"
    #    return -1
    #####################3
    ## return from table
    if area==0.05 and df==30:
        return 43.77
    elif area==0.05 and df==25:
        return 37.65
    elif area==0.05 and df==20:
        return 31.41

    ###################
    ## initialization
    if area==0.05 and df>200:
        x=200
        step=1
    elif area==0.05 and df>100:
        x=100
        step=1
    elif area==0.05 and df>30:
        x=40
        step=1
    elif area==0.05 and df<=30:
        step=0.05

    pt=0
    ###################
    ## cumulative fn
    while(1):
        cum_density=lower_gamma(k/2,x/2)/math.exp(log_factorial(int(k/2)-1))
        #print x, cum_density, 1-cum_density, k, x
        if 1-cum_density<area:
            pt=x
            break
        x+=step

    return pt

################
# return log(n!)
def log_factorial(n):
    if n==0 or n==1:
        return 0

    if n<20:
        f = 1
        while (n > 0):
            f = f * n
            n = n - 1
        f=math.log(f)
    else:
        #f=(math.log(n)*n)-n
        #f=(2*math.pi*n)**(0.5)*((n/math.e)**(n))
        f=0.5*math.log(2*math.pi*n)+n*math.log(n/math.e)
    #f=math.exp(f)
    return f


def lower_gamma(a,x):
    ret=0
    step=0.1
    t=0.00001
    while(t<=x):
        log_val=(a-1)*math.log(t)-t+math.log(step)
        #val=t**(a-1)*math.exp(-t)*step
        ret+=math.exp(log_val)
        #ret+=val
        t+=step
    return ret

def get_prob_diff_dist(db, diff_prob, diff_prob_avg, winsz):
    initialize_global_vars()
    density=20

    good_matepair_table=props["good_matepair_table"]
    cursor = db.cursor()
    sql_stmt = "SELECT distinct insert_size FROM %s" % (good_matepair_table)
    cursor.execute(sql_stmt)
    result_set = cursor.fetchall()
    size_list=[]
    for result_set_row in result_set:
        size_list.append(result_set_row[0])

    mp_cnt_list={}
    for size in size_list:
        winsz[size]=int(size/density)
        #print size, density, winsz[size]
        #winsz[size]=density;
        norm=0
        #prob_dist=pyskip.SkipList(keyFun=None, cmpFun=None, allowDups=0, count=50000);
        mp_cnt={}
        cursor = db.cursor()
        sql_statement = "SELECT mapped_dist, count FROM %s WHERE insert_size=%d and mapped_dist<%d" % (good_matepair_table, size, 2*size)

        cursor.execute(sql_statement)
        result_set = cursor.fetchall()
        for result_set_row in result_set:
            key=int(result_set_row[0]/winsz[size])*winsz[size]
            if key in mp_cnt.keys():
                mp_cnt[key]=mp_cnt[key]+result_set_row[1]
            else:
                mp_cnt[key]=result_set_row[1]
            #norm=norm+result_set_row[1]
        #for k in mp_cnt.keys():
        #    mp_cnt[k]=mp_cnt[k]/norm
        cursor.close()
        mp_cnt_list[size]=mp_cnt

    #size_list=[44000,12500]

    for size1 in size_list:
        diff_prob[size1]={}
        diff_prob_avg[size1]={}
        for size2 in size_list:
            if size2>=size1:
                #print "processing diff prob... ", size1, size2
                mp1=mp_cnt_list[size1]
                mp2=mp_cnt_list[size2]
                diff={}
                for key1 in mp1.keys():
                    for key2 in mp2.keys():
                        if key1<=key2:
                            dkey=abs(key1-key2)
                            dkey=int(dkey/diff_density)*diff_density
                            if dkey not in diff.keys():
                                diff[dkey]=mp1[key1]*mp2[key2]
                            else:
                                diff[dkey]=diff[dkey]+(mp1[key1]*mp2[key2])
                avg=0
                denum=0
                for key in diff.keys():
                #    if size1==2250 and size2==44000:
                    #print "key, val", diff[key]
                    avg=avg+(key*diff[key])
                    denum=denum+diff[key]    
                #print avg,denum
                avg=avg/denum
                #    denum=denum+diff[key]
                # compute normalization factor
                #print "avg:", size1, size2, avg

                for dkey in diff.keys():
                    diff[dkey]=diff[dkey]/denum

                diff_prob[size1][size2]=diff
                diff_prob_avg[size1][size2]=avg
                #print "avg: ", str(avg)
    #temp=diff_prob[10000][10000]
    #temp2=0
    #for k in temp.keys():
    #    print "diff:", k, temp[k]
    #    temp2=temp2+temp[k]
    #print temp2    
    #for x in range(100):
    #    y=x*1000
    #    #a=450259
    #    #b=425123
    #    tmp=prob_diff_dist(10, 12500, y, 44000, diff_prob, diff_prob_avg, winsz)
    #    print "diff:", abs(y-10), math.exp(tmp) , diff_prob_avg[12500][44000]
        
    #tmp=prob_diff_dist(450259, 12500, 425123, 44000, diff_prob, diff_prob_avg, winsz)
    #print "result:", math.exp(tmp)


#####################
## ABSOLID STATISTICS 
def absolid_statistics(mean_list, std_list):
    # for now, insert sizes are hard coded
    size_list=[[1711,316],[1179,221],[600,58],[804,184],[2841,611],[3469,697],[2382,726]]
    for i in size_list:
        mean_list[i[0]]=i[0]
        std_list[i[0]]=i[1]

#################################
## NORMAL DISTRIBUTION DENSITY FN
def normal_dist(mean, sigma, x):

    ##########################
    ## return gaussian density
    pdf = 1/(sigma*((2*math.pi)**(1/2)))*math.exp(-(((x-mean)**2)/(2*(sigma**2))))

    return pdf

###########################################
# THIS DISTRIBUTIONS IS FOR AB SOLiD DATA!!!
# N(X-Y) ~ N(mu_X-mu_Y, var_X+var_Y)

def absolid_prob_diff(len1, insert_size1, len2, insert_size2, mean_list, std_list):

    xval=abs(len1-len2)
    mean=abs(mean_list[insert_size1]-mean_list[insert_size2])
    var=std_list[insert_size1]**2+std_list[insert_size2]**2
    sigma=var**0.5
    density = normal_dist(mean, sigma, xval)
    
    return density

def prob_diff_dist(len1, insert_size1, len2, insert_size2, diff_prob, diff_prob_avg, winsz):
    TINY=10**-5
    len1=int(len1/winsz[insert_size1])*winsz[insert_size1]
    len2=int(len2/winsz[insert_size2])*winsz[insert_size2]

    dif=abs(len1-len2)

    s1=min(insert_size1, insert_size2)
    s2=max(insert_size1, insert_size2)
    mu=diff_prob_avg[s1][s2]

    # get expectation value
    sum=0

    t=abs(dif-mu)
    for key in diff_prob[s1][s2].keys():
        if key>=mu-t and key<mu+t:
            sum=sum+diff_prob[s1][s2][key]
    #if key>=dif:
    #    sum=sum+diff_prob[s1][s2][key]
    if sum>=1:
        return math.log(TINY)
    else:
    #print "sum: " + str(sum)+", prob:" + str(math.log(sum)) + "mu&t:", mu,t
        return math.log(1-sum)

def gaussian(x,mu, sigma):
    #TINY=10**-10
    TINY=0
    coeffi=1/(sigma*((2*math.pi)**(0.5))+TINY)
    exponent=math.exp(-((x-mu)**2)/((2*(sigma**2)+TINY)))

    return coeffi*exponent

def normal_pdf(mean,sigma,x):
    ##########################
    ## return gaussian density
    #mean=mean_list[mu]
    #sigma=std_list[mu]

    pdf = 1/(sigma*((2*math.pi)**(1/2)))*math.exp(-(((x-mean)**2)/(2*(sigma**2))))

    return pdf


def cumulative_dist(diff, mu, sigma):
    xval=max(mu-diff,0)
    cum=0
    while(1):
        cum=cum+normal_pdf(mu,sigma,xval)
        xval=xval+1
        if xval>mu+diff or xval>2*mu:
            break
    return cum

############################################
## Used for finding config for AB SOLiD data

def absolid_indel_avg_dist(mu, sigma, mapped_dist):
    prob=1-cumulative_dist(abs(mapped_dist-mu), mu, sigma)
    return prob



def indel_logprob(len1, insert_size1, len2, insert_size2, mean_list, getprob, variant, span, alpha):
    insert1=abs(mean_list[insert_size1]-len1)
    insert2=abs(mean_list[insert_size2]-len2)
    if variant==0: # INSERTION
        insert=(insert1+insert2)/2
    elif variant==1: # DELETION
        insert=min((insert1+insert2)/2, span)
        
    loop=0
    prob=[0,0]
    while(loop<2):
        if loop==0:
            insert_size=insert_size1;
            if variant==0:
                len=len1+insert
            elif variant==1:
                len=len1-insert
        elif loop==1:
            insert_size=insert_size2;
            if variant==0:
                len=len2+insert
            elif variant==1:
                len=len2-insert

        if len>mean_list[insert_size]+alpha[insert_size] or len<mean_list[insert_size]-alpha[insert_size]:
            prob[loop]=10**-10
        else:
            temp=int((abs(len-mean_list[insert_size])/100))*100
            prob[loop]=max(getprob[insert_size][temp], 10**-10)
        loop=loop+1

    return math.log(prob[0])+math.log(prob[1])

#---------

def main(argv):
    initialize_global_vars()
    #diff_prob={}
    #diff_prob_avg={} 
    #winsz={}
    #get_prob_diff_dist(db, diff_prob, diff_prob_avg, winsz)
    mean_list={}
    std_list={}
    #absolid_statistics(mean_list, std_list)
    #print absolid_prob_diff(1000,1711, 3000, 1179, mean_list, std_list)
    #print absolid_prob_diff(1100,1711, 1100, 1179, mean_list, std_list)
    #print absolid_prob_diff(1200,1711, 1100, 1179, mean_list, std_list)
    #print absolid_prob_diff(1300,1711, 1100, 1179, mean_list, std_list)
    #print absolid_prob_diff(1400,1711, 1100, 1179, mean_list, std_list)
    #print absolid_prob_diff(1711,1711, 1179, 1179, mean_list, std_list)
    #print absolid_prob_diff(343,1179, 0, 1179, mean_list, std_list)
#
    initialize_statistics(db, mean_list, std_list)
    #print mean_list, std_list
    #print absolid_indel_avg_dist(1711,316,1711)
    #print chi_table_val(0.05, 12)

#---------

if __name__=="__main__":
    main(sys.argv[1:])
