#!/usr/bin/env python

from __future__ import division
import os
import sys
sys.path.append('../')
import MPProperties
import datetime
import logging
import math
from sets import Set
import random
import gzip
import copy
import dbhash

# logger configuration
mean_list={}
std_list={}

alpha={}
getprob={}
coverage_list={}



props = {}
chr_list=['chr1','chr2', 'chr3','chr4','chr5','chr6','chr7','chr8','chr9','chr10','chr11','chr12','chr13','chr14','chr15','chr16','chr17','chr18','chr19','chr20', 'chr21','chr22','chrX','chrY']

queue="doubleSlot"

def complement(seq):
    #complement = {'A': 'T', 'C': 'G', 'G': 'C', 'T': 'A'}
    complement = {'A': 'T', 'C': 'G', 'G': 'C', 'T': 'A', 'N':'N'}
    complseq = [complement[base] for base in seq]
    return complseq

def reverse_complement(seq):
    seq = list(seq)
    seq.reverse()
    return ''.join(complement(seq))

def initialize_global_vars():
    # set the properties hash
    global props
    # read the properties hash
    props = MPProperties.read_config()


def get_sys_command_result(command):
    result = []
    command_result_pipe = os.popen(command)
    while True:
        line = command_result_pipe.readline()
        if not line:
            break
        line = line.strip()
        result.append(line)
    command_result_pipe.close()
    return result

#---------

def print_message_and_current_time(message):
    curr_time = datetime.datetime.now().ctime()
    curr_time_msg = message + curr_time
    
    # log the time message
    log.info(curr_time_msg)
    
    # print message and current time
    print curr_time_msg
    print

#---------


# given a (one-liner) SQL statement, return the command needed
# at the command prompt to execute this on this system
#---------

def return_list_from_result_set_column(mysqldb_result_set, column_index):
    result = []
    for result_set_row in mysqldb_result_set:
        result.append(result_set_row[column_index])
    return result

#---------

def return_str_list_from_list(list):
    result = []
    for item in list:
        result.append(str(item))
    return result

#---------    

class Searcher_print:
    def __init__(self, filename):
        self.f = open(filename, 'rb')
        self.f.seek(0,2)
        self.length = self.f.tell()

    def find(self, chr, start, end):

        low = 0
        high = self.length
        while low < high:
            if abs(low-high)<1:
                break
            mid = (low+high)/2
            p = mid
            while p > 0:
                self.f.seek(p)
                if self.f.read(1) == '\n': break
                p -= 1
            line = self.f.readline()
            tmp=line.split()
            if tmp==[]:
                low=mid+1
                continue
            chr_=tmp[1]
            start_=int(tmp[2])

            if chr_<chr:
                low = mid+1
            elif chr_>chr:
                high = mid
            elif start_<start:
                low = mid+1
            else:
                high = mid
        p = low
        while p > 0:
            self.f.seek(p)
            if self.f.read(1) == '\n': break
            p -= 1

        result = []
        counter=0
        prev_start_=0
        while True:
            line = self.f.readline()
            tmp=line.split()
            if tmp==[]:
                break
            chr_=tmp[1]
            start_=int(tmp[2])
            end_=int(tmp[3])
            if not line or chr_!=chr or start_>end:
                break
            result.append([chr_,start_,end_, tmp[0]])
            counter+=1
        return result



############################
## not affected by SNPs
def running_on_qsub(qsubcommand, job_name_prefix, file_num, queue):
    # create a file for qsub to execute (embedding our command in the file)
    qsub_file_name = "%s/%s_%d.sh" % (JOBS_DIR, job_name_prefix, file_num)
    #curr_working_directory = os.getcwd()
    qsub_file = open(qsub_file_name, "w")
    print >> qsub_file, qsubcommand
    qsub_file.close()
    chmod_command = "chmod +x %s" %(qsub_file_name)
    os.system(chmod_command)

    # run qsub
    job_name = "\"%s_%d\"" % (job_name_prefix, file_num)
    #command = "qsub -S /bin/sh -N %s -o %s -e %s %s" % (job_name, std_out_file_name, std_err_file_name, qsub_file_name)
    command = "qsub -S /bin/sh -N %s -q %s -o %s -e %s %s" % (job_name, queue, LOG_DIR, LOG_DIR, qsub_file_name)
    print command
    os.system(command)

def merge_results(chr):
    exe_dir=props["EXE_DIR"]
    result_dir=props["RESULTS_DIR"]


    fcnt=0
    for prefix in ['ins','del']:
	print prefix, 'processing ...', chr
        print 

        # sort results files
        command='sort -S 1G -k 3n,3 -k 4n,4 %s/POST_PROCESSING/%s/%s_%s_results.txt > %s/POST_PROCESSING/%s/%s_%s_results.txt.sorted' % (result_dir, chr, chr, prefix, result_dir, chr, chr, prefix)
        print command
        os.system(command)

        f_list='%s_%s_results.txt.sorted' % (chr, prefix)
        for f in [f_list]:
	    if prefix=='ins':
		indl_type=0
	    else:
		indl_type=1
            full_fname="%s/POST_PROCESSING/%s/%s" % (result_dir, chr, f)
	    qsub_command="%s/post_processing/qsub_merging_results.py %d %s %s" %(exe_dir, indl_type, full_fname, chr)
	    print "command: ", qsub_command
            os.system(qsub_command)
            #running_on_qsub(qsub_command, "f_"+chr, fcnt, queue)
	    fcnt+=1

def make_template_id_files_no_qsub(chr):
    chr_dir=chr
    command='ls %s/%s | grep results' % (inference_dir,chr_dir)
    f_list=get_sys_command_result(command)
    fcnt=0
    for f in f_list:
	fname=inference_dir+'/'+chr_dir+'/'+f
	command="%s/qsub_make_template_id_files.py %s %s %d" %(EXE_DIR, chr, fname, fcnt)
	print command
        os.system(command)
	fcnt+=1

def make_template_id_files(chr):
    #chr='chr1'
    chr_dir=chr
    #chr_dir='inference_chr1_maq_mappings_no_factor'
    #chr='chr20'
    #chr_dir='chr20'
    command='ls %s/%s | grep results' % (inference_dir,chr_dir)
    f_list=get_sys_command_result(command)
    fcnt=0
    for f in f_list:
	fname=inference_dir+'/'+chr_dir+'/'+f
	qsub_command="%s/qsub_make_template_id_files.py %s %s %d" %(EXE_DIR, chr, fname, fcnt)
	#print qsub_command
        running_on_qsub(qsub_command, "tl_"+chr, fcnt, queue)
	fcnt+=1



def compute_pval_using_qsub(chr):
    #command='ls %s/%s | grep results | egrep ^[^s] | egrep ^[^p]' % (inference_dir2,chr)
    #f_list=get_sys_command_result(command)
    #fcnt=0
    #for f in f_list:
        #qsub_command="%s/qsub_score_filtering.py %s/%s/%s %s" %(EXE_DIR, inference_dir2, chr, f, chr)
    qsub_command="%s/qsub_pval_filtering.py %s" %(EXE_DIR, chr)
    print qsub_command
    running_on_qsub(qsub_command, "sc_"+chr, 0, queue)
    #    fcnt+=1

def compute_pval(chr):
    exe_dir=props["EXE_DIR"]
    command="%s/post_processing/qsub_pval_filtering.py %s" %(exe_dir, chr)
    os.system(command)

def concatenate_results(chr):
    exe_dir=props["EXE_DIR"]
    command="%s/post_processing/qsub_concatenate_results.py %s" %(exe_dir, chr)
    #print qsub_command
    os.system(command)
    #running_on_qsub(qsub_command, "conc_"+chr, 0, queue)

def make_template_ids_db_using_qsub(chr):
    qsub_command="%s/qsub_make_template_ids_db.py %s" %(EXE_DIR, chr)
    print qsub_command
    running_on_qsub(qsub_command, "pd_"+chr, 0, queue)

def compute_score_using_qsub2(chr):
    qsub_command="%s/qsub_score_filtering2.py %s" %(EXE_DIR, chr)
    #print qsub_command
    running_on_qsub(qsub_command, "s2_"+chr, 0, queue)
    #os.system(qsub_command)

def post_FDR(chr):
    exe_dir=props["EXE_DIR"]
    command="%s/post_processing/qsub_post_FDR.py %s" %(exe_dir, chr)
    print command
    os.system(command)

#qsub_command="%s/qsub_filtering_results.py %s %s %d" %(EXE_DIR, chr, fname, fcnt)
def extract_results_using_qsub(chr_dir):
    results_dir=props["RESULTS_DIR"]
    inference_dir=results_dir+"/INFERENCE_RESULTS/"
    command='ls %s/%s | grep results' % (inference_dir,chr_dir)
    f_list=get_sys_command_result(command)
    fcnt=0
    for f in f_list:
	fname=inference_dir+'/'+chr_dir+'/'+f
	qsub_command="%s/qsub_post_estimating_results.py %s %s %d" %(EXE_DIR, chr, fname, fcnt)
        running_on_qsub(qsub_command, "p_"+chr, fcnt, queue)
	fcnt+=1

def extract_results(chr):
    exe_dir=props["EXE_DIR"]
    results_dir=props["RESULTS_DIR"]
    inference_dir=results_dir+"/INFERENCE_RESULTS"

    command='ls %s/%s | grep results' % (inference_dir,chr)
    f_list=get_sys_command_result(command)
    for f in f_list:
	fname=inference_dir+'/'+chr+'/'+f
	command="%s/post_processing/qsub_post_estimating_results.py %s %s" %(exe_dir, chr, fname)
        print command
        os.system(command)

def concatenate_results_(type, chr):
    if type==0:
        FILE_out=gzip.open(inference_dir+'/RESULTS/'+chr+'_ins_results.txt.gz','wb')
    elif type==1:
        FILE_out=gzip.open(inference_dir+'/RESULTS/'+chr+'_del_results.txt.gz','wb')
    else:
	print 'ERROR ARG'
	return 

    command='ls %s/RESULTS/%s | grep filtered' % (inference_dir,chr)
    f_list=get_sys_command_result(command)
    for f in f_list:
	print 'processing...', type, f
        FILE=gzip.open(inference_dir+'/RESULTS/'+chr+'/'+f,'rb') 
	line=FILE.readline()
	while line:
	    tmp=line.split()
	    if int(tmp[5])==type:
		FILE_out.write(line)
	    line=FILE.readline()
	FILE.close()    
    FILE_out.close()
		
def pre_merge_results(type, results_file):
    if type==0:
        prefix='ins'
    else:
        prefix='del'
    #results_file=inference_dir+'/RESULTS/'+prefix+'_results.txt'
    results_file_gzip=results_file+'.gz'
    command='rm %s.sorted.gz' % (results_file)
    print '** clean up'
    os.system(command)
    ################################3
    ## 1.gunzip 
    ## 2.sort
    ## 3.gzip
    command='gunzip %s' % results_file_gzip
    print '** gunzip'
    os.system(command)
    command='sort -S 10G -k 3n,3 -k 4n,4 %s > %s.sorted' % (results_file, results_file)
    print '** sorting'
    os.system(command)
    command='gzip %s;gzip %s.sorted' % (results_file, results_file)
    print '** gzip'
    #print command
    os.system(command)

def make_template_ids_db(chr):
    result_dir=props["RESULTS_DIR"]
    result_dir+="POST_PROCESSING"
    #chr='chr1'
    #chr='chr20'
    #db = dbhash.open(result_dir+'/template_db_'+chr, "c")
    db = dbhash.open('/tmp/template_db_'+chr, "c")
    command='ls %s/%s | grep template' % (result_dir,chr)
    f_list=get_sys_command_result(command)
    for f in f_list:
	print 'making template db... ', f
        FILE_t=gzip.open(result_dir+'/'+chr+'/'+f, 'rb')	
	line=FILE_t.readline()
	while line:
	    tmp=line.split()
	    template_id_str=''
	    for t in tmp[1:]:
		template_id_str+=t+'\t'
            db[tmp[0]]=template_id_str
	    line=FILE_t.readline()
        FILE_t.close()	
    db.close()
    print 'copy file'
    command='cp /tmp/template_db_%s %s/template_db_%s; rm /tmp/template_db_%s' % (chr,result_dir,chr,chr)
    os.system(command)
    

def multi_sweep_merge(indl_type):
    ## insertion
    if indl_type==0:
        print '== merging for insertion ==='
        merge_results_(0)
        merge_results_(2)
    elif indl_type==1:
        print '== merging for deletion ==='
        merge_results_(1)
        merge_results_(3)
    else:
	print 'not valid type'
    
    

def merge_results_(type):
    result_dir=props["RESULTS_DIR"]
    result_dir+="POST_PROCESSING"
    inference_dir=props["RESULTS_DIR"]
    inference_dir+="INFERENCE_RESULTS"

    if type==0:
        prefix='ins'
	input_fn=inference_dir+'/RESULTS/'+prefix+'_results.txt'
    elif type==1:
        prefix='del'
	input_fn=inference_dir+'/RESULTS/'+prefix+'_results.txt'
    elif type==2:
        prefix='ins'
        input_fn=inference_dir+'/RESULTS/'+prefix+'_results.txt.final'
    elif type==3:
        prefix='del'
        input_fn=inference_dir+'/RESULTS/'+prefix+'_results.txt.final'
	

    pre_merge_results(type, input_fn) ## make sorted list


    print '** load template ids'
    db = dbhash.open(result_dir+'/template_db', "r")

    print '** merge results start!'
    FILE=gzip.open(input_fn+'.sorted.gz','rb')
    line=FILE.readline()
    open_list=[[-1, -1, 10**10, 10**10, 10, 0, 0]]

    progress=0
    clus_set=Set()
    clusters_list=[]
    clusters_info={}
    pre_t_set=Set()
    t_set=Set()

    while line:

	tmp=line.split()
        std_dev=13/(int(tmp[0])**0.5)
	err_range=std_dev*2
        start_pos=int(tmp[2])
        end_pos=int(tmp[3])
	mu=float(tmp[6])
	pval=float(tmp[8])
	clusid=tmp[9]
	membership=tmp[10]
	clusters_info[clusid]=[pval, int(tmp[0]), start_pos]
	#print line, clusid, db['0']

	if progress % 10000==0:
            print 'type', type, 'progress: ', progress, clusid
	progress+=1

        t_list=db[str(clusid)].split()
	t_set.clear()
	assign_template_id=0
	for t in t_list:
	    if t=='0' or t=='1': ## membership flag
	        if t==membership:
	            assign_template_id=1
		else:
	            assign_template_id=0
	    elif assign_template_id==1:
                t_set.add(t)
	
	#print clusid, membership, t_set, t_list

	##############################################
	## minus pvalue means it's nothing 
 	if pval<0:
            line=FILE.readline()
	    continue

	add_success=1
        for open in open_list: 
	    pre_start_pos=open[0]
	    pre_end_pos=open[1]
	    pre_mu=open[2]
	    pre_err_range=open[3]
	    pre_pval=open[4]
	    pre_clusid=open[5]
	    pre_membership=open[6]

            mu_range=[mu-err_range, mu+err_range]
            pre_mu_range=[pre_mu-pre_err_range, pre_mu+pre_err_range]
	    cond1=min(mu_range[1], pre_mu_range[1])-max(mu_range[0], pre_mu_range[0]) # size
	    cond2=min(end_pos, pre_end_pos)-max(start_pos, pre_start_pos) # location
	    pre_t_list=db[str(pre_clusid)].split()
	    pre_t_set.clear()
	    assign_template_id=0
	    for t in pre_t_list:
	        if t=='0' or t=='1': ## membership flag
	            if t==pre_membership:
	                assign_template_id=1
		    else:
	                assign_template_id=0
	        elif assign_template_id==1:
	            pre_t_set.add(t)
	    if min(len(t_set), len(pre_t_set))<10:
	        threshold_num_matepairs=3
	    else:
	        threshold_num_matepairs=5
	    if len(t_set.intersection(pre_t_set))>=threshold_num_matepairs:
		cond3=1
	    else:
	        cond3=0 ## if they share >2 matepairs

	    #print '---'
	    #print t_list, pre_t_list, len(t_set.intersection(pre_t_set))
	    #print t_set.intersection(pre_t_set)
	    #print '---'

	    if clusid=='131':
		print open, 'pre_clusid', pre_clusid, 'clusid', clusid, ' => cond1:', cond1, 'cond2:', cond2, 'cond3:', cond3, '(', len(t_set.intersection(pre_t_set)),')', open_list
		
	       
	        #print 'pre matepairs:', pre_t_set 
	        #print 'matepairs:', t_set 

	    if (type%2==1 and cond1>=0 and cond2>=max(mu, pre_mu) and cond3==1) or (type%2==0 and cond1>=0 and cond2>=0 and cond3==1): ## merge
		add_success*=1
	    else:
	        #if clusid=='296':
		#    print '*********** no dup ===>', 'pre_clusid', pre_clusid, 'clusid', clusid, ' => cond1:', cond1, 'cond2:', cond2, 'cond3:', cond3, '(', len(t_set.intersection(pre_t_set)),')'
		add_success*=0
		break
	        #if (pre_pval>pval) or (pre_pval==pval and err_range<=pre_err_range): ## still the best
                    #add_clus_set.add(clusid)
		    #if end_pos==198735925:
		#	print 'del', pre_clusid, 'curr:', clusid
		#    if pre_clusid=='40406030':
		#	print '1 deleted', clusid
	        #else:                                                                ## there is better one
		#    if clusid=='40406030':
		#	print '2 deleted', clusid, pre_clusid
		#    break
		    #if end_pos==243687948:
		#	print 'del', clusid
	        
        #if '710' in clus_set:
	#    print ' && ', clus_set
	#    print '---', add_success
        #if '711' in clus_set:
	#    print ' ** ', clus_set
	#    print '---', add_success
	    
	if add_success==0:
	    clusters_list.append(copy.copy(clus_set))
	    clus_set.clear()
            open_list=[]

 	open_list.append([start_pos, end_pos, mu, err_range, pval, clusid, membership])
	clus_set.add(clusid)

        line=FILE.readline()

    
    clusters_list.append(copy.copy(clus_set)) ## left over

    print '*** choose unique indel from clusters **' 
    #print clusters_list
    good_clus_set=Set()
    for clus in clusters_list:
	best_stat=[100, 0]
	best_clusid='99999'
	best_start_pos=0
	for clusid in clus: 
	    info=clusters_info[clusid]
	    pval=info[0]
	    num=info[1]
	    start_pos=info[2]
	    if (best_stat[0]>pval) or (best_stat[0]==pval and best_stat[1]<=num):
		best_stat=[pval, num]
	  	best_clusid=clusid
		best_start_pos=start_pos
        #if int(best_start_pos/1000)==219695:
	#    print 
	#    print clus
	#    print '** ', best_clusid
		
	good_clus_set.add(best_clusid)
		    
    print '*** WRITE final RESULTS'
    FILE_out=gzip.open(input_fn +'.final.gz','wb')
    FILE.seek(0)
    line=FILE.readline()
    while line: 
	tmp=line.split()
	clusid=tmp[9]
        if clusid in good_clus_set:
	    FILE_out.write(line)
        line=FILE.readline()
	

    db.close()
    FILE_out.close()
    FILE.close()
    
          

##############################################33
## simulate Markov process 
## to estimate location
def estimate_location(cluster_list):
    region=[-1,10**10]
    if len(cluster_list)<2:
	return region
	
    std_dev=13/(len(cluster_list)**0.5)

    tmp_list=[]
    for tmp in cluster_list:
        mu=float(tmp[3])
        mp_span=int(tmp[2])
	
        if abs(mp_span-mu)<=max(std_dev*3,10):
            tmp_list.append(tmp)
    #print cluster_list
    #################################
    ## compute transition probability
    markov={}
    markov_pfn={}
    markov_max={}
    stay={}
    markov_state_list=[]
    for tmp1 in tmp_list:
        tmp_start_pos_1=int(tmp1[0])
        tmp_end_pos_1=int(tmp1[1])
        mapid_1=tmp1[4]

        tmp_markov=[]
        stay[mapid_1]=0
        pfn=0
        max_overlap=-1

        for tmp2 in tmp_list:
	    mapid_2=tmp2[4]
            if mapid_1==mapid_2:
                continue
            tmp_start_pos_2=int(tmp2[0])
            tmp_end_pos_2=int(tmp2[1])
            overlap=max(min(tmp_end_pos_1, tmp_end_pos_2)-max(tmp_start_pos_1, tmp_start_pos_2), 0)
            if overlap>0:
                pfn+=overlap
                max_overlap=max(max_overlap, overlap)
                tmp_markov.append([mapid_2, overlap])

	########################
        ## store states to transition
        if max_overlap>0:
            markov_pfn[mapid_1]=pfn
            markov_max[mapid_1]=max_overlap
            markov[mapid_1]=copy.copy(tmp_markov)
	    markov_state_list.append(mapid_1)

    if len(markov_state_list)==0:
	return region

    #####################################
    ## simulate markov random walk
    r_state=random.randrange(0,len(markov_state_list))	
    state=markov_state_list[r_state]
    stay[state]=1
    for simul in range(2000):
        num_state=len(markov[state])
        if num_state==0:
	    print 'check it out!!! - no state to transition!!'
            break

        choose=random.randrange(0,num_state)
        dice=random.randrange(0, markov_max[state])
        if markov[state][choose][1]>=dice: 
            state=markov[state][choose][0]
            stay[state]+=1

    sort_state_list=[]
    for state in stay.keys():
        sort_state_list.append([-stay[state],state])
    sort_state_list.sort()
    use_mapid=Set()

    #prev_stay=sort_state_list[0][0]
    for state_ in sort_state_list:
        if state_[0]/sort_state_list[0][0]>0.5:
            use_mapid.add(state_[1])
        else:
            break

    for tmp in tmp_list:
        if tmp[4] not in use_mapid:
            continue
        start_pos=int(tmp[0])
        end_pos=int(tmp[1])
        mu=float(tmp[3])
        tmp_region=[max(region[0],start_pos),min(region[1],end_pos)]

        if (mu<0 and abs(tmp_region[1]-tmp_region[0])>abs(mu)) or mu>0:
            region[0]=tmp_region[0]
	    region[1]=tmp_region[1]
        else:
	    break

    return region

    #command = "ls %s | awk '{print \"rm -f %s/\"$0;}' | sh" % (JOBS_DIR, JOBS_DIR)
    #os.system(command)
    #command = "ls %s | awk '{print \"rm -f %s/\"$0;}' | sh" % (LOG_DIR, LOG_DIR)
    #os.system(command)




def main(argv):
    #handle_command_line_args(argv)
    initialize_global_vars()
    chr_input = sys.argv[1]

    #clear_log=1
    #if clear_log==1:
    #    command = "ls %s | awk '{print \"rm -f %s/\"$0;}' | sh" % (JOBS_DIR, JOBS_DIR)
    #    print '*** clean jobs directory ***'
    #    os.system(command)
    #    command = "ls %s | awk '{print \"rm -f %s/\"$0;}' | sh" % (LOG_DIR, LOG_DIR)
    #    print '*** clean log directory ***'
    #    os.system(command)


    #############################################
    ## 1. extract results & estimate location
    #chr_list=['chr1', 'chr2', 'chr3','chr4','chr5','chr6','chr7','chr8','chr9','chr10','chr11','chr12','chr13','chr14','chr15','chr16','chr17','chr18','chr19','chr20', 'chr21','chr22','chrX','chrY']
    chr_list=[chr_input]

    for chr in chr_list:
        print 'extract results ...', chr
        extract_results(chr)
        #extract_results_using_qsub(chr)


    #################################################
    ## 2. concatenate results for insertion/deletion
    for chr in chr_list:
        print 'concatenate results ...', chr
        concatenate_results(chr)


    #######################################
    ## 4. merge duplicated indel calls 
    for chr in chr_list:
        print 'merge results ...', chr
        merge_results(chr)

    ####################################
    ## 5. generate pvalue file
    for chr in chr_list:
        print 'pvalue file ...', chr
        compute_pval(chr)

    #####################################################
    ## final filtering with FDR 
    
    for chr in chr_list:
        print 'FDR control ...', chr
        post_FDR(chr)



if __name__=="__main__":
    main(sys.argv[1:])
