#!/usr/bin/env python

from __future__ import division
import os
import sys
import datetime
import math
import MPProperties
from sets import Set
import gzip

# logger configuration
#---------

props = {}

def initialize_global_vars():
    # set the properties hash
    global props
    # read the properties hash
    props = MPProperties.read_config()


def get_sys_command_result(command):
    result = []
    command_result_pipe = os.popen(command)
    while True:
        line = command_result_pipe.readline()
        if not line:
            break
        line = line.strip()
        result.append(line)
    command_result_pipe.close()
    return result

#---------

def print_message_and_current_time(message):
    curr_time = datetime.datetime.now().ctime()
    curr_time_msg = message + curr_time
    
    # log the time message
    log.info(curr_time_msg)
    
    # print message and current time
    print curr_time_msg
    print

#---------


# given a (one-liner) SQL statement, return the command needed
# at the command prompt to execute this on this system
#---------

def return_list_from_result_set_column(mysqldb_result_set, column_index):
    result = []
    for result_set_row in mysqldb_result_set:
        result.append(result_set_row[column_index])
    return result

#---------

def return_str_list_from_list(list):
    result = []
    for item in list:
        result.append(str(item))
    return result

#---------    


#G3203203231310012222133232     	
#53301_1_72/2    chr1    4404    -       -221    18      99      82      82      0       0       1       0       36      aagccagccttccgctccttgaagctggtctccaca88;;;;;;;7;;;;;;;;;;;;3;;;;;;;;;;;;;
#chr1    881930  +       8       1:G     0       2       GAGCCGGCTAGGACAGTGCCGGAAACTCAGCT        GCCTCTCCCCTTCAACTCAGCTGGCCCCCCGC        6       @31,28,26,17,8,4,       @5,8,10,19,28,32,       @0,0,0,0,1,0,

def running_on_qsub(qsubcommand, job_name_prefix, file_num, queue):
    # create a file for qsub to execute (embedding our command in the file)
    qsub_file_name = "%s/%s_%d.sh" % (JOBS_DIR, job_name_prefix, file_num)
    #curr_working_directory = os.getcwd()
    qsub_file = open(qsub_file_name, "w")
    print >> qsub_file, qsubcommand
    qsub_file.close()
    chmod_command = "chmod +x %s" %(qsub_file_name)
    os.system(chmod_command)

    # run qsub
    job_name = "\"%s_%d\"" % (job_name_prefix, file_num)
    #command = "qsub -S /bin/sh -N %s -o %s -e %s %s" % (job_name, std_out_file_name, std_err_file_name, qsub_file_name)
    command = "qsub -N %s -q %s -o %s -e %s %s" % (job_name, queue, LOG_DIR, LOG_DIR, qsub_file_name)
    print command
    os.system(command)


def concatenate_results(type, chr):
    results_dir=props["RESULTS_DIR"]
    
    if type==0:
        final_output=results_dir+'/POST_PROCESSING/'+chr+'/'+chr+'_ins_results.txt'
    elif type==1:
        final_output=results_dir+'/POST_PROCESSING/'+chr+'/'+chr+'_del_results.txt'
    else:
	print 'WRONG ARGUMENT'
	return 

    tmpdir=results_dir+'/POST_PROCESSING'
    command='ls %s/%s | grep filtered' % (tmpdir,chr)
    f_list=get_sys_command_result(command)
    fcnt=0
    for f in f_list:
	print 'processing...', type, f
        fname=tmpdir+'/'+chr+'/'+f
        output_name=tmpdir+'/'+chr+'/'+str(fcnt)+'.output.txt.'+str(type)
        fcnt+=1
        FILE_out_tmp=open(output_name,'w')
        FILE=gzip.open(fname,'r') 
        line=FILE.readline()
        while line:
            tmp=line.split()
            if int(tmp[5])!=type:
                line=FILE.readline()
                continue
                
            str_p=''
            for s in tmp:
                str_p+=s+'\t'
            str_p+='\n'
            FILE_out_tmp.write(str_p)
            line=FILE.readline()
        FILE.close()    
        FILE_out_tmp.close()

    ##############write
    output_name=tmpdir+'/'+chr+'/'+str(0)+'.output.txt.'+str(type)
    command='cat %s > %s' % (output_name, final_output)
    os.system(command)
    for f in range(1,fcnt):
        output_name=tmpdir+'/'+chr+'/'+str(f)+'.output.txt.'+str(type)
        command='cat %s >> %s' % (output_name, final_output)
        os.system(command)

    #FILE_out.close()
    #os.system(command)
    #command = "ls %s | awk '{print \"rm -f %s/\"$0;}' | sh" % (LOG_DIR, LOG_DIR)
    #os.system(command)
def main(argv):
    chr = sys.argv[1]
    #handle_command_line_args(argv)
    initialize_global_vars()
    concatenate_results(0,chr)
    concatenate_results(1,chr)



if __name__=="__main__":
    main(sys.argv[1:])
