#!/usr/bin/env python

from __future__ import division
import os
import sys
import datetime
import logging
import math
import MPProperties
from sets import Set
import random
import gzip
import copy

# logger configuration
mean_list={}
std_list={}

alpha={}
getprob={}
coverage_list={}
pval_dic={}

#---------
db = None
search=None
search2=None
props = {}

def initialize_global_vars():
    # set the properties hash
    global props
    # read the properties hash
    props = MPProperties.read_config()


def get_sys_command_result(command):
    result = []
    command_result_pipe = os.popen(command)
    while True:
        line = command_result_pipe.readline()
        if not line:
            break
        line = line.strip()
        result.append(line)
    command_result_pipe.close()
    return result

#---------

def print_message_and_current_time(message):
    curr_time = datetime.datetime.now().ctime()
    curr_time_msg = message + curr_time
    
    # log the time message
    log.info(curr_time_msg)
    
    # print message and current time
    print curr_time_msg
    print

#---------


# given a (one-liner) SQL statement, return the command needed
# at the command prompt to execute this on this system
#---------

def return_list_from_result_set_column(mysqldb_result_set, column_index):
    result = []
    for result_set_row in mysqldb_result_set:
        result.append(result_set_row[column_index])
    return result

#---------

def return_str_list_from_list(list):
    result = []
    for item in list:
        result.append(str(item))
    return result

#---------    

def create_formatted_extracted_results_with_FDR(chr, indel_type):
    MIN_NUM_MATEPAIRS=int(props["MIN_NUM_MATEPAIRS"])
    result_dir=props["RESULTS_DIR"]
    post_dir=result_dir+'/POST_PROCESSING'
    FDR=float(props["FDR_CONTROL"])
    if indel_type==0:
        pval_fn='%s/%s/pval_%s_ins_results.txt.sorted__final' % (post_dir, chr, chr)
    else:
        pval_fn='%s/%s/pval_%s_del_results.txt.sorted__final' % (post_dir, chr, chr)
    
    FILE = open(pval_fn, 'r')
    line=FILE.readline()
    while line:
        tmp=line.split()
        cluskey=tmp[0]
        pval=float(tmp[1])
        pval_dic[cluskey]=pval
        line=FILE.readline()
    FILE.close()

    if indel_type==0:

        #fn=post_dir+'/'+chr+'/ins_results.txt.final.final'
        #fn='%s/%s/%s_ins_results_split.txtaa__final.gz'
        fn='%s/%s/%s_ins_results.txt.sorted__final' % (post_dir,chr,chr)

    elif indel_type==1:

        fn='%s/%s/%s_del_results.txt.sorted__final' % (post_dir,chr,chr)

    else:
        print 'ERROR'
        return -1

    print 'generate results ', fn, '... with FDR: ', FDR

    FILE=open(fn,'r')
    line=FILE.readline()
    sort_list=[]
    progress=0


    pval_keys=pval_dic.keys()
    pval_keys_set=Set(pval_keys)
    while line:
        if progress % 50000 ==0:
            print 'read file', progress
        progress+=1
        tmp=line.split()
        cardi=int(tmp[13])

        if float(tmp[0])<0.9:
            line=FILE.readline()
            continue
            
        if indel_type==0 and cardi<MIN_NUM_MATEPAIRS:
            line=FILE.readline()
            continue
        elif indel_type==1 and cardi<MIN_NUM_MATEPAIRS:
            line=FILE.readline()
            continue

        chrom=tmp[1]
        call_start_pos=int(tmp[2])
        call_end_pos=int(tmp[3])
        phat=float(tmp[7])

        #0.999998509908  chr1    40378   40605   0       1       -15.0   0.989944836457  1.4511768813e-08        991     0       23      15      14      17      40420
        #40589
        clusid=tmp[9]
        member=int(tmp[10])
        cluskey=clusid+'_'+str(member)
        if cluskey in pval_keys_set:
            #if member==0:
            pval=pval_dic[cluskey]
            #else:
            #pval=pval_dic[cluskey]
        else:
            print 'pval not found'
            pval=float(tmp[8])


        if clusid=='3903291':
            print clusid, pval
        ################################################33333
        ## flitering out repeat
        #res_list=search.find(chrom, call_start_pos, call_end_pos)
        #if len(res_list)>=1:
        #    line=FILE.readline()
        #    continue
        if pval>=0:# and phat>0:
            sort_list.append([pval, line])
        #print pval, line
        line=FILE.readline()
    FILE.close()
    #print 'SORTING STARTS '
    sort_list.sort()
    #print 'SORTING IS DONE!'

    #print 'CLEAN EXISTING FILE'
    command='cd %s/%s; ls |grep %s.FDR |awk \'{print "rm "$1}\'|sh' % (post_dir,chr,fn)
    #print command
    os.system(command)

    #print
    #print 'WRITING SORTED LIST TO FILE'
    fn_out=fn+'.FDR'
    FILE_out=open(fn_out,'w')

    sum_pval=0
    totnum=0
    for l in sort_list:
        if sum_pval>FDR:
            break
        sum_pval+=l[0]
        totnum+=1
        FILE_out.write(l[1])
    FILE_out.close()
    print "totnum clusters", indel_type, chr, totnum
    print 



def merge_ins_del_final_results_with_FDR(chr):
    result_dir=props["RESULTS_DIR"]
    post_dir=result_dir+'/POST_PROCESSING'

    fn_ins='%s/%s/%s_ins_results.txt.sorted__final.FDR' % (post_dir,chr,chr)
    fn_del='%s/%s/%s_del_results.txt.sorted__final.FDR' % (post_dir,chr,chr)
    fn_all=post_dir+'/'+chr+'/'+chr+'_all_results.txt.final.final.FDR'
    command='cat %s %s > %s' % (fn_ins, fn_del, fn_all)
    os.system(command)

def main(argv):
    initialize_global_vars()
    chr = sys.argv[1]

    #command='ls %s/%s | grep pval_' % (inference_dir2,chr)
    #f_list=get_sys_command_result(command)
    #fcnt=0
    #for f in f_list:
    #    print 'processing ...', f
    #FILE = gzip.open('/home/'+user+'/file_repository/INDEL_RESULTS/'+chr+'/'+f, 'rb')

    create_formatted_extracted_results_with_FDR(chr,0)
    create_formatted_extracted_results_with_FDR(chr,1)
    merge_ins_del_final_results_with_FDR(chr)

if __name__=="__main__":
    main(sys.argv[1:])
