#!/usr/bin/env python

from __future__ import division
import os
import sys
import datetime
import logging
import math
import MPProperties
from sets import Set
import random
import copy
import gzip

# logger configuration
mean_list={}
std_list={}
cb_dic={}

getprob={}
coverage_list={}
#---------
search=None
search2=None
props = {}

probdist={}
probdist_keys={}
cum_prob_dist={}

SMOOTH_WINDOW=2

class Mem_Searcher:
    def __init__(self, filename):
        self.true_indel_list=[]
        self.f = open(filename, 'rb')
        line=self.f.readline()
        cnt=0
        while line:
            tmp=line.split()
            item_cnt=0
            tmp_=[]
            for item in tmp:
                item_cnt+=1
                if item_cnt==6:
                    tmp_.append(len(item))
                else:
                    tmp_.append(item)

            self.true_indel_list.append(tmp_)
            cnt+=1
            line=self.f.readline()
        self.length = cnt-1
        self.f.close()

    def find(self, chr, start, end):
        low = 0
        high = self.length
        while low < high:

            if abs(low-high)<1: # low == high
                break
            mid = (low+high)/2  # update position
            p = int(mid)
            #while p > 0:
            #    self.f.seek(p)
            #    if int(p)==0:
            #        break
            #    elif self.f.read(1) == '\n': break
            #    p -= 1
            tmp=self.true_indel_list[p]
            if tmp==[]:
                low=mid+1
                continue
            chr_=tmp[0]
            start_=int(tmp[1])

            if chr_<chr:
                low = mid+1
            elif chr_>chr:
                high = mid
            elif start_<start:
                low = mid+1
            else:
                high = mid
        p = max(int(low)-1, 0)
        #while p > 0:
        #    self.f.seek(p)
        #    if int(p)==0:
        #        break
        #    elif self.f.read(1) == '\n': break
        #    p -= 1

        result = []
        counter=0
        while True:
            tmp = self.true_indel_list[p]
            p+=1
            if p>self.length or tmp==[]:
	        break
            chr_=tmp[0]
            start_=int(tmp[1])
            end_=int(tmp[2])
            #if not line or chr_!=chr or start_>end:
            #print start_, end_
            if chr_!=chr or start_>end:
                break
            elif max(start_, start)<=min(end_, end):
                result.append([chr_,start_,end_, tmp[5], tmp[3]])
                counter+=1
        return result


def complement(seq):
    #complement = {'A': 'T', 'C': 'G', 'G': 'C', 'T': 'A'}
    complement = {'A': 'T', 'C': 'G', 'G': 'C', 'T': 'A', 'N':'N'}
    complseq = [complement[base] for base in seq]
    return complseq



def reverse_complement(seq):
    seq = list(seq)
    seq.reverse()
    return ''.join(complement(seq))

def initialize_global_vars():
    # set the properties hash
    global props
    # read the properties hash
    props = MPProperties.read_config()


def get_sys_command_result(command):
    result = []
    command_result_pipe = os.popen(command)
    while True:
        line = command_result_pipe.readline()
        if not line:
            break
        line = line.strip()
        result.append(line)
    command_result_pipe.close()
    return result

#---------

def print_message_and_current_time(message):
    curr_time = datetime.datetime.now().ctime()
    curr_time_msg = message + curr_time
    
    # log the time message
    log.info(curr_time_msg)
    
    # print message and current time
    print curr_time_msg
    print

#---------


# given a (one-liner) SQL statement, return the command needed
# at the command prompt to execute this on this system
#---------

def return_list_from_result_set_column(mysqldb_result_set, column_index):
    result = []
    for result_set_row in mysqldb_result_set:
        result.append(result_set_row[column_index])
    return result

#---------

def return_str_list_from_list(list):
    result = []
    for item in list:
        result.append(str(item))
    return result

#---------    

class Searcher_print:
    def __init__(self, filename):
        self.f = open(filename, 'rb')
        self.f.seek(0,2)
        self.length = self.f.tell()

    def find(self, chr, start, end):

        low = 0
        high = self.length
        while low < high:
            if abs(low-high)<1:
                break
            mid = (low+high)/2
            p = mid
            while p > 0:
                self.f.seek(p)
                if self.f.read(1) == '\n': break
                p -= 1
            line = self.f.readline()
            tmp=line.split()
            if tmp==[]:
                low=mid+1
                continue
            chr_=tmp[1]
            start_=int(tmp[2])

            if chr_<chr:
                low = mid+1
            elif chr_>chr:
                high = mid
            elif start_<start:
                low = mid+1
            else:
                high = mid
        p = low
        while p > 0:
            self.f.seek(p)
            if self.f.read(1) == '\n': break
            p -= 1

        result = []
        counter=0
        prev_start_=0
        while True:
            line = self.f.readline()
            tmp=line.split()
            if tmp==[]:
                break
            chr_=tmp[1]
            start_=int(tmp[2])
            end_=int(tmp[3])
            if not line or chr_!=chr or start_>end:
                break
            result.append([chr_,start_,end_, tmp[0]])
            counter+=1
        return result



#######################################################################################

#[clusid, start_pos, end_pos, ndist, member, mu[0], mu[1]]
def compute_pvalue(cluster_list):
    multiple=2
    std_list=[0,0]
    mu=[0,0]

    #r_num=[0,0]
    #for clus in cluster_list:
    #    mem=clus[4]
    #    if mem==0:
    #        r_num[0]+=1
    #    else:
    #        r_num[1]+=1


    #std_dev=[0,0]
    #for i in [0,1]:
    #    if r_num[i]!=0:
    #        std_dev[i]=13/(r_num[i]**0.5)
    
    effect_r_num=[0,0]
    for clus in cluster_list:
        mem=clus[4]
        if mem==0:
            mu[0]=clus[5]
            if abs(clus[3]-mu[0])<multiple*13:
                effect_r_num[0]+=1
        else:
            mu[1]=clus[6]
            if abs(clus[3]-mu[1])<multiple*13:
                effect_r_num[1]+=1

    effect_r_num[0]=min(50,effect_r_num[0])
    effect_r_num[1]=min(50,effect_r_num[1])

    pval=get_pvalue(mu, effect_r_num)
    return pval


def compute_ks_score(cluster_list):

    ########################## 
    ## compute ks stat

    #tmp_cluster.append([clusid, start_pos, end_pos, ndist, member])
    prior=[0,0]
    mu_list=[0,0]

    sup_val_list=[0,0]
    for member in [0,1]:
        bin_list=[] 
        max_bin=-1
        min_bin=10**10

        mu=-1
        totnum=0
        for clus in cluster_list:
            ndist=clus[3]
            mem=clus[4]
            if member==0:
                mu=clus[5]
            else:
                mu=clus[6]

            if mem==member:
                bin=208-ndist
                bin_list.append([bin, 1])
                max_bin=max(max_bin, bin)
                min_bin=min(min_bin, bin)
                totnum+=1

        mu_list[member]=mu
        bin_list.sort()
        prior[member]=totnum
        ################################################
        ## 1. make experimental cumulative distribution
        ind=0
        bin_idx=0
        cum_exp_dist={}
        for ebin in range(min_bin,max_bin+1,1):
            while(1):
                if bin_idx>=len(bin_list):
                    break
                bin_item=bin_list[bin_idx]
                if bin_item[0]>ebin:
                    break
                ind+=bin_item[1]
                bin_idx+=1
            cum_exp_dist[ebin]=ind/totnum

        #print cum_exp_dist

        sup_val=-1
        smooth_ks={}
        for sks_idx in range(SMOOTH_WINDOW):
            smooth_ks[sks_idx]=0
        idx=0
        for bin in probdist_keys[208]:
            cum_E=cum_prob_dist[208][bin]
            bin2=bin-mu

            if bin2<=max_bin and bin2>=min_bin:
                cum_O=cum_exp_dist[bin2]
            elif bin2<min_bin:
                cum_O=0
            else:
                cum_O=1
            #if (mu_[member]<mu_[1-member] and bin<(sum(mu_)/2)) or (mu_[member]>mu_[1-member] and bin>(sum(mu_)/2)) or (mu_[member]==mu_[1-member]):
            smooth_ks[idx%SMOOTH_WINDOW]=abs(cum_O-cum_E)
            idx+=1
            if idx<SMOOTH_WINDOW:
                ks=float((sum(smooth_ks.values()))/idx)
            else:
                ks=float((sum(smooth_ks.values()))/SMOOTH_WINDOW)
            sup_val=max(sup_val, ks)

       
        sup_val_list[member]=sup_val 
        
    #if prior[1]==0 or prior[0]==0: ## homozygous
    if prior[1]/sum(prior)<0.1 or prior[0]/sum(prior)<0.1: ## homozygous
        model_type=0
    elif mu_list[0]!=0 and mu_list[1]!=0:
        model_type=2
    else:
        model_type=1

    score=0
    for member in [0,1]:
        score+=prior[member]/sum(prior)*sup_val_list[member]

    #if model_type==1:
    #    score=prior[member]/sum(prior)*sup_val_list[member]
    if model_type==0:
        score*=1.5
    elif model_type==2:
        score*=2

    #if abs(int(mu_list[0]))>20 or abs(int(mu_list[1]))>20:
    #    print score, mu_list
    return score
    ############################
    ## multiplied by priors



# ['16100161', '267301416', 'chr1', '171401143', '171401273', '9.0', '071217_EAS51_0066:7:3:971:222', '171401176', '171401284', '1', '0.00451752678747', '0.510740752133', '-5', '0', '0.19166728613', '0.00749850335185']
def score_filtering(input_fn,chr):
    FILE = gzip.open(input_fn, 'rb')
    ifn=input_fn.split('/')
    last_name=ifn[len(ifn)-1]
    prefix='pval_'
    FILE_o = gzip.open('/home/'+user+'/file_repository/INDEL_RESULTS/'+chr+'/'+prefix+last_name, 'wb')
    line=FILE.readline()
    pre_clusid=''
    tmp_cluster=[]
    while line:
	tmp=line.split()
        clusid=tmp[0]
        mapid=tmp[1]
        chrom=tmp[2]
        start_pos=int(tmp[3])
        end_pos=int(tmp[4])
        ndist=int(float(tmp[5]))
        template_id=tmp[6]
        member=int(tmp[9])
        
        mu=[float(tmp[12]), float(tmp[13])]
        #print mu
        if clusid!=pre_clusid and len(tmp_cluster)>0:
            #score=compute_ks_score(tmp_cluster)
            pval=compute_pvalue(tmp_cluster)
            #print pval
            #print tmp[10], tmp[11]
            #print '---'
            #print score
            #if score<0.7:
            #FILE_o.write(clusid+'\t'+str(score)+'\n')
            FILE_o.write(pre_clusid+'\t'+str(pval[0])+'\t'+str(pval[1])+'\n')
            #print tmp_cluster
            tmp_cluster=[]

        tmp_cluster.append([clusid, start_pos, end_pos, ndist, member, mu[0], mu[1]])
        pre_clusid=clusid
        line=FILE.readline()
    
    pval=compute_pvalue(tmp_cluster)
    FILE_o.write(pre_clusid+'\t'+str(pval[0])+'\t'+str(pval[1])+'\n')

    FILE.close()
    FILE_o.close()


def recompute_pval(chr):
    result_dir=props["RESULTS_DIR"]
    post_dir=result_dir+'/POST_PROCESSING'
    
    for prefix in ['ins','del']:
        fn_o='%s/%s/pval_%s_%s_results.txt.sorted__final' % (post_dir, chr,chr,prefix)
        FILE_o=open(fn_o,'w')

        fn='%s/%s/%s_%s_results.txt.sorted__final' % (post_dir, chr,chr, prefix)
        FILE=open(fn, 'r')
        line=FILE.readline()
        while line:
            #print line
            tmp=line.split()
            
            mem=int(tmp[10])
            mu=float(tmp[6])
            clusid=tmp[9]
            key=clusid+'_'+str(mem)
            rnum=int(tmp[13])
            #if mem==0:  
            #    rnum=int(tmp[11])
            #else:
            #    rnum=int(tmp[12])
            #print mem, rnum, mu
            pval=get_pvalue([mu,0], [rnum,0])
            FILE_o.write(key+'\t'+str(pval[0])+'\n')
            #print pval
            line=FILE.readline()
        FILE.close()
        FILE_o.close()



def get_pvalue(mu, r_num):
    mean_size=int(props["MEAN_INSERT_SIZE"])
    std_size=int(props["STD_INSERT_SIZE"])
    ###
    ## find sigma
    pval_res=[0,0]
    std_cluster=[0,0]
    avail_dist=[]
    #r_num[0]=min(50, int(r_num[0]))
    #r_num[1]=min(50, int(r_num[1]))
    for i in [0,1]:
        if round(r_num[i])>2: # at least minimum number of matepairs!
            std_cluster[i]=std_size/(round(r_num[i])**0.5)
            avail_dist.append(i)
        else:
            pval_res[i]=-1
    #print std_cluster
    ## use pval: 0.05
    ###
    ## find the decision boundary


    #######################
    ## assign p-value
    unit=0.1
    for k in avail_dist:

        sum_err=0
        #i=db[k]
        i=0
        while(1):
            val=gaussian(i, mu[k], std_cluster[k])*unit
            #print mu[k],std_cluster[k], i, val
            sum_err+=val
            if mu[k]<0:
                i+=unit
            else:
                i-=unit
            if val<10**-10:
                #sum_err+=10**-15
                break
        pval_res[k]=min(sum_err,1)
    return pval_res

def gaussian(x,mu, sigma):
    #TINY=10**-10
    #coeffi=1/(sigma*((2*math.pi)**(0.5))+TINY)
    #exponent=math.exp(-((x-mu)**2)/((2*(sigma**2)+TINY)))
    coeffi=1/(sigma*((2*math.pi)**(0.5)))
    exponent=math.exp(-((x-mu)**2)/((2*(sigma**2))))
    return coeffi*exponent

def main(argv):
    #handle_command_line_args(argv)
    initialize_global_vars()

    #fn = sys.argv[1]
    chr = sys.argv[1]

    #######################################
    exe_dir=props["EXE_DIR"]
    mean_size=int(props["MEAN_INSERT_SIZE"])
    std_size=int(props["STD_INSERT_SIZE"])

    probdist[mean_size]={}
    FILE_l=open(exe_dir+"/data/illumina_PofY.txt","r")

    illu_line=FILE_l.readline()
    while illu_line:
        tmp=illu_line.split()
        probdist[mean_size][int(tmp[0])]=float(tmp[1])
        illu_line=FILE_l.readline()

    FILE_l.close()
    mean_list[mean_size]=mean_size
    std_list[mean_size]=std_size

    probdist_keys[mean_size]=probdist[mean_size].keys()
    probdist_keys[mean_size].sort()
    cum_prob_dist[mean_size]={}
    cum_E=0
    for ebin in probdist_keys[mean_size]:
        cum_E+=probdist[mean_size][ebin]
        cum_prob_dist[mean_size][ebin]=cum_E

    #######################################33
    ## 4. merge duplicated indel calls 
    #score_filtering(fn,chr)

    recompute_pval(chr)

    #pval=get_pvalue([4,0], [49,0])
    #print pval
    #merge_results(indl_type,fn,chr)

if __name__=="__main__":
    main(sys.argv[1:])
