#!/usr/bin/env python

from __future__ import division
import os
import sys
import datetime
import logging
import math
import MPProperties
import Statistics
from sets import Set
import random
import copy
#import dbhash
import gdbm

# logger configuration
mean_list={}
std_list={}
cb_dic={}

getprob={}
coverage_list={}
#---------
search=None
search2=None
props = {}
chr_list=['chr1','chr2', 'chr3','chr4','chr5','chr6','chr7','chr8','chr9','chr10','chr11','chr12','chr13','chr14','chr15','chr16','chr17','chr18','chr19','chr20', 'chr21','chr22','chrX','chrY']

class Mem_Searcher:
    def __init__(self, filename):
        self.true_indel_list=[]
        self.f = open(filename, 'rb')
        line=self.f.readline()
        cnt=0
        while line:
            tmp=line.split()
            item_cnt=0
            tmp_=[]
            for item in tmp:
                item_cnt+=1
                if item_cnt==6:
                    tmp_.append(len(item))
                else:
                    tmp_.append(item)

            self.true_indel_list.append(tmp_)
            cnt+=1
            line=self.f.readline()
        self.length = cnt-1
        self.f.close()

    def find(self, chr, start, end):
        low = 0
        high = self.length
        while low < high:

            if abs(low-high)<1: # low == high
                break
            mid = (low+high)/2  # update position
            p = int(mid)
            #while p > 0:
            #    self.f.seek(p)
            #    if int(p)==0:
            #        break
            #    elif self.f.read(1) == '\n': break
            #    p -= 1
            tmp=self.true_indel_list[p]
            if tmp==[]:
                low=mid+1
                continue
            chr_=tmp[0]
            start_=int(tmp[1])

            if chr_<chr:
                low = mid+1
            elif chr_>chr:
                high = mid
            elif start_<start:
                low = mid+1
            else:
                high = mid
        p = max(int(low)-1, 0)
        #while p > 0:
        #    self.f.seek(p)
        #    if int(p)==0:
        #        break
        #    elif self.f.read(1) == '\n': break
        #    p -= 1

        result = []
        counter=0
        while True:
            tmp = self.true_indel_list[p]
            p+=1
            if p>self.length or tmp==[]:
	        break
            chr_=tmp[0]
            start_=int(tmp[1])
            end_=int(tmp[2])
            #if not line or chr_!=chr or start_>end:
            #print start_, end_
            if chr_!=chr or start_>end:
                break
            elif max(start_, start)<=min(end_, end):
                result.append([chr_,start_,end_, tmp[5], tmp[3]])
                counter+=1
        return result


def complement(seq):
    #complement = {'A': 'T', 'C': 'G', 'G': 'C', 'T': 'A'}
    complement = {'A': 'T', 'C': 'G', 'G': 'C', 'T': 'A', 'N':'N'}
    complseq = [complement[base] for base in seq]
    return complseq



def reverse_complement(seq):
    seq = list(seq)
    seq.reverse()
    return ''.join(complement(seq))

def initialize_global_vars():
    # set the properties hash
    global props
    # read the properties hash
    props = MPProperties.read_config()


def get_sys_command_result(command):
    result = []
    command_result_pipe = os.popen(command)
    while True:
        line = command_result_pipe.readline()
        if not line:
            break
        line = line.strip()
        result.append(line)
    command_result_pipe.close()
    return result

#---------

def print_message_and_current_time(message):
    curr_time = datetime.datetime.now().ctime()
    curr_time_msg = message + curr_time
    
    # log the time message
    log.info(curr_time_msg)
    
    # print message and current time
    print curr_time_msg
    print

#---------


# given a (one-liner) SQL statement, return the command needed
# at the command prompt to execute this on this system
#---------

def return_list_from_result_set_column(mysqldb_result_set, column_index):
    result = []
    for result_set_row in mysqldb_result_set:
        result.append(result_set_row[column_index])
    return result

#---------

def return_str_list_from_list(list):
    result = []
    for item in list:
        result.append(str(item))
    return result

#---------    

class Searcher_print:
    def __init__(self, filename):
        self.f = open(filename, 'rb')
        self.f.seek(0,2)
        self.length = self.f.tell()

    def find(self, chr, start, end):

        low = 0
        high = self.length
        while low < high:
            if abs(low-high)<1:
                break
            mid = (low+high)/2
            p = mid
            while p > 0:
                self.f.seek(p)
                if self.f.read(1) == '\n': break
                p -= 1
            line = self.f.readline()
            tmp=line.split()
            if tmp==[]:
                low=mid+1
                continue
            chr_=tmp[1]
            start_=int(tmp[2])

            if chr_<chr:
                low = mid+1
            elif chr_>chr:
                high = mid
            elif start_<start:
                low = mid+1
            else:
                high = mid
        p = low
        while p > 0:
            self.f.seek(p)
            if self.f.read(1) == '\n': break
            p -= 1

        result = []
        counter=0
        prev_start_=0
        while True:
            line = self.f.readline()
            tmp=line.split()
            if tmp==[]:
                break
            chr_=tmp[1]
            start_=int(tmp[2])
            end_=int(tmp[3])
            if not line or chr_!=chr or start_>end:
                break
            result.append([chr_,start_,end_, tmp[0]])
            counter+=1
        return result



#######################################################################################

def contig_breaks(fname):
    global cb_dic
    command = "cat %s | grep \"contig\" | awk '{print $2, $3, $4;}'" % (fname)
    cblist=get_sys_command_result(command)

    for chr in chr_list:
        cb_dic[chr]=[]
    for cb in cblist:
        tmp=cb.split()
        chr=tmp[0]
        if chr not in chr_list:
            continue
        start_pos=int(tmp[1])
        end_pos=int(tmp[2])
        cb_dic[chr].append([start_pos,end_pos])
   
def merge_results_until_convergence(indl_type,fn,chr):
    
    totnum=merge_results(indl_type,fn,chr) # first inter
    iter=0
    totnum_pre=10**10
    if indl_type==0:
        prefix='ins'
    else:
        prefix='del'

    while 1:
        iter+=1
        #if totnum_pre<=totnum:
        if abs(totnum_pre-totnum)<100:
            print "coverged", iter
            break

        totnum_pre=totnum
        #for prefix in ['ins','del']:
        #command='ls %s/%s |grep %s_%s_results_split | grep __final.gz' % (post_dir, chr, chr, prefix)
        #f_list=get_sys_command_result(command) 
        #for f in f_list:
        #input_fn=fn[:len(fn)-3]+'__final.gz'
        input_fn=fn+'__final'
        #print 'input file name: ', input_fn
        #input_fn='%s/%s/%s' % (post_dir, chr, fn_last)
        #print input_fn
        totnum=merge_results(indl_type,input_fn,chr)


    ###############################
    ## final merging
    merge_overlapping_results(indl_type,input_fn,chr)


def merge_overlapping_results(type,input_fn,chr):        
    
    FILE = open(input_fn, 'r')
    line=FILE.readline()
    cluster_list=[]
    cluster_ptr_info={}
    while line:
	tmp=line.split()
        num_of_discor=int(tmp[13])

        std_dev=13/(num_of_discor**0.5)
	err_range=std_dev*2
        start_pos=int(tmp[2])
        end_pos=int(tmp[3])
	mu=float(tmp[6])
	phet=float(tmp[7])
	pval=float(tmp[8])
	clusid=tmp[9]
	membership=tmp[10]
        chk_key=clusid+'_'+membership

	cluster_list.append([start_pos, end_pos, abs(mu), chk_key])
        cluster_ptr_info[chk_key]=tmp
        line=FILE.readline()
    FILE.close()

    cluster_list.sort()


    used_cluster_set=Set()
    clus_set=Set()
    ##############################
    ## resolve overlapping indels
    cluster_final_list=[]

    search_cluster_list=cluster_list
    search_idx=0
    progress=0
    for clus in cluster_list:
        search_idx+=1
        if progress % 100==0:
            search_cluster_list=cluster_list[search_idx:]
            #print 'progress', progress
        progress+=1



        if clus[3] in used_cluster_set:
            continue

        clus_sz=clus[2]
        clus_set.add(clus[3])
        used_cluster_set.add(clus[3])

        for inner_clus in search_cluster_list:
            if inner_clus[3] in used_cluster_set:
                continue

            inner_clus_sz=inner_clus[2]
            # overlapping
            cond1=min(clus[1], inner_clus[1])-max(clus[0], inner_clus[0])
            cond2=abs(clus_sz-inner_clus_sz)
            if type==0:
                if cond1>=0 and cond2<10:
                    #print '----'
                    #print 'clus:', clus
                    #print 'inner:', inner_clus
                    clus_set.add(inner_clus[3])
                    used_cluster_set.add(inner_clus[3])
            elif type==1:
                if cond1>=max(clus_sz,inner_clus_sz) and cond2<10:
                    clus_set.add(inner_clus[3])
                    used_cluster_set.add(inner_clus[3])
            
            if clus[1]<inner_clus[0]: ## stop
                break
                
        cluster_final_list.append(copy.copy(clus_set))
        clus_set.clear()
            
           
    #print 'pick one'         
    good_clus_list=[]
    for sim_clus in cluster_final_list:
        best_score=-1
        best_ky='0000'
        best_start_pos=-1
        for ky in sim_clus:
            start_pos=int(cluster_ptr_info[ky][2])
            end_pos=int(cluster_ptr_info[ky][3])
            score=abs(float(cluster_ptr_info[ky][6]))

            if score>best_score:        
                best_ky=ky
                best_score=score
                best_start_pos=start_pos

        if best_ky!='0000':
            good_clus_list.append([best_start_pos, best_ky])


    good_clus_list.sort()

    output_fn=input_fn.split('__') 
    dst_f='%s__final' % output_fn[0]
    FILE_out=open(dst_f,'w')

    for good_clus in good_clus_list:
        ky=good_clus[1] 
        str_p=''
        for ptr in cluster_ptr_info[ky]:
            str_p+=ptr+'\t'
        str_p+='\n'
        FILE_out.write(str_p)
    FILE_out.close()
    print 'final tot num', len(good_clus_list) 




def merge_results(type,input_fn,chr):
    exe_dir=props["EXE_DIR"]
    MINMATEPAIRS=int(props["MIN_NUM_MATEPAIRS"])
#31      chr1    7604029 7604245 0       1       -8.0    0.43435203009   0.000330621483922       704535  0       32
    #command='cp %s/%s/template_db_%s.dbm /tmp' % (post_dir, chr, chr)
    #print command
    #os.system(command)

    #print '** load template ids'
    #dbname='%s/POST_PROCESSING/%s/template_db_%s.dbm' % (inference_dir, chr, chr)

    #dbname='/tmp/template_db_%s.dbm' % (chr)
    #dbname='/tmp/template_db_%s' % (chr)
    #print dbname
    #db = dbhash.open(result_dir+'/template_db_chr21', "r")

    #print '** merge results start!'
    #FILE=gzip.open(input_fn+'.sorted.gz','rb')


    
    #input_fn_sorted=input_fn+'.sorted'
    #open_list=[[-1, -1, 10**10, 10**10, 10, 0, 0]]
    open_list=[]

    progress=0
    clus_set=Set()
    clusters_list=[]
    clusters_info={}
    cluster_ptr_info={}
    pre_t_set=Set()
    t_set=Set()

    contig_breaks(exe_dir+"/data/contig_breaks_hg17.txt")
    #db = gdbm.open(dbname, 'r')
    #0.899088988541  chr1    181852632       181852828       0       1       -8.0    0.488315640914  0.0212648844296 17314352        0       9       14      7
    #    5       -1      10000000000
    FILE = open(input_fn, 'r')
    line=FILE.readline()
    while line:
	tmp=line.split()
        ### check if the nuber of matepairs is greater than threshold
        #print tmp
        #if len(tmp)==12:
        #10      chr1    93773839        93774051        0       1       -8.0    0.46944585637   0.0212648844296 9404589 0       11      8       8       -1
        #print tmp
        #1.0     chr1    193871767       193871986       0       1       -14.0   0.786448090545  2.38370598808e-07       18300125        0       22      11      14      16      193871940       193872076
        #if int(tmp[10])==0:

        num_of_discor=int(tmp[13])
        #if float(tmp[0])<0.8 or int(tmp[15])==-1:# or (int(tmp[0])+int(tmp[11])<2*MINMATEPAIRS):
        if num_of_discor<MINMATEPAIRS or int(tmp[15])==-1:# or (int(tmp[0])+int(tmp[11])<2*MINMATEPAIRS):
            #print 'skip', float(tmp[0]), tmp[15] 
            line=FILE.readline()
            continue

        #else:
        #    print '**************************************** ' , tmp    
        std_dev=13/(num_of_discor**0.5)
	err_range=std_dev*2
        start_pos=int(tmp[2])
        end_pos=int(tmp[3])
	mu=float(tmp[6])
	phet=float(tmp[7])
	pval=float(tmp[8])
	clusid=tmp[9]
	membership=tmp[10]
        chk_key=clusid+'_'+membership
	clusters_info[chk_key]=[pval, num_of_discor, start_pos, end_pos, mu, phet, clusid, membership, tmp[11], tmp[12], tmp[13], tmp[14], float(tmp[0])]
        cluster_ptr_info[chk_key]=tmp


        #if chk_key=='500369_0':
        #    print pval, clusid, membership
        ######################################3
        ## contig break
        cb_flag=0
        for cb in cb_dic[chr]:
            if max(cb[0],start_pos)<=min(cb[1],end_pos):
                cb_flag=1
                break
        if cb_flag==1:
            print 'contig break'
            line=FILE.readline()
            continue

	#print line, clusid, db['0']

	if progress % 200000==0:
            print type, 'progress: ', progress, clusid
	progress+=1

	##############################################
	## minus pvalue means it's nothing 
 	if pval<0 or pval>0.001:
            line=FILE.readline()
	    continue


        mu_range=[mu-err_range, mu+err_range]
        #t_list=db[str(clusid)].split()
	#t_set.clear()
	#assign_template_id=0
	#for t in t_list:
	#    if t=='0' or t=='1': ## membership flag
	#        if t==membership:
	#            assign_template_id=1
	#	else:
	#            assign_template_id=0
	#    elif assign_template_id==1:
        #        t_set.add(t)
	


	add_success=0
        for op in open_list: 
	    pre_start_pos=op[0]
	    pre_end_pos=op[1]
	    pre_mu=op[2]
	    pre_err_range=op[3]
	    pre_pval=op[4]
	    pre_clusid=op[5]
	    pre_membership=op[6]

            pre_mu_range=[pre_mu-pre_err_range, pre_mu+pre_err_range]
	    cond1=min(mu_range[1], pre_mu_range[1])-max(mu_range[0], pre_mu_range[0]) # size
	    cond2=min(end_pos, pre_end_pos)-max(start_pos, pre_start_pos) # location
            cond3=1

            ## rule base 1
            if abs(pre_start_pos-start_pos)<10 and abs(pre_end_pos-end_pos)<10 and abs(mu-pre_mu)<10:
                add_success=10**10
                break
            ## rule base 2
            if pre_start_pos>=start_pos and pre_end_pos<=end_pos and abs(mu-pre_mu)<10:
                add_success=10**10
                break
            ## rule base 3
            if pre_start_pos<=start_pos and pre_end_pos>=end_pos and abs(mu-pre_mu)<10:
                add_success=10**10
                break
            ## rule base 4
            if (pre_start_pos==start_pos or pre_end_pos==end_pos) and abs(mu-pre_mu)<10:
                add_success=10**10
                break

	    if (type%2==1 and cond1>=0 and cond2>=max(mu, pre_mu)) or (type%2==0 and cond1>=0 and cond2>=0): ## merge
                add_success+=1
   
        #if clusid=='11015928':
        #    print '>>>     clusid: ', clusid, open_list 
        #    print 

        if clusid=='11015929' or '11015929' in open_list:
            if '11015928' in open_list:
                print 
                print '>>>   clusid', clusid, '  >>> OK '
                print 
            else:
                print 
                print '>>>   clusid', clusid, '  >>> noup '
                print 
            #print 'decision value:', add_success/denominator

	if len(clus_set)!=0 and add_success/len(open_list)<1: ## disconnect 
            if ('11015928_0' in clus_set):
                print '-------------------------- cut AA', clusid

            if ('11015929_0' in clus_set):
                print '-------------------------- cut BB', clusid

	    clusters_list.append(copy.copy(clus_set))
	    clus_set.clear()
            open_list=[]

 	open_list.append([start_pos, end_pos, mu, err_range, pval, clusid, membership])
	clus_set.add(clusid+'_'+membership)

        line=FILE.readline()

    
    #if ('11015928' in open_list) or ('11015929' in open_list):
    #    print '-------------------------- cut', open_list
    FILE.close()
    clusters_list.append(copy.copy(clus_set)) ## left over
    #db.close()

    #print '*** choose unique indel from clusters **' 
    #print clusters_list
    good_clus_list=[]
    for clus in clusters_list:
	best_stat=[100, 0]
	best_clusid='0000'
	best_key=''
        best_mu=0
        best_start_pos=10**10
        best_likeli=-1
	for clus_mem in clus: 
            cm=clus_mem.split('_')
            clusid=cm[0]
            membership=cm[1]

	    info=clusters_info[clus_mem]
	    pval=info[0]
	    num=info[1]
	    start_pos=info[2]
            mu=abs(info[4])
            likeli=info[12]

            update_f=0
	    if (best_stat[0]>pval):# or (best_stat[0]==pval and best_stat[1]<=num):
	    #if best_mu<mu or (best_mu==mu and best_stat[1]<num):
                update_f=1
            elif best_stat[0]==pval and best_stat[1]<num: 
                update_f=1
            elif best_stat[0]==pval and best_stat[1]==num and best_mu<mu:
                update_f=1
            
            if update_f==1:
                best_mu=mu
		best_stat=[pval, num]
	  	best_clusid=clusid
                best_key=clusid+'_'+membership
                best_start_pos=start_pos
        #if int(best_start_pos/1000)==219695:
	#    print 
	#    print clus
	#    print '** ', best_clusid
	if best_clusid!='0000':
	    good_clus_list.append([best_start_pos, best_key])
		    
    #print '*** WRITE final RESULTS'
    good_clus_list.sort()
    #fn_list=output_fn[0].split('/')
    #fn_last=fn_list[len(fn_list)-1]

    #print 'ouput file', output_fn[0]+'__tmp.gz'

    #10      chr1    93773839        93774051        0       1       -8.0    0.46944585637   0.0212648844296 9404589 0       11      8       8       -1

	#clusters_info[chk_key]=[pval, int(tmp[0]), start_pos, end_pos, mu, phet, clusid, membership]
    output_fn=input_fn.split('__') 
    dst_f='%s__final' % output_fn[0]
    FILE_out=open(dst_f,'w')

    for good_clus in good_clus_list:
        ky=good_clus[1] 
        str_p=''
        for ptr in cluster_ptr_info[ky]:
            str_p+=ptr+'\t'
        str_p+='\n'
        FILE_out.write(str_p)
            
        #pval=clusters_info[ky][0]
        #cardi=clusters_info[ky][1]
        #start_pos=clusters_info[ky][2]
        #end_pos=clusters_info[ky][3]
        #mu=clusters_info[ky][4]
        #phet=clusters_info[ky][5]
        #clusid=clusters_info[ky][6]
        #mem=clusters_info[ky][7]

        #str_p=str(cardi)+'\t'
        #str_p+=chr+'\t'
        #str_p+=str(start_pos)+'\t'+str(end_pos)+'\t'
        #if mu>0:
        #    str_p+='1\t0\t'
        #else:
        #    str_p+='0\t1\t'
        #str_p+=str(mu)+'\t'
        #str_p+=str(phet)+'\t'
        #str_p+=str(pval)+'\t'
        #str_p+=clusid+'\t'
        #str_p+=str(mem)+'\t'
        #str_p+=str(clusters_info[ky][8])+'\t'
        #str_p+=str(clusters_info[ky][9])+'\t'
        #str_p+=str(clusters_info[ky][10])+'\t'
        #str_p+=str(clusters_info[ky][11])+'\n'
        
    FILE_out.close()

    print 'tot num of clusters:', len(good_clus_list)
    return len(good_clus_list)

    #FILE_p = open(input_fn, 'r')
    #totnum=0
    #final_lines=[]
    #line=FILE_p.readline()
    #while line: 
#	tmp=line.split()
#
#        if int(tmp[0])<=MINMATEPAIRS or int(tmp[14])==-1:# or (int(tmp[0])+int(tmp[11])<2*MINMATEPAIRS):
#            #print 'write, skip', tmp[14] 
#            line=FILE_p.readline()
#            continue

#        start_pos=int(tmp[2])
#        end_pos=int(tmp[3])
#	pval=float(tmp[8])
#	clusid=tmp[9]
#	membership=tmp[10]
#        chk_key=clusid+'_'+membership
#
#
#        ######################################3
#        ## contig break
#        cb_flag=0
#        for cb in cb_dic[chr]:
#            if max(cb[0],start_pos)<=min(cb[1],end_pos):
#                cb_flag=1
#                break
#        if cb_flag==1:
#            line=FILE_p.readline()
#            continue
#
# 	if pval<0 or pval>=0.01:
#            line=FILE_p.readline()
#	    continue
#
#        #if chk_key=='500369_0':
#        #    print pval, clusid, membership
#
#        if chk_key in good_clus_set:
#            final_lines.append(line)
#            totnum+=1
#            if pval<0: 
#                print 'minus -------------------- ', pval, chk_key, membership
#        line=FILE_p.readline()
#    FILE_p.close()
#
#


    #tmp_fname='%s/%s/tmp__%s' % (post_dir,chr,fn_last)
#    output_fn=input_fn.split('__') 
#    dst_f='%s__final' % output_fn[0]
#    FILE_out=open(dst_f,'w')
#    for li in final_lines:
#        FILE_out.write(li)
#    FILE_out.close()
    
    ###################################
    ## delete dbm file in temporary directory
    #command='ls /tmp|grep template_db_%s.dbm|awk \'{print "rm /tmp/"$1}\'|sh' % (chr)
    #print command
    #os.system(command)

          
    #command='cd %s/%s; ls |grep %s__final|awk \'{print "rm "$1}\'|sh' % (post_dir, chr, fn_last)
    #print command
    #os.system(command)


    ###########################################
    ## rename    
    #try:
    #    os.remove(dst_f)
    #except OSError, detail: # file not found
    #    if detail.errno != 2:
    #        print 'catch exception'
    #        raise
#
#    src_f=tmp_fname
#    print '1 source:', src_f, 'target:', dst_f
#    while 1:
#        if not os.path.exists(dst_f):
#            print "exist?", os.path.exists(dst_f)
#            print '2 source:', src_f, 'target:', dst_f
#            os.rename(src_f, dst_f)
#            break

    ###############################################
    ## cleaning sorted file
    #command='cd %s/%s; ls | grep %s__final.sorted |awk \'{print "rm "$1}\' | sh' % (post_dir, chr, fn_last)
    #os.system(command)

def main(argv):
    #handle_command_line_args(argv)
    initialize_global_vars()
    target_chr='chr1'

    indl_type = int(sys.argv[1])
    fn = sys.argv[2]
    chr = sys.argv[3]

    #######################################33
    ## 4. merge duplicated indel calls 

    #merge_results(indl_type,fn,chr)
    merge_results_until_convergence(indl_type,fn,chr)

if __name__=="__main__":
    main(sys.argv[1:])
