#!/usr/bin/env python

from __future__ import division
import os
import sys
import datetime
import logging
import math
import MPProperties
from sets import Set
import random
import gzip
import copy

# logger configuration

alpha={}
getprob={}
coverage_list={}
#---------
db = None
search=None
search2=None
props = {}
chr_list=['chr1','chr2', 'chr3','chr4','chr5','chr6','chr7','chr8','chr9','chr10','chr11','chr12','chr13','chr14','chr15','chr16','chr17','chr18','chr19','chr20', 'chr21','chr22','chrX','chrY']

probdist={}
probdist_keys={}
mean_list={}
std_list={}

def initialize_global_vars():
    # set the properties hash
    global props
    # read the properties hash
    props = MPProperties.read_config()


def get_sys_command_result(command):
    result = []
    command_result_pipe = os.popen(command)
    while True:
        line = command_result_pipe.readline()
        if not line:
            break
        line = line.strip()
        result.append(line)
    command_result_pipe.close()
    return result

#---------

def print_message_and_current_time(message):
    curr_time = datetime.datetime.now().ctime()
    curr_time_msg = message + curr_time
    
    # log the time message
    log.info(curr_time_msg)
    
    # print message and current time
    print curr_time_msg
    print

#---------

def return_list_from_result_set_column(mysqldb_result_set, column_index):
    result = []
    for result_set_row in mysqldb_result_set:
        result.append(result_set_row[column_index])
    return result

#---------

def return_str_list_from_list(list):
    result = []
    for item in list:
        result.append(str(item))
    return result

#---------    

#53301_1_72/2    chr1    4404    -       -221    18      99      82      82      0       0       1       0       36      aagccagccttccgctccttgaagctggtctccaca88;;;;;;;7;;;;;;;;;;;;3;;;;;;;;;;;;;
def create_formatted_maq_hom_results():
    maq_dir='/home/brudno/genome/fastq'
    maq_fn=maq_dir+'/high_coverage.all.hom.indelpe_'
    #maq_fn=maq_dir+'/all.hom.indelpe'
    
    maq_res_fn=maq_dir+'/high_coverage_maq_hom_results.txt_'
    FILE_out=open(maq_res_fn,'w')
    FILE=open(maq_fn, 'r')
    line=FILE.readline()
    progress=0
    while line:
	tmp=line.split()
	chrom=tmp[0]
	start=int(tmp[1])
	end=int(tmp[1])+abs(int(tmp[2]))

        str_out='template_id'+'\t'
        str_out+=chrom+'\t'
        str_out+=str(start)+'\t'
        str_out+=str(end)+'\t'
        if int(tmp[2])>0:
            str_out+='1\t0\t'
        elif int(tmp[2])<0:
	    str_out+='0\t1\t'
	else:
            line=FILE.readline()
	    continue
	    
        str_out+=str(int(tmp[2]))+'\t'
        str_out+='0\t0\t0\n'

        FILE_out.write(str_out)
        line=FILE.readline()
    FILE.close()
    FILE_out.close()

#chr1    881930  +       8       1:G     0       2       GAGCCGGCTAGGACAGTGCCGGAAACTCAGCT        GCCTCTCCCCTTCAACTCAGCTGGCCCCCCGC        6       @31,28,26,17,8,4,       @5,8,10,19,28,32,       @0,0,0,0,1,0,

def create_formatted_maq_results():
    #maq_dir='/home/brudno/genome_additional/fastq'
    maq_dir='/home/brudno/genome/fastq'
    #maq_fn=maq_dir+'/all.indelpe'
    maq_fn=maq_dir+'/high_coverage.all.indelpe'

    #maq_res_fn=maq_dir+'/maq_results.txt'
    maq_res_fn=maq_dir+'/high_coverage_maq_results.txt'
    FILE_out=open(maq_res_fn,'w')

    #indel_list=[]
    FILE=open(maq_fn, 'r')
    line=FILE.readline()
    progress=0
    while line:
        #if progress % 5000 == 0:
        #    print 'progress :', progress
        #progress+=1
        tmp=line.split()


        chr_event=tmp[0]
        start_event=int(tmp[1])
	confi=tmp[2]
	if confi=='-' or confi=='.':
            line=FILE.readline()
	    continue
	seq_=tmp[4]	
	seq=seq_.split(':')
	end_event=start_event+abs(int(seq[0]))-1

        str_out='template_id'+'\t'
        str_out+=chr_event+'\t'
        str_out+=str(start_event)+'\t'
        str_out+=str(end_event)+'\t'
        if int(seq[0])>0:
            str_out+='1\t0\t'
        elif int(seq[0])<0:
	    str_out+='0\t1\t'
        str_out+=str(seq[0])+'\t'
        str_out+='0\t0\t0\n'

        FILE_out.write(str_out)
	#print str_out
        line=FILE.readline()
    FILE.close()
    FILE_out.close()

def generate_heatmap_code():
    FILE=open('./modil_heatmap2.txt','r')
    FILE_out=open('./modil_heatmap2_matlab.m','w')
    line=FILE.readline()
    x_axis='x=['
    y_axis='y=['
    while line:
	tmp=line.split()
	indel_size=abs(int(tmp[0]))
	clus_size=abs(int(tmp[1]))
	cardi=round(float(tmp[2])*20)
	#print tmp
	for i in range(cardi):
	    x_axis+=str(clus_size)+','
	    y_axis+=str(indel_size)+','
	
	print indel_size, clus_size, cardi
        line=FILE.readline()

    x_axis=x_axis[0:len(x_axis)-1]
    y_axis=y_axis[0:len(y_axis)-1]
    x_axis+='];\n'
    y_axis+='];\n'

    #print x_axis
    #print y_axis

    FILE_out.write(x_axis)
    FILE_out.write(y_axis)
	
    FILE.close()	
    FILE_out.close()	


#
#def make_template_id_file(chr,full_fname,fcnt):
#    result_dir=props["RESULTS_DIR"]
#    result_dir+="POST_PROCESSING"
    ## clusid template id list
    ## aaa 1 2 3 4 5 6 7 8 9 0
#    fname_tmp=full_fname.split('/')
#    fname=fname_tmp[len(fname_tmp)-1]

#    f_list=[full_fname]
#    print fname
    #['211', '34981_1_107', 'chr1', '3976', '4115', '-2.0', 'SIMUL_34981_1_107', '-1', '10000000000', '1', '1', '1', '-7', '0', '0.233689455073', '0.0818942602793']
#    FILE_out=gzip.open(result_dir+'/'+chr+'/template_id_'+fname, 'wb')
#    key_set=Set()
#    for f in f_list:
#	key_set.clear()
#	print 'processing...', f
#        FILE=gzip.open(f,'rb') 
#	line=FILE.readline()
#	clusid_pre=''
#	tmp_cluster_list=[[],[]]
#	template_id_list=[[],[]]
#	while line:
#	    tmp=line.split()
#	    clusid=tmp[0]
#	    template_id=tmp[6]
#	    membership=int(tmp[9])
#	
#	    if clusid_pre!=clusid and (len(template_id_list[0])+len(template_id_list[1]))>1:
#		print_str=clusid_pre+'\t'
#	        for member in [0,1]:
#		    print_str+=str(member)+'\t'
#		    for tid in template_id_list[member]:
#		        print_str+=tid+'\t'
#	        print_str+='\n'
#		FILE_out.write(print_str)
#		template_id_list=[[],[]]
#
#	    template_id_list[membership].append(template_id)		
#	    clusid_pre=clusid
#	    line=FILE.readline()
#
#	FILE.close()
#    FILE_out.close() 
#



#########################################################33
## extract results
def extract_results(chr,full_fname): 
    #command='ls %s/%s | grep results' % (inference_dir,chr)
    length_of_read=int(props["READ_LENGTH"])
    result_dir=props["RESULTS_DIR"]
    result_dir+="/POST_PROCESSING"
    std_dev=int(props["STD_INSERT_SIZE"])

    likeli_thres=0.7

    fname_tmp=full_fname.split('/')
    fname=fname_tmp[len(fname_tmp)-1]

    f_list=[full_fname]
    FILE_out=gzip.open(result_dir+'/'+chr+'/filtered_'+fname, 'wb')
    key_set=Set()
    for f in f_list:
	key_set.clear()
	print 'processing...', f
        FILE=gzip.open(f,'rb') 
	line=FILE.readline()
	clusid_pre=''
	tmp_cluster_list=[[],[]]
	while line:
	    tmp=line.split()
	    clusid=tmp[0]
	    mapid=tmp[1]
	    chrom=tmp[2]
	    start_pos=int(tmp[3])-length_of_read+1
	    end_pos=int(tmp[4])+length_of_read+1
	    ndist=float(tmp[5]) 
	    template_id=tmp[6]
	
	    clus_start_pos=int(tmp[7])
	    clus_end_pos=int(tmp[8])
	    membership=int(tmp[9])
	    pval=[float(tmp[10]), float(tmp[11])]
            mu=[float(tmp[12]), float(tmp[13])]	
	    phat=float(tmp[14])
	    score=float(tmp[15])
          

	    if clusid_pre!=clusid:
	        phat=compute_phet(tmp_cluster_list)
		for allel in [0,1]:
                    #################################
                    ## count discordanat matepairs
                    if len(tmp_cluster_list[allel])<2:
                        continue
                    mu_allel=tmp_cluster_list[allel][0][3]
                    if mu_allel==0:
                        continue

                    likeli=likelihood_of_indel(tmp_cluster_list[allel])
                    #if likeli<likeli_thres:
                    #    print 'likeli: ', likeli, mu_allel
                    #    continue

                    discor_num=[0,0]
                    std_dev=std_dev/(len(tmp_cluster_list[allel])**0.5)
                    region_estimate_list=[]
                    #for x in [0,1]:
                    for discor in tmp_cluster_list[allel]:
                        if discor[2]*mu_allel>0:
                            if abs(discor[2])>=abs(mu_allel):
                                region_estimate_list.append(discor)
                                discor_num[0]+=1

                            if abs(discor[2]-mu_allel)<=max(std_dev*2,10):
                                discor_num[1]+=1

		    #print 'TYPE: ', allel
                    #[loc,clus_cardi]=estimate_location(tmp_cluster_list[allel])
                    [loc,clus_cardi]=estimate_location(region_estimate_list)
		    #print allel, loc, clusid
		    if clus_cardi==0:
		        continue

		    #print_str=str(len(tmp_cluster_list[allel]))+'\t'+chr+'\t'
		    print_str=str(likeli)+'\t'+chr+'\t'
		    #print_str=str(clus_cardi)+'\tchr1\t'
		    print_str+=str(loc[0])+'\t'+str(loc[1])+'\t'
		    if mu_allel>0:
		        print_str+='1\t0\t'
		    else:
		        print_str+='0\t1\t'
		    print_str+=str(mu_allel)+'\t'
		    print_str+=str(phat)+'\t' ## phat, greater -> hetero
		    print_str+=str(tmp_cluster_list[allel][0][5])+'\t' ## pval
	
		    print_str+=clusid_pre+'\t'
		    print_str+=str(allel)+'\t'
                    print_str+=str(len(tmp_cluster_list[0]))+'\t'
                    print_str+=str(len(tmp_cluster_list[1]))+'\t'
                    print_str+=str(discor_num[0])+'\t'
		    print_str+=str(discor_num[1])+'\t'
                    print_str+=str(tmp_cluster_list[allel][0][7])+'\t'
                    print_str+=str(tmp_cluster_list[allel][0][8])+'\n'
			
	            FILE_out.write(print_str)

	        tmp_cluster_list=[[],[]] ## initialize

            tmp_cluster_list[membership].append([start_pos, end_pos, ndist, mu[membership], mapid, pval[membership], phat, clus_start_pos, clus_end_pos])
            clusid_pre=clusid

	    #print clus_start_pos, clus_end_pos, mu, membership, uniq_key
	    ## write

	    line=FILE.readline()
	#########################
        ## for the last one
        phat=compute_phet(tmp_cluster_list)
        for allel in [0,1]:
            if len(tmp_cluster_list[allel])<2:
                continue
            mu_allel=tmp_cluster_list[allel][0][3]
            if mu_allel==0:
                continue

            likeli=likelihood_of_indel(tmp_cluster_list[allel])

            discor_num=[0,0]
            std_dev=std_dev/(len(tmp_cluster_list[allel])**0.5)
            region_estimate_list=[]
            for discor in tmp_cluster_list[allel]:
                if discor[2]*mu_allel>0:
                    if abs(discor[2])>=abs(mu_allel):
                        region_estimate_list.append(discor)
                        discor_num[0]+=1
                    if abs(discor[2]-mu_allel)<=max(std_dev*2,10):
                        discor_num[1]+=1

            [loc,clus_cardi]=estimate_location(region_estimate_list)
            if clus_cardi==0:
                continue
            print_str=str(likeli)+'\t'+chr+'\t'
            print_str+=str(loc[0])+'\t'+str(loc[1])+'\t'
            if mu_allel>0:
                print_str+='1\t0\t'
            else:
                print_str+='0\t1\t'
            print_str+=str(mu_allel)+'\t'
            print_str+=str(phat)+'\t' ## phat, greater -> hetero
            print_str+=str(tmp_cluster_list[allel][0][5])+'\t' ## pval
            print_str+=clusid_pre+'\t'
            print_str+=str(allel)+'\t'
            print_str+=str(len(tmp_cluster_list[0]))+'\t'
            print_str+=str(len(tmp_cluster_list[1]))+'\t'
            print_str+=str(discor_num[0])+'\t'
            print_str+=str(discor_num[1])+'\t'
            print_str+=str(tmp_cluster_list[allel][0][7])+'\t'
            print_str+=str(tmp_cluster_list[allel][0][8])+'\n'
            FILE_out.write(print_str)

        FILE.close()
    FILE_out.close()

#############################
## lieklihood of indels take>0.8
## outliers are computed for both so don't need special care

def likelihood_of_indel(cluster_list):
    std_dev=int(props["STD_INSERT_SIZE"])
    mean_insert=int(props["MEAN_INSERT_SIZE"])

    if len(cluster_list)<2:
        return 0 # hom

    insert_size_list=[]
    insert_size_list_fixed=[]
    for clus in cluster_list:
        ins_size=mean_insert-clus[2]
        insert_size_list.append(int(ins_size))
        insert_size_list_fixed.append(int(ins_size+clus[3]))
                

    #######################
    ## compute likelihood with no indel
    log_likeli_no=0
    count=[0,0]
    for x in insert_size_list:
        if x in probdist_keys[mean_insert]:
            log_likeli_no+=math.log(probdist[mean_insert][x])
            count[0]+=1
        else:
            log_likeli_no+=math.log(10**-7)

    log_likeli_indel=0
    for x in insert_size_list_fixed:
        if x in probdist_keys[mean_insert]:
            log_likeli_indel+=math.log(probdist[mean_insert][x])
            count[1]+=1
        else:
            log_likeli_indel+=math.log(10**-7)

    likeli_no=math.exp(log_likeli_no)
    likeli_indel=math.exp(log_likeli_indel)
    
    denomi=likeli_indel+likeli_no
    #print count, insert_size_list, insert_size_list_fixed
    if denomi!=0:
        return likeli_indel/denomi
    else:
        return 0


def compute_phet(cluster_list):
    ###################
    ## find sigma
    for i in [0,1]:
	if len(cluster_list[i])<2:
	    return 0 # hom

    tot_clus_num=[len(cluster_list[0]),len(cluster_list[1])]
    std_dev=int(props["STD_INSERT_SIZE"])
    mean_insert=int(props["MEAN_INSERT_SIZE"])
    #std_dev=13#/((sum(tot_clus_num)/2)**0.5)
    
    mu_=[cluster_list[0][0][3], cluster_list[1][0][3]]
    if mu_[0]!=0 and mu_[1]!=0: # type 3
        return 0.5 # cannot determine
 
     
    ###############################################
    ## 1. compute p(x1,.., xn|hom) and p(x1,.., xn|het)
   
    # make list of mapped distance
    x_list=[]
    tmp=[]
    for clus in cluster_list[0]:
	if (abs(mu_[0]-clus[2])<=2*std_dev) or (abs(mu_[1]-clus[2])<=2*std_dev):
            x_list.append(mean_insert-clus[2])
	else:
	    tmp.append(clus)
    #print len(x_list)
    for clus in cluster_list[1]:
	if (abs(mu_[0]-clus[2])<=2*std_dev) or (abs(mu_[1]-clus[2])<=2*std_dev):
            x_list.append(mean_insert-clus[2])
	else:
	    tmp.append(clus)
    #print len(x_list), x_list
    signal_level=len(x_list)/sum(tot_clus_num)

    mu=[0,0]
    if mu_[0]!=0:
        mu=[mu_[0], mu_[1]]
	prior_het=tot_clus_num[0]/sum(tot_clus_num)
    else:
        mu=[mu_[1], mu_[0]] # first mu is not 0
	prior_het=tot_clus_num[1]/sum(tot_clus_num)

    prior=[1/2,1/2] # hom, heterozygous
    
    # compute p(x1,.., xn|hom)
    p_hom=math.log(prior[0])
    p_het=math.log(prior[1])
    for x in x_list:
        expected_insert_size=int(x+mu[0])
        if expected_insert_size in probdist_keys[mean_insert]:
            p_hom+=math.log(probdist[mean_insert][expected_insert_size])
	else:
            p_hom+=math.log(10**-5)

	allel_1=0
	allel_2=0
        if expected_insert_size in probdist_keys[mean_insert]:
            allel_1=probdist[mean_insert][expected_insert_size]*prior_het
	else:
            allel_1=(10**-5)*prior_het

        expected_insert_size=int(x+mu[1])
        if expected_insert_size in probdist_keys[mean_insert]:
            allel_2=probdist[mean_insert][expected_insert_size]*(1-prior_het)
	else:
            allel_2=(10**-5)*(1-prior_het)
	p_het+=math.log(allel_1+allel_2)

    additive=0
    p_hom=math.exp(p_hom+additive)
    p_het=math.exp(p_het+additive)
    #print 'log likeli:', p_hom+additive, p_het+additive
     
    if p_hom+p_het==0:
        return 0.5 # cannot determine
    else:
        likelihood_of_het=p_het/(p_hom+p_het)
	#if likelihood_of_het<0.3 or likelihood_of_het>0.7:
        #    print likelihood_of_het, prior_het, mu, x_list
	
        #print 'x_list:', x_list
        #print 'mu: ', mu
        #print '---'
	if signal_level<0.1:
	    #print signal_level
	    #print cluster_list, mu_
	    #print tmp
            return -likelihood_of_het
	else:
            return likelihood_of_het




def compute_stat(list):
    #print list
    std=0
    for x in list:
	span=x[0] 
	mu=x[1]
	std+=(span-mu)**2
    std/=len(list)
    return std

##########################################
## based on probability
def estimate_location(cluster_list_pre):
    if len(cluster_list_pre)<2 or cluster_list_pre[0][3]==0: ## noindel
	return [[-1,-1], 0]

    if cluster_list_pre[0][3]>0:
	indl_type=0 # insertion
    else:
	indl_type=1 # deletion
    
    std_dev=int(props["STD_INSERT_SIZE"]) #/(len(cluster_list_pre)**0.5)
    cluster_list=[]
    for tmp in cluster_list_pre:
        mu=float(tmp[3])
        mp_span=int(tmp[2])
        if abs(mp_span-mu)<=std_dev*2:
            cluster_list.append(tmp)
    #print 'pre: ', cluster_list_pre
    #print 'aft: ', cluster_list

    if len(cluster_list)<2: ## statistically insignificant
	return [[-1,-1],0]

    #tmp_cluster_list[membership].append([start_pos, end_pos, ndist, mu[membership], mapid, pval[membership], phat])
    region=[10**10,-1]

    ###################################
    ## 1. find possible range
    tmp_list=[]
    mu_win=0
    for tmp in cluster_list:
	mu_win=int(float(tmp[3]))
        start_pos=int(tmp[0])
        end_pos=int(tmp[1])
	region[0]=min(region[0], start_pos)
	region[1]=max(region[1], end_pos)

    #############################################
    ## 2. compute  statistics for each position   
    stat_dic={}
    for bp in range(region[0], region[1]+1):
	mu_list=[]
	for tmp in cluster_list:
            start_pos=int(tmp[0])
            end_pos=int(tmp[1])
            mu=float(tmp[3])
            span=int(tmp[2])
	    if start_pos<=bp and end_pos>=bp: ## overlap?
		mu_list.append([span,mu])
 	stat=compute_stat(mu_list) # compute std from mu		
	stat_dic[bp]=stat
	

    ####################################
    ## 3. take the best position
    possible_ranges=[]
    for tmp in cluster_list:
        start_pos=int(tmp[0])
	for tmp in cluster_list: 
            end_pos=int(tmp[1])
	    if start_pos<=end_pos:
	        possible_ranges.append([start_pos,end_pos])

    choose_best=[]
    for candi_range in possible_ranges:  
	if indl_type==0:
	    candi_stat=0
	    for pt in range(candi_range[0], candi_range[1]+1):
	        candi_stat+=stat_dic[pt]
	    candi_stat/=(candi_range[1]-candi_range[0]+1.0)
	    choose_best.append([candi_stat, candi_range])
	else:
	    candi_stat=0
	    if candi_range[1]-candi_range[0]+1<mu_win:
	        choose_best.append([10**10, [-1,-1]]) ## invalid
		continue

	    for pt in range(candi_range[0], candi_range[1]+1):
	        candi_stat+=stat_dic[pt]
	    candi_stat/=(candi_range[1]-candi_range[0]+1.0)
	    choose_best.append([candi_stat, candi_range])

    choose_best.sort()
    return [choose_best[0][1], len(cluster_list_pre)]

	    
def main(argv):
    chr = sys.argv[1]
    fname = sys.argv[2]

    #handle_command_line_args(argv)
    initialize_global_vars()
    exe_dir=props["EXE_DIR"]
    mean_insert=int(props["MEAN_INSERT_SIZE"])
    std_insert=int(props["STD_INSERT_SIZE"])

    probdist[mean_insert]={}
    FILE_l=open(exe_dir+"/data/illumina_PofY.txt","r")
    illu_line=FILE_l.readline()
    while illu_line:
        tmp=illu_line.split()
        probdist[mean_insert][int(tmp[0])]=float(tmp[1])
        illu_line=FILE_l.readline()

    FILE_l.close()
    mean_list[mean_insert]=mean_insert
    std_list[mean_insert]=std_insert

    for ins_size in mean_list.keys():
        probdist_keys[ins_size]=probdist[ins_size].keys()
        probdist_keys[ins_size].sort()
    #for insert_size in mean_list.keys():
    #    if insert_size==208:
    #        keys= probdist[insert_size].keys()
    #        keys.sort()
    #        n=0
            #for k in keys:
            #    print k, probdist[insert_size][k]



    ################################
    ## extract results
    extract_results(chr,fname)


if __name__=="__main__":
    main(sys.argv[1:])
