#!/usr/bin/env python

import os
import sys
import datetime
import MPProperties
import MoDIL_simple
from sets import Set
import random

db = None

chr_list=['chr1','chr2', 'chr3','chr4','chr5','chr6','chr7','chr8','chr9','chr10','chr11','chr12','chr13','chr14','chr15','chr16','chr17','chr18','chr19','chr20', 'chr21','chr22','chrX','chrY']
target_chr='chr1'
#---------
probdist = {}
mean_list = {}
std_list = {}
probdist_keys = {}

# handle command-line args
def initialize_global_vars():
    # set the properties hash
    global props
    # read the properties hash
    props = MPProperties.read_config()

#---------
def get_sys_command_result(command):
    result = []
    command_result_pipe = os.popen(command)
    while True:
        line = command_result_pipe.readline()
        if not line:
            break
        line = line.strip()
        result.append(line)
    command_result_pipe.close()
    return result

def gen_dir():
    print '1. generate directories'

    DATA_DIR = props["DATA_DIR_USER"]

    command='mkdir %s' % (DATA_DIR)
    print command
    os.system(command)

    dir_list=['INTERESTING_DB', 'INTERESTING_SIMULATION_DB','DATA','SIMUL_DATA']
    for dir in dir_list:
        command='mkdir %s/%s' % (DATA_DIR, dir)
        print command
        os.system(command)

    dir_list=['POST_PROCESSING','INFERENCE_RESULTS','CLUSTERING_RESULTS','tmp']
    for dir in dir_list:
        command='mkdir %s/DATA/%s' % (DATA_DIR, dir)
        os.system(command)
        command='mkdir %s/SIMUL_DATA/%s' % (DATA_DIR, dir)
        os.system(command)

    for dir in chr_list:
        command='mkdir %s/DATA/INFERENCE_RESULTS/%s' % (DATA_DIR, dir)
        os.system(command)
        command='mkdir %s/DATA/CLUSTERING_RESULTS/%s' % (DATA_DIR, dir)
        os.system(command)
        command='mkdir %s/DATA/POST_PROCESSING/%s' % (DATA_DIR, dir)
        os.system(command)

        command='mkdir %s/SIMUL_DATA/INFERENCE_RESULTS/%s' % (DATA_DIR, dir)
        os.system(command)
        command='mkdir %s/SIMUL_DATA/CLUSTERING_RESULTS/%s' % (DATA_DIR, dir)
        os.system(command)
        command='mkdir %s/SIMUL_DATA/POST_PROCESSING/%s' % (DATA_DIR, dir)
        os.system(command)

def gen_demo_files():
    initialize_global_vars()

    EXE_DIR = props["EXE_DIR"]
    DATA_DIR = props["DATA_DIR"]
    g_dir=DATA_DIR+'/SIMUL_DATA/tmp'
    simul_dir=DATA_DIR+'/INTERESTING_SIMULATION_DB'

    #simul_data='./data/simul_indels.txt'
    #FILE=open(simul_data,'r')
    #line=FILE.readline()
    #while line:
    #    tmp=line.split()
    #    print tmp
    #    line=FILE.readline()
    #FILE.close()
    
    simul.create_simul_donor_genome(0)
    simul.create_simul_donor_genome(1)
    simul.make_no_new_line_file()

    simul.generate_chain(0)
    simul.generate_chain(1)

    ########
    print 'generate matepairs'
    
    simul.generate_matepairs(0,30,0)  # 40x coverage
    simul.generate_matepairs(1,30,0)

    #simul.reverse_complement_matepairs() 

    simul.input_simul_db(0,0)
    simul.input_simul_db(1,0)

    #########################################
    print 'merge interesting files  & sort'
    command='cat %s/%s.0.* %s/%s.1.* > %s/%s_' % (simul_dir,target_chr,simul_dir,target_chr,simul_dir,target_chr)
    get_sys_command_result(command)
    # sorting interesting file
    command='sort -S 1G -k 1n,1 -k 2n,2 %s/%s_>%s/%s; rm %s/%s_' % (simul_dir,target_chr,simul_dir,target_chr,simul_dir,target_chr)
    get_sys_command_result(command)


def gen_simul_indel():
    FILE=open('./data/simul_indels.txt','w')
    FILE.write('chromosome\tstart_pos\tend_pos\ttype\theterozygosity\tsequence')
    
    num_of_indels=1000
    from_pos=10
    for i in range(num_of_indels):
        size_of_indel=random.randrange(20,1000)
        SEQ=''
        for j in range(size_of_indel):
            SEQ+='A'

        start_pos=random.randrange(from_pos, from_pos+4000)
        end_pos=start_pos+size_of_indel-1
        from_pos+=4010 

        if random.random()>0.5:
            line='chr1\t'+str(start_pos)+'\t'+str(end_pos)+'\tINS'
        else:
            line='chr1\t'+str(start_pos)+'\t'+str(end_pos)+'\tDEL'

        if random.random()>0.5:
            line+='\tHOM\t'+SEQ+'\n'
        else:
            line+='\tHET\t'+SEQ+'\n'
            
        FILE.write(line)
    FILE.close()        

def gen_data_files():

    print "simulation to find thresholds for filters"
    MoDIL_simple.montecarlo_run()


def fast_Kolm_montecarlo(sample_size):
    MEAN_INSERT_SIZE = int(props["MEAN_INSERT_SIZE"])
    ALL_DATA_SOURCES_INSERT_SIZE=[MEAN_INSERT_SIZE]
    KOLM_DIST={}
    KOLM_set={}
    for i in ALL_DATA_SOURCES_INSERT_SIZE:
        KOLM_DIST[i]={}
        KOLM_set[i]=Set()

    bin_set={}
    sup_val={}
    cum_exp_dist={}
    cum_prob_dist={}

    for ins_size in ALL_DATA_SOURCES_INSERT_SIZE:
        cum_exp_dist[ins_size]={}
        cum_prob_dist[ins_size]={}
        bin_set[ins_size]=[]
        sup_val[ins_size]=-1

    # this is slow
    ## reject sampling
    for ins_size in ALL_DATA_SOURCES_INSERT_SIZE:
        #max_den=max(probdist[ins_size].keys())
        max_den=max(probdist[ins_size].values())
        while(1):
            for bin in probdist[ins_size].keys():
                rannum=random.random()*max_den
                #print rannum, probdist[ins_size][bin]                
                if rannum<probdist[ins_size][bin]:
                    bin_set[ins_size].append(bin)

            if len(bin_set[ins_size])>=sample_size:
                break


    #print bin_set
    for ins_size in ALL_DATA_SOURCES_INSERT_SIZE:
        probkeys=probdist[ins_size].keys()
        probkeys.sort()
        cum_E=0
        for ebin in probkeys:
            ind=0
            for bin in bin_set[ins_size]:
                if bin<=ebin:
                    ind+=1
            cum_exp_dist[ins_size][ebin]=ind/len(bin_set[ins_size])

            cum_E+=probdist[ins_size][ebin]
            cum_prob_dist[ins_size][ebin]=cum_E


    for ins_size in ALL_DATA_SOURCES_INSERT_SIZE:
        for bin in probdist[ins_size].keys():
            cum_E=cum_prob_dist[ins_size][bin]#*len(soft_cluster_dic[ins_size])
            cum_O=cum_exp_dist[ins_size][bin]
            sup_val[ins_size]=max(sup_val[ins_size], abs(cum_O-cum_E))

    for ins_size in ALL_DATA_SOURCES_INSERT_SIZE:
        sup_bin=int(sup_val[ins_size]/0.0001)*0.0001
        if sup_bin in KOLM_set[ins_size]:
            KOLM_DIST[ins_size][sup_bin]+=1
        else:
            KOLM_DIST[ins_size][sup_bin]=1
            KOLM_set[ins_size].add(sup_bin)


    ########################################
    ## find threshold
    kolmo_thres={}
    for ins_size in ALL_DATA_SOURCES_INSERT_SIZE:
        kolmo_thres[ins_size]=0
    for ins_size in ALL_DATA_SOURCES_INSERT_SIZE:
        klkeys=KOLM_DIST[ins_size].keys()
        if len(klkeys)>0:
            klkeys.sort()
            norm=0
            for i in klkeys:
                norm+=KOLM_DIST[ins_size][i]

            tmp=0
            for i in klkeys:
                tmp+=(KOLM_DIST[ins_size][i]/norm)
                if tmp>=0.95:
                    kolmo_thres[ins_size]=i
                    break
    return kolmo_thres


def fast_gen_data_files():
    print "Approximated version of simulation to find thresholds for filters"
    ########################
    ## SIMULATION
    #initialize_global_vars()


    MEAN_INSERT_SIZE = int(props["MEAN_INSERT_SIZE"])
    STD_INSERT_SIZE = int(props["STD_INSERT_SIZE"])
    EXE_DIR = props["EXE_DIR"]

    probdist[MEAN_INSERT_SIZE]={}
    FILE_l=open(EXE_DIR+"/data/ApproximatedPofY.txt","r")
    illu_line=FILE_l.readline()
    while illu_line:
        tmp=illu_line.split()
        probdist[MEAN_INSERT_SIZE][int(tmp[0])]=float(tmp[1])
        illu_line=FILE_l.readline()

    FILE_l.close()
    mean_list[MEAN_INSERT_SIZE]=MEAN_INSERT_SIZE ## change if nessasary
    std_list[MEAN_INSERT_SIZE]=STD_INSERT_SIZE


    for ins_size in mean_list.keys():
        probdist_keys[ins_size]=probdist[ins_size].keys()
        probdist_keys[ins_size].sort()

    #############################################
    ## monte carlo, find threshold for rejection

    num_of_sampling=10
    range_sample_size=[20,401]
    src_dir=props["INPUT_DATA_DIR"]
    illumina_usage=3
    ALL_DATA_SOURCES_INSERT_SIZE=[MEAN_INSERT_SIZE]
    if illumina_usage==1:
        file_prefix="Illumina_"
    elif illumina_usage==2:
        file_prefix="Simul_"
    else:
        file_prefix=""

    fname_monte=EXE_DIR+"/data/"+file_prefix+"montecarlo_20_400.threshold"
    FILE_monte=open(fname_monte,"w")
    kolms={}
    for sample_sz in range(range_sample_size[0], range_sample_size[1], 1):
        kolms[sample_sz]={}
        for ins_size in ALL_DATA_SOURCES_INSERT_SIZE:
            kolms[sample_sz][ins_size]=0
    print "... start monte carlo ...", num_of_sampling, range_sample_size
    for i in range(num_of_sampling):
        print
        for sample_sz in range(range_sample_size[0], range_sample_size[1], 1):
            tmp=fast_Kolm_montecarlo(sample_sz)
            for ins_size in ALL_DATA_SOURCES_INSERT_SIZE:
                kolms[sample_sz][ins_size]+=tmp[ins_size]/num_of_sampling
                #print "monte:", sample_sz, tmp[ins_size]
            print "---- progress ---- ", str(i+1)+"-th sample out of "+str(num_of_sampling), ", sample size:", sample_sz

    for sz in kolms.keys():
        for ins_size in ALL_DATA_SOURCES_INSERT_SIZE:
            FILE_monte.write(str(sz)+'\t'+str(ins_size)+'\t'+str(kolms[sz][ins_size])+'\n')

    FILE_monte.close()




def fast_gen_pofy():
    EXE_DIR = props["EXE_DIR"]
    INPUT_DATA_DIR = props["INPUT_DATA_DIR"]
    MEAN_INSERT_SIZE = int(props["MEAN_INSERT_SIZE"])
    READ_LENGTH = int(props["READ_LENGTH"])
    MAX_SAMPLE_SIZE_FOR_POFY = int(props["MAX_SAMPLE_SIZE_FOR_POFY"])
    STD_INSERT_SIZE = int(props["STD_INSERT_SIZE"])

    num_of_samples = 100000
    sample_data = []
    for i in range(0,num_of_samples):
        sample =  int(random.gauss(MEAN_INSERT_SIZE, STD_INSERT_SIZE))
        sample_data.append(sample)
        #print sample

    pofy={}
    pofy_key=Set()
    pofy_list=[]

    #print f_list

    for mapped_dist in sample_data:
        if mapped_dist<3*MEAN_INSERT_SIZE and mapped_dist>=READ_LENGTH*2:
            if mapped_dist in pofy_key:
                pofy[mapped_dist]+=1
            else:
                pofy[mapped_dist]=1
                pofy_key.add(mapped_dist)
                pofy_list.append(mapped_dist)
    pofy_list.sort()
    n=sum(pofy.values())

    #print n
    fn='data/ApproximatedPofY.txt'
    FILE=open(fn, 'w')
    for m in pofy_list:
        den=float(pofy[m])/float(n)
        #print m, den
        pstr=str(m)+'\t'+str(den)+'\n'
        FILE.write(pstr)
    FILE.close()



     


def gen_pofy():
    EXE_DIR = props["EXE_DIR"]
    INPUT_DATA_DIR = props["INPUT_DATA_DIR"]
    MEAN_INSERT_SIZE = int(props["MEAN_INSERT_SIZE"])
    READ_LENGTH = int(props["READ_LENGTH"])
    MAX_SAMPLE_SIZE_FOR_POFY = int(props["MAX_SAMPLE_SIZE_FOR_POFY"])

    command='ls %s|grep chr' % (INPUT_DATA_DIR)
    f_list=get_sys_command_result(command)
    pofy={}
    pofy_key=Set()
    pofy_list=[]
    
    #print f_list
    for f in f_list:
        if len(f)>6:
            continue
        dbname="%s/%s" % (INPUT_DATA_DIR,f)
        print '-- making empirical dist. -- sampling ', dbname
        FILE = open(dbname, "r")
        line = FILE.readline()
        lcnt=0
        while line:
            if lcnt>1000: # after burnin time
                p=line.split()
                mapped_dist=int(p[4])-int(p[3])
                if mapped_dist<3*MEAN_INSERT_SIZE and mapped_dist>=READ_LENGTH*2:
                    if mapped_dist in pofy_key:
                        pofy[mapped_dist]+=1
                    else:
                        pofy[mapped_dist]=1
                        pofy_key.add(mapped_dist)
                        pofy_list.append(mapped_dist)

                #print p, mapped_dist
            if lcnt>MAX_SAMPLE_SIZE_FOR_POFY: # max sample size
                break
            lcnt+=1
            line = FILE.readline()
        FILE.close()       
    pofy_list.sort() 
    n=sum(pofy.values())
    #print n
    fn='data/PofY.txt'
    FILE=open(fn, 'w')
    for m in pofy_list:
        den=float(pofy[m])/float(n)
        #print m, den
        pstr=str(m)+'\t'+str(den)+'\n'
        FILE.write(pstr)  
    FILE.close()

    #FILE=open(fn, 'r')
    #line=FILE.readline()
    #t=0
    #while line:
    #    p=line.split()
    #    t+=float(p[1])
    #    line=FILE.readline()
    #print t
    #FILE.close()


def main(argv):
    if len(sys.argv)<2:
        print "Usage : python setup.py [type]"
        print 
        print "type 0: generate directories for modil (ex) setup.py 0"
        print "type 1: generate distribution of insert sizes for my own dataset (ex) setup.py 1"
        print "        use type 1 when you run MoDIL on your dataset"
    else:
        type = sys.argv[1]
        initialize_global_vars()
        #############################################
        ## create directories



        if type=="0":
            gen_dir()
        elif type=="1":
            gen_pofy()
            #gen_data_files()
        #elif type=="2":
        #    fast_gen_pofy()
            #fast_gen_data_files()
        else:
            print "Usage : python setup.py [type]"
            print 
            print "type 0: generate directories for modil (ex) setup.py 0"
            print "type 1: generate distribution of insert sizes for my own dataset (ex) setup.py 1"
            #print "        use type 1 when you run MoDIL on your dataset"
            #print "type 2: generate approximated distribution of insert sizes for my own dataset (ex) setup.py 2"

        
    #print '2. generate demo data files'
    #gen_simul_indel()
    #gen_demo_files()



#---------

if __name__=="__main__":
    main(sys.argv[1:])

