#!/usr/bin/env python

from __future__ import division
import os
import sys
sys.path.append('../')
import MPProperties
import datetime
import math
import Statistics
from sets import Set
import random
import gzip

# logger configuration
mean_list={}
std_list={}

alpha={}
getprob={}
coverage_list={}
#---------
props = {}

read_length=36
mp_size=208
EXE_DIR=''
target_chr=''
ref_genome_name = 'chr1.fa.short'

def complement(seq):
    #complement = {'A': 'T', 'C': 'G', 'G': 'C', 'T': 'A'}
    complement = {'A': 'T', 'C': 'G', 'G': 'C', 'T': 'A', 'N':'N'}
    complseq = [complement[base] for base in seq]
    return complseq



def reverse_complement(seq):
    seq = list(seq)
    seq.reverse()
    return ''.join(complement(seq))

def initialize_global_vars():
    # set the properties hash
    global props
    # read the properties hash
    props = MPProperties.read_config()



def get_sys_command_result(command):
    result = []
    command_result_pipe = os.popen(command)
    while True:
        line = command_result_pipe.readline()
        if not line:
            break
        line = line.strip()
        result.append(line)
    command_result_pipe.close()
    return result

#---------

def print_message_and_current_time(message):
    curr_time = datetime.datetime.now().ctime()
    curr_time_msg = message + curr_time
    
    # log the time message
    log.info(curr_time_msg)
    
    # print message and current time
    print curr_time_msg
    print

#---------


# given a (one-liner) SQL statement, return the command needed
# at the command prompt to execute this on this system

def return_list_from_result_set_column(mysqldb_result_set, column_index):
    result = []
    for result_set_row in mysqldb_result_set:
        result.append(result_set_row[column_index])
    return result

#---------

def return_str_list_from_list(list):
    result = []
    for item in list:
        result.append(str(item))
    return result

#---------    

class Searcher_print:
    def __init__(self, filename):
        self.f = open(filename, 'rb')
        self.f.seek(0,2)
        self.length = self.f.tell()

    def find(self, chr, start, end):

        low = 0
        high = self.length
        while low < high:
            if abs(low-high)<1:
                break
            mid = (low+high)/2
            p = mid
            while p > 0:
                self.f.seek(p)
                if self.f.read(1) == '\n': break
                p -= 1
            line = self.f.readline()
            tmp=line.split()
            if tmp==[]:
                low=mid+1
                continue
            chr_=tmp[1]
            start_=int(tmp[2])

            if chr_<chr:
                low = mid+1
            elif chr_>chr:
                high = mid
            elif start_<start:
                low = mid+1
            else:
                high = mid
        p = low
        while p > 0:
            self.f.seek(p)
            if self.f.read(1) == '\n': break
            p -= 1

        result = []
        counter=0
        prev_start_=0
        while True:
            line = self.f.readline()
            tmp=line.split()
            if tmp==[]:
                break
            chr_=tmp[1]
            start_=int(tmp[2])
            end_=int(tmp[3])
            if not line or chr_!=chr or start_>end:
                break
            result.append([chr_,start_,end_, tmp[0]])
            counter+=1
        return result



def create_simul_donor_genome(hom_flag):
    TARGET_REF='chr1'
    initialize_global_vars()

    EXE_DIR = props["EXE_DIR"]
    INPUT_DATA_DIR = props["INPUT_DATA_DIR"]
    RESULTS_DIR = props["RESULTS_DIR"]

    g_dir=RESULTS_DIR+'/tmp'
    simul_dir=INPUT_DATA_DIR

    ##############################
    ## ge positions to sample from

    f_in=EXE_DIR+'/data/simul_indels_input.txt'
    #print EXE_DIR, f_in
    FILE=open(f_in,'r')
    INDEL_LIST={}

    ## descard description
    line=FILE.readline()
    line=FILE.readline()
    for chr in [TARGET_REF]:
        INDEL_LIST[chr]={}
   
    #######################
    ## 1. Read input indels

    #########################
    ## assume they are sorted
    s_id=1 
    while line:
        ###############################################
        tmp=line.split()
        if hom_flag==1:
            ## 1 includes wild type in one copy of chromosome
            if tmp[4]=='HOM':
                INDEL_LIST[tmp[0]][s_id]=[int(tmp[1]), int(tmp[2]), tmp[3], tmp[5]]
                s_id+=1

        else:
            INDEL_LIST[tmp[0]][s_id]=[int(tmp[1]), int(tmp[2]), tmp[3], tmp[5]]
            s_id+=1
        line=FILE.readline()
    FILE.close()


    ## key 0 is not used
    #for i in range(s_id):
    #    SEQ_RES[TARGET_REF][i]=''

    #print INDEL_LIST
    ##################################
    ## 2. Make simulated DNA sequences

    f_donor=g_dir+'/donor_'
    f_donor+=TARGET_REF+'.'+str(hom_flag)

    REF_genome=EXE_DIR+'/data/' + ref_genome_name 
    FILE=open(REF_genome,'r')
    FILE_donor=open(f_donor,'w')
    line=FILE.readline() ## first line
    line=FILE.readline()
    curr_pos=1
    line_cnt=0

    IL_keys=INDEL_LIST[TARGET_REF].keys()
    IL_keys.sort() 

    IL_key_set=Set(IL_keys)

    Del_key_set=Set()

    progress=0
    log_freq=10000
    dup_rel_set=Set()
    while line:
        #if progress % log_freq ==0:
        #    print 'donor is being created - progress log:', progress

        ###################################
        ## reflush del sets
        if progress==0 or progress % log_freq ==0:
            #print '-- before --'
            IL_key_set=IL_key_set.difference(Del_key_set)
            #print IL_key_set, Del_key_set
            IL_key_list=[]
            for item in IL_key_set:
                IL_key_list.append(item)
            IL_key_list.sort()
            IL_key_set.clear()
            IL_key_set=Set(IL_key_list)
            Del_key_set.clear()
            #print 'reflush del list', IL_key_set, tmp_list
        progress+=1

        #################
        ## make new keys

        line_s=line.strip()
        line_len=len(line_s)
        curr_pos_end=curr_pos+line_len-1

        #################################
        ## check if indels exist
        ## if exist, then do one by one
        ## otherwise, just write the line
        related_indels=[]
	dup_rel_set.clear()
	log_flag=0
        for t_id in IL_key_list:
            if t_id in Del_key_set:
                continue

            t_start=INDEL_LIST[TARGET_REF][t_id][0]
            t_end=INDEL_LIST[TARGET_REF][t_id][1]
            indel_type=INDEL_LIST[TARGET_REF][t_id][2]
            seq=INDEL_LIST[TARGET_REF][t_id][3]

	    dup_key=str(t_start)+'_'+str(t_end)
	    if dup_key in dup_rel_set:
	        continue
            #print 'info: ', t_start, t_end, indel_type, seq
            if max(curr_pos, t_start)<=min(curr_pos_end, t_end):
                related_indels.append([t_start,t_end,indel_type,seq,t_id])
	   	dup_rel_set.add(dup_key)
	        #if t_start==1004834:
		#    print curr_pos, curr_pos_end, progress, seq, INDEL_LIST[TARGET_REF][t_id]
		#    log_flag=1
            #print t_start, curr_pos_end
            if curr_pos_end<t_start:
                break
        #print related_indels

        if len(related_indels)==0:
            FILE_donor.write(line)
            #print line,
            line=FILE.readline()
            line_cnt+=1
            #print 'line len', line_len 
            curr_pos+=line_len
            continue
        ###############################################
        ## character by character
        ## Need to DELETE
	append_f=0
        for c in line_s:
            if c=='\n':
                continue
            no_write=0
            for r_indel in related_indels:
                t_start=r_indel[0]
                t_end=r_indel[1]
                indel_type=r_indel[2]
                seq=r_indel[3]
                t_id=r_indel[4]
                
		#if log_flag==1:
		#    print curr_pos, t_start, t_end, related_indels
                if indel_type=='DEL':
                    if curr_pos>=t_start and curr_pos<=t_end:
                        no_write=1
                        break
                elif indel_type=='INS':
                    if curr_pos==t_start+1:
                        FILE_donor.write(seq)
			#if curr_pos==1004813+1:
			#    print 'print', r_indel
		        break
		    elif curr_pos==curr_pos_end and curr_pos==t_start:
			append_f=1
		        break
        
            curr_pos+=1 # increment current position 
            if no_write==0:
                FILE_donor.write(c)
                #print ':char:', c,

        if append_f==1:
	    #if curr_pos==1693602+1 or curr_pos==1693600+1:
	    #    print 'print', r_indel
	    FILE_donor.write(seq)
        #print '\n'
        FILE_donor.write('\n')

        ################################################################
        ## remove elements in the list which are already considered
        remove_list=[]
        for t_id in IL_key_list:
            if t_id in Del_key_set:
                continue
            t_start=INDEL_LIST[TARGET_REF][t_id][0]
            t_end=INDEL_LIST[TARGET_REF][t_id][1]

            if t_end<curr_pos: 
                remove_list.append(t_id)
            if curr_pos_end<t_start:
                break

        for t_id_r in remove_list:
            #print ">> before ", INDEL_LIST
            #print t_id_r
            Del_key_set.add(t_id_r)
            #del(INDEL_LIST[TARGET_REF][t_id_r])
            #print ">> after", INDEL_LIST
        #print 'line len', line_len 

        #print line_s, line_len
        line=FILE.readline()
        line_cnt+=1


    FILE.close()
    FILE_donor.close()

    ###########################
    ## 3. each line has 50 letters
    formatting(f_donor)

def formatting(fname):
    FILE=open(fname, 'r')
    FILE_out=open(fname+'.txt', 'w')
    line=FILE.readline()
    buffer=''
    line_length=50
    while line:
        buffer+=line.strip()
        if len(buffer)>50:
            FILE_out.write(buffer[0:line_length]+'\n')
            buffer=buffer[line_length:len(buffer)]
        line=FILE.readline()
    FILE_out.write(buffer)

    FILE.close()
    FILE_out.close()

##########################################
## make matepairs from simulated genome
## total coverage num of fcnt * coverage
def generate_matepairs(hom_flag, coverage, fcnt):
    TARGET_REF='chr1'
    initialize_global_vars()

    EXE_DIR = props["EXE_DIR"]

    INPUT_DATA_DIR = props["INPUT_DATA_DIR"]
    RESULTS_DIR = props["RESULTS_DIR"]
    g_dir=RESULTS_DIR+'/tmp'
    simul_dir=INPUT_DATA_DIR

    f_donor=g_dir+'/donor_'
    f_donor+=TARGET_REF+'.'+str(hom_flag)+'.txt'
    #f_donor=g_dir+'/test.txt'
    f_mp=g_dir+'/mp_'
    f_mp+=TARGET_REF+'.'+str(hom_flag)+'.txt.'+str(fcnt)

    ## chr1 size is
    ## 247249719

    ## get chromosome size
    FILE=open(f_donor,'r')
    line=FILE.readline()
    chr_size=0 
    line_length=0
    while line:
        chr_size+=len(line.strip())
	line_length=len(line.strip())
        line=FILE.readline()
	
    print 'chromosome size:', chr_size, 'line_len:',line_length
    FILE.close()
    #chr_size=247249719


    tot_number_of_matepairs=int((chr_size*coverage)/mp_size)
    print ':', hom_flag, '-->', tot_number_of_matepairs, 'matepairs are being created'

    ###################################
    ## randomly generate staring points
    mp_list=[]
    m_id=1
    res_list={}
    final_list={}
    pos_dic={}

    #####################################
    ## load distribution
    probdist={}
    probdist[mp_size]={}
    FILE_l=open(EXE_DIR+"/data/illumina_PofY.txt","r")
    illu_line=FILE_l.readline()
    min_val=10**10
    max_val=-1
    max_density=-1
    while illu_line:
        tmp=illu_line.split()
        probdist[mp_size][int(tmp[0])]=float(tmp[1])
        min_val=min(min_val, int(tmp[0]))
        max_val=max(max_val, int(tmp[0]))
        max_density=max(max_density, float(tmp[1]))
        illu_line=FILE_l.readline()
    FILE_l.close()
    mean_dist=mp_size ## change if nessasary
    std_dist=13
    ####################################

    for i in range(tot_number_of_matepairs):
        if i % 1000 == 0:
            print 'progress log:', i, 'random matepairs are being generated'
        start_pos=random.randrange(1,chr_size-mp_size) 

        ############################################################
        ## sample mp_size_from_dist from experimental distribution
        
        mp_size_from_dist=0
        for cast in range(500):
            #print min_val, max_val, max_density
            random_dist=random.randrange(min_val, max_val)
            if max_density*random.random()<=probdist[mp_size][random_dist]:
                mp_size_from_dist=random_dist
                break
        #print cast
        #print mp_size_from_dist
        if mp_size_from_dist==0:
            print '-- failed to sample --\n' 
            continue

        end_pos=start_pos+mp_size_from_dist-1

        m_id_f=str(m_id)+'_f'
        m_id_r=str(m_id)+'_r'

        mp_list.append([start_pos, start_pos+read_length-1, m_id_f])
        mp_list.append([end_pos-read_length+1, end_pos, m_id_r])

        pos_dic[m_id_f]=[start_pos,start_pos+read_length-1]
        pos_dic[m_id_r]=[end_pos-read_length+1, end_pos]

        #res_list[m_id_f]=''
        #res_list[m_id_r]=''
        final_list[str(m_id)]=['','']

        m_id+=1
    mp_list.sort()
    print 'sampling is done!!'

    FILE=open(f_donor+'_singleline', 'r')
    for mp in mp_list:
        t_start=mp[0]
        t_end=mp[1]
        t_id=mp[2]

	#num_of_new_line=int((t_start-1)/line_length)
	#t_start+=num_of_new_line
        FILE.seek(t_start-1)
        dna_seq_=FILE.read(2*read_length+5)
        dna_seq_list=dna_seq_.split()
        
        dna_seq=''
        for i in range(len(dna_seq_list)):
            dna_seq+=dna_seq_list[i]
        dna_seq=dna_seq[0:read_length]
	if len(dna_seq)!=read_length:
	    continue
        cb=1
	for i in range(read_length):
	    if dna_seq[i]!='N':
		cb*=0
        if cb==0:
            res_list[t_id]=dna_seq
    FILE.close()

    #print mp_list
    ################################# 
    ## create search index
    #search_idx={}
    #mp_sort_num=0
    #for mp in mp_list:
    #    search_idx[mp[2]]=mp_sort_num
    #    mp_sort_num+=1

    #################################

    #line=FILE.readline()
    #curr_pos=1
    #while line:
        #print line
    #    line_s=line.strip()
    #    line_len=len(line_s)
    #    curr_pos_end=curr_pos+line_len-1

        #######################################
        ## make matepair data 
        #print '----------'
    #    if progress % 2000 ==0: 
    #        print 'sampling seq - progress :', progress, mp_list_curr
    #    progress+=1

    #    if progress % 2000 ==0:
    #        mp_list_search=mp_list[mp_list_curr:len(mp_list)]
    #        mp_list_curr_base=mp_list_curr
            

    #    for mp in mp_list_search:
    #        t_start=mp[0]
    #        t_end=mp[1]
    #        t_id=mp[2]

            ###########################
            ## overlap with start point
    #        if max(curr_pos, t_start)<=min(curr_pos_end, t_start):
    #            seq_start=t_start-curr_pos
    #            seq_end=min(seq_start+read_length,line_len)
    #            dna_seq=line_s[seq_start:seq_end]
    #            res_list[t_id]+=dna_seq

            #########################
            ## only overlap with end point
    #        elif max(curr_pos, t_end)<=min(curr_pos_end, t_end):
    #            seq_start=0
    #            seq_end=t_end-curr_pos+1
    #            dna_seq=line_s[seq_start:seq_end]
    #            res_list[t_id]+=dna_seq
#
#            if t_start>curr_pos_end:
#                break
            #print 'a', t_start, curr_pos_end
        #print 'd'
        #######################################
        ## find search index
#        mp_list_curr_=mp_list_curr_base
#        for mp in mp_list_search:
#            t_start=mp[0]
#            t_end=mp[1]
#            t_id=mp[2]
#            #print t_end, curr_pos
#            if t_end<curr_pos:
#                mp_list_curr_=max(mp_list_curr_,search_idx[t_id]+1)
#                #print curr_pos, mp_list_curr_, search_idx[t_id], t_id
#        
#            #print t_start,t_end, t_id
#            if t_start>curr_pos_end:
#                break
#        #print 'e'
#        mp_list_curr=mp_list_curr_
#        curr_pos=curr_pos+line_len
#        #print 'b', t_start, curr_pos_end
#        line=FILE.readline()
#    
        
        
    for key in res_list.keys():
        tmp=key.split('_')
        #print tmp[0], tmp[1]
        if tmp[1]=='f':
            final_list[tmp[0]][0]=res_list[key]
        elif tmp[1]=='r':
            final_list[tmp[0]][1]=res_list[key]
        

    FILE_mp=open(f_mp, 'w')
    n_map_id=1
    map_id=str(n_map_id)+'_'+str(hom_flag)+'_'+str(fcnt)

    for key in final_list.keys():
	if len(final_list[key][0])==0 or len(final_list[key][1])==0:
	    continue
       # print final_list[key]
    
        f_start=pos_dic[key+'_f'][0]
        f_end=pos_dic[key+'_f'][1]
        r_start=pos_dic[key+'_r'][0]
        r_end=pos_dic[key+'_r'][1]

        buffer=str(f_start)+'\t'
        buffer+=str(f_end)+'\t'
        buffer+=final_list[key][0]+'\t'

        buffer+=str(r_start)+'\t'
        buffer+=str(r_end)+'\t'
        buffer+=final_list[key][1]+'\t'
       
        buffer+=str(r_end-f_start+1)+'\t'
	buffer+=str(map_id)+'\n' 
        #print buffer

        FILE_mp.write(buffer)

	n_map_id+=1
        map_id=str(n_map_id)+'_'+str(hom_flag)+'_'+str(fcnt)
    
    FILE_mp.close()
    print



############################
## not affected by SNPs
def generate_chain(hom_flag):
    initialize_global_vars()

    EXE_DIR = props["EXE_DIR"]

    INPUT_DATA_DIR = props["INPUT_DATA_DIR"]
    RESULTS_DIR = props["RESULTS_DIR"]
    g_dir=RESULTS_DIR+'/tmp'
    simul_dir=INPUT_DATA_DIR

    ####################################
    ## validate chain
    REF_genome=EXE_DIR+'/data/' + ref_genome_name
    FILE_ref=open(REF_genome,'r')
    FILE_ref_w=open(REF_genome+'_singleline','w')
    line_ref=FILE_ref.readline()
    line_ref=FILE_ref.readline()
    while line_ref:
        FILE_ref_w.write(line_ref.strip())
        line_ref=FILE_ref.readline()
    FILE_ref_w.close()
    FILE_ref.close()

    ## chr1 size is
    ## 247249719

    ## get chromosome size
    #FILE=open(REF_genome,'r')
    #line=FILE.readline()
    #chr_size=0 
    #while line:
    #    chr_size+=len(line.strip())
    #    line=FILE.readline()
    #print 'REF chromosome size:', chr_size
    #FILE.close()

    #chr1    10      11      INS     HET     NNNNNNNNNNNNNN

    print 'generate chain'
    f_in=EXE_DIR+'/data/simul_indels_input.txt'
    FILE_indels=open(f_in,'r')

    line_indels=FILE_indels.readline() # first line
    line_indels=FILE_indels.readline()
    INDEL_list=[] 
    while line_indels:
        tmp=line_indels.split()
        #print tmp
        start_pos=int(tmp[1])
        end_pos=int(tmp[2])
        type=tmp[3]
	sindel=len(tmp[5])
	seq=tmp[5]	
        if hom_flag==1:
            if tmp[4]=='HOM':
                if type=='INS':
                    INDEL_list.append([start_pos, end_pos, 'I', sindel, seq])
                elif type=='DEL':
                    INDEL_list.append([start_pos, end_pos, 'D', sindel,seq])
        else:
            if type=='INS':
                INDEL_list.append([start_pos, end_pos, 'I', sindel,seq])
            elif type=='DEL':
                INDEL_list.append([start_pos, end_pos, 'D', sindel,seq])

	if type=='DEL' and end_pos-start_pos+1!=sindel:
	    print 'ERROR!!!'
	    break
        line_indels=FILE_indels.readline()
   
    FILE_indels.close() 
    INDEL_list.sort() 
         
    donor=0
     
    FILE_ref=open(REF_genome+'_singleline','r')
    FILE_don=open(g_dir+'/donor_chr1.'+str(hom_flag)+'.txt'+'_singleline','r')
    chain_list=[]
    for indel in INDEL_list:
        ## increment only donor 
	seq=indel[4]
        if indel[2]=='I':
            ref_pos=indel[0]
            donor_pos=donor+indel[0]
            #FILE_chain.write(str(ref_pos)+' '+str(donor_pos)+'\n')
	    chain_list.append([ref_pos,donor_pos,'I'])
            donor+=indel[3]
            ref_pos=indel[0]+1
            donor_pos=donor+indel[0]+1
            #FILE_chain.write(str(ref_pos)+' '+str(donor_pos)+'\n')
	    chain_list.append([ref_pos,donor_pos,'R'])
	
	    FILE_ref.seek(ref_pos)
	    FILE_don.seek(donor_pos)

        elif indel[2]=='D':
            ref_pos=indel[0]-1
            donor_pos=donor+indel[0]-1
            #FILE_chain.write(str(ref_pos)+' '+str(donor_pos)+'\n')
	    chain_list.append([ref_pos,donor_pos,'D'])

            donor-=indel[3]

            ref_pos=indel[1]+1
            donor_pos=donor+indel[1]+1

            #FILE_chain.write(str(ref_pos)+' '+str(donor_pos)+'\n')
	    chain_list.append([ref_pos,donor_pos,'R'])
	    FILE_ref.seek(ref_pos)
	    FILE_don.seek(donor_pos)
        
	#if FILE_ref.read(5) !=  FILE_don.read(5):
	#    print donor_pos, ref_pos, FILE_ref.read(5), FILE_don.read(5)
	#else:
	#    print donor_pos, ref_pos, FILE_ref.read(5), FILE_don.read(5), 'same'
	
	prev_indel=indel
    FILE_ref.close()
    FILE_don.close()

    chain_list.sort()
    ##################################
    FILE_chain=open(g_dir+'/simul_chain.'+str(hom_flag)+'.txt','w')
    FILE_chain.write(str(1)+' '+str(1)+' R\n')
    for chain in chain_list:
	ref_pos=chain[0]
	donor_pos=chain[1]
        FILE_chain.write(str(ref_pos)+' '+str(donor_pos)+' '+str(chain[2])+'\n')
    FILE_chain.close()

    FILE_chain=open(g_dir+'/simul_chain.'+str(hom_flag)+'.txt','r')
    FILE_ref=open(REF_genome+'_singleline','r')
    FILE_don=open(g_dir+'/donor_chr1.'+str(hom_flag)+'.txt_singleline','r')
    line=FILE_chain.readline()
    progress=0
    while line:
	tmp=line.split()
	ref_offset=int(tmp[0])
	don_offset=int(tmp[1])

	FILE_ref.seek(ref_offset-1)
	FILE_don.seek(don_offset-1)
	a=FILE_ref.read(10)
	b=FILE_don.read(10)	
	if progress % 100 == 0:
	    if a.lower()!=b.lower():	
	        print ref_offset, a.lower(), ':::', don_offset, b.lower()
	    else:
	        print ref_offset, a.lower(), ':::', don_offset, b.lower(), 'same'
	progress+=1
        line=FILE_chain.readline()
    FILE_chain.close()
    FILE_don.close()
    FILE_ref.close()	

#######################################################################################

def input_simul_db(hom_flag,f_cnt):
    initialize_global_vars()

    EXE_DIR = props["EXE_DIR"]

    INPUT_DATA_DIR = props["INPUT_DATA_DIR"]
    RESULTS_DIR = props["RESULTS_DIR"]
    g_dir=RESULTS_DIR+'/tmp'
    simul_dir=INPUT_DATA_DIR
    print 'make simul_db, hom_flag:', hom_flag
    #db_obj[fwd_chr].write(fwd_end_pos+' '+rev_start_pos+' '+str(map_id)+' '+str(ndist)+' '+str(insert_size)+' '+template_id+'\n')
    FILE=open(g_dir+'/mp_chr1.'+str(hom_flag)+'.txt.'+str(f_cnt), 'r')
    FILE_out=open(simul_dir+'/chr1.'+str(hom_flag)+'.'+str(f_cnt), 'w')
    FILE_chain=open(g_dir+'/simul_chain.'+str(hom_flag)+'.txt','r')

    ################################
    ## load chain
    chain_list=[]
    line_chain=FILE_chain.readline()
    while line_chain:
        tmp2=line_chain.split()
        ref_pos=int(tmp2[0])
        don_pos=int(tmp2[1])
        type=tmp2[2]
        chain_list.append([ref_pos, don_pos, type])
        line_chain=FILE_chain.readline()
    FILE_chain.close()

    #212982645       212982680       ATTCAGAACAAAAGCTTCTCATGGGGCTTCCCTGTC    212982835       212982870       CTACTTATCTGCTCTTTGGCCCAATACATGTGGAGA    226     1_0_0
    ##############################
    ## matepair pos list
    mp_pos_list=[]
    map_id_key={}
    line=FILE.readline()
    while line:
        tmp=line.split() 
        f_start=int(tmp[0])
        f_end=int(tmp[1])
        r_start=int(tmp[3])
        r_end=int(tmp[4])
	map_id=tmp[7]

        mp_pos_list.append(f_start)
        mp_pos_list.append(f_end)
        mp_pos_list.append(r_start)
        mp_pos_list.append(r_end)
	comp_key=str(f_start)+'_'+str(f_end)+'_'+str(r_start)+'_'+str(r_end)
	map_id_key[comp_key]=map_id
        line=FILE.readline()

    mp_pos_list.sort()
    print 'reading mp data is done!'


    ###########################
    ## create search index
    idx=0 
    s_idx={}
    for mp in mp_pos_list:
        s_idx[mp]=idx
        idx+=1

    ########################################
    ## mapping between mp_pos and reference 
    ## using chain
    mapping={}
    search_idx=0 
    search_idx_=0 
    s_list=[]
    progress=0
    del_set=Set()
    #print mp_pos_list[0:200]
    pre_type='R' # reset
    for chain in chain_list:
	#if progress % 100 ==0:
	#    print 'progress log: ', progress
	if progress % 100 ==0:
	    s_list=mp_pos_list[search_idx:len(mp_pos_list)]	    
	    #print search_idx
	    del_set.clear()
	
 	progress+=1

        for mp_pos in s_list:
	    if mp_pos in del_set:
		continue

            if mp_pos==chain[1]:
                mapping[mp_pos]=chain[0]
                search_idx_=max(search_idx_,s_idx[mp_pos]+1)

            if mp_pos<chain[1]:
                offset=mp_pos-pre_don_chain
                ref_pos=pre_ref_chain+offset 
		if pre_type!='I':
                    mapping[mp_pos]=ref_pos
		else:
                    mapping[mp_pos]=-100
		#if mp_pos==2:
		#    print 'log : ', mp_pos, ref_pos
                search_idx_=max(search_idx_,s_idx[mp_pos]+1)
		del_set.add(mp_pos)

            ### stop searching
            if mp_pos>chain[1]:
                break

        search_idx=search_idx_
        pre_don_chain=chain[1] 
        pre_ref_chain=chain[0]
	pre_type=chain[2]

    #print pre_ref_chain, pre_don_chain
    ## the rest of pos
    for mp_pos in mp_pos_list[search_idx:len(mp_pos_list)]:
        offset=mp_pos-pre_don_chain
        ref_pos=pre_ref_chain+offset 
        mapping[mp_pos]=ref_pos

    mp_pos_list=[]
    ## mappings are done
    ######################################
    
    s_idx=[]
    #print 'mappings: ', mapping

    FILE.seek(0)
    line=FILE.readline()
    #n_mapid=1
    #map_id=str(n_mapid)+'_'+str(hom_flag)
    while line:
        tmp=line.split() 
        f_start=int(tmp[0])
        f_end=int(tmp[1])
        f_seq=tmp[2]
        r_start=int(tmp[3])
        r_end=int(tmp[4])
        r_seq=tmp[5]

       
	comp_key=str(f_start)+'_'+str(f_end)+'_'+str(r_start)+'_'+str(r_end)
	map_id=map_id_key[comp_key]

        ####################################
        ## exact mapping 
        if abs(mapping[f_end]-mapping[f_start]+1)!=read_length:
            line=FILE.readline()
            continue
        if abs(mapping[r_end]-mapping[r_start]+1)!=read_length:
            line=FILE.readline()
            continue
    
        #print tmp
        #insert_size=int(tmp[6]) ## real insert size - unknown in reality
        buffer=str(mapping[f_end])+' '
        buffer+=str(mapping[r_start])+' '
        buffer+=str(map_id)+' '
        mapped_dist=mapping[r_end]-mapping[f_start]+1
        #if mapped_dist==insert_size:
        #    line=FILE.readline()
        #    continue

        buffer+=str(mp_size-mapped_dist)+' '
        buffer+=str(mp_size)+' '
        buffer+='SIMUL_'+str(map_id)+'\n'

        ###################
        ## test
        #buffer+=str(f_start)+' '
        #buffer+=str(f_end)+' '
        #buffer+=str(r_start)+' '
        #buffer+=str(r_end)+' '
        #buffer+='\t'
        
        #buffer+=str(mapping[f_start])+' '
        #buffer+=str(mapping[f_end])+' '
        #buffer+=str(mapping[r_start])+' '
        #buffer+=str(mapping[r_end])+'\n'

        FILE_out.write(buffer)
        

        #print f_start, f_end, f_seq, r_start, r_end, r_seq, insert_size
        #n_mapid+=1
        #map_id=str(n_mapid)+'_'+str(hom_flag)
        line=FILE.readline()

    FILE.close()
    
    FILE_out.close()
     

def extract_mills_indels():
    EXE_DIR = props["EXE_DIR"]
    #chromosome      start_pos       end_pos    type       heterozygosity     sequence
    fn=EXE_DIR+'/data/supplemental_table_chr1.txt'
    indel_f=EXE_DIR+'/data/indels_input.txt'
    FILE=open(fn,'r')
    FILE_out=open(indel_f+'_','w')

    line=FILE.readline() ## firstline
    line=FILE.readline()
    
    buffer='chromosome \t start_pos \t end_pos type \t heterozygosity \t sequence \n'
    FILE_out.write(buffer)
    
    while line:
        tmp=line.split()
        id=tmp[1]
        t_start_pos=int(tmp[5])
        t_end_pos=int(tmp[6])
        chrom=tmp[4]

        start_pos=int(tmp[7])
        end_pos=int(tmp[8])
        seq=tmp[35]

        type=tmp[12]
        buffer=chrom+'\t'
        buffer+=str(start_pos)+'\t'
        buffer+=str(end_pos)+'\t'
        buffer+=type+'\t'
        if random.random()>0.5:
            buffer+='HET'+'\t'
        else:
            buffer+='HOM'+'\t'
        buffer+=seq+'\n'
        FILE_out.write(buffer)
        #print type, chrom, id, start_pos, end_pos, seq, len(seq)
        line=FILE.readline()
        
    FILE.close()
    FILE_out.close()

    ###########################
    ## sort the indels files
    command='sort -k 2n,2 -k 3n,3 %s>%s' % (indel_f+'_', indel_f) 
    get_sys_command_result(command)
    ## delete temporary files
    command='rm %s' % (indel_f+'_') 
    get_sys_command_result(command)


    #command='cat %s/mp_%s.0.txt.* %s/mp_%s.1.txt.* > %s/mp_%s.txt' % (g_dir,target_chr,g_dir,target_chr,g_dir,target_chr)
    #command='cat %s/mp_%s.0.txt.* > %s/mp_%s.0.txt' % (g_dir,target_chr,g_dir,target_chr)
    #get_sys_command_result(command)
    #command='cat %s/mp_%s.1.txt.* > %s/mp_%s.1.txt' %  (g_dir,target_chr,g_dir,target_chr)
    #get_sys_command_result(command)
    #print 'merging file is done!'

def remove_dup_in_mill_indels():
    EXE_DIR = props["EXE_DIR"]
    indel_f=EXE_DIR+'/data/indels_input.txt'
    indel_f_out=EXE_DIR+'/data/simul_indels_input.txt'

    FILE=open(indel_f,'r')
    line=FILE.readline()
    line=FILE.readline()
    prev_line=''
    cnt=0
    indel_list=[]
    prev_start_pos=-1
    #start_dic={}
    prev_indel=[-1,-1]
    while line:
        #print line
	tmp=line.split()
	start_pos=int(tmp[1])
	end_pos=int(tmp[2])

 	dup_flag=0
	for dup in indel_list:
	    if max(start_pos,dup[0])<=min(end_pos, dup[1])+1:
		### consider indels which can be merged
	        cnt+=1
	        dup_flag=1
		break
        if dup_flag==1:
	    line=FILE.readline()
	    continue

	#print start_pos, end_pos, prev_indel
	#if max(start_pos,prev_indel[0])<=min(end_pos, prev_indel[1]):
	#else:	
	indel_list.append([start_pos, end_pos])
	#prev_indel=[start_pos,end_pos]
        line=FILE.readline()
    indel_list.sort()

    FILE_w=open(indel_f_out, 'w')
    buffer='chromosome \t start_pos \t end_pos type \t heterozygosity \t sequence \n'
    FILE_w.write(buffer)
    FILE.seek(0)    
    line=FILE.readline()
    line=FILE.readline()
    dup_set=Set()
    while line:
        tmp=line.split()
	start_pos=int(tmp[1])
	end_pos=int(tmp[2])
	comp_key=tmp[1]+'_'+tmp[2]

	if comp_key in dup_set:
            line=FILE.readline()
	    continue

	if [start_pos, end_pos] in indel_list:
            if random.random()<0.05: # use only 5% of indels for this test
                FILE_w.write(line)
	dup_set.add(comp_key)
        line=FILE.readline()

    print 'tot removed:', cnt
    
    FILE.close()
    FILE_w.close()

def make_no_new_line_file():
    initialize_global_vars()

    RESULTS_DIR = props["RESULTS_DIR"]
    g_dir=RESULTS_DIR+'/tmp'

    EXE_DIR = props["EXE_DIR"]

    for hom_flag in [0,1]:
        f_donor=g_dir+'/donor_'
        f_donor+='chr1'+'.'+str(hom_flag)+'.txt'

        FILE=open(f_donor,'r')
        line=FILE.readline()
        chr_size=0
        line_length=0
        FILE_do_w=open(f_donor+'_singleline','w')
        while line:
            chr_size+=len(line.strip())
            line_length=len(line.strip())
            FILE_do_w.write(line.strip())
            line=FILE.readline()

        print 'created no new line file', hom_flag
        FILE.close()
        FILE_do_w.close()






# 97967006        97967041        TTATTCTATTTTAAAGGGTCATTCATTTATCTTTTA    97967156        97967191        AGCCTAAGGTGTTTTATGTGAATATATCCAACATTT    186     19218_0_100
def reverse_complement_matepairs():
    initialize_global_vars()

    EXE_DIR = props["EXE_DIR"]
    RESULTS_DIR = props["RESULTS_DIR"]
    g_dir=RESULTS_DIR+'/tmp'

    command='ls %s|grep %s' % (g_dir,'mp_chr1')
    f_list=get_sys_command_result(command)
    
    for f in f_list:
	print 'processing file ... ', f
	fname='%s/%s' % (g_dir,f)
	fp=open(fname, 'r')
	#fname_w='%s/matepair_data/%s' % (g_dir,f)
	fname_w='%s/%s_reversed' % (g_dir,f)
	fp_w=open(fname_w, 'w')
	line=fp.readline()
	progress=0
        while line:
	    if progress % 10000 ==0:
	        print 'processing file progress... ', f, progress
	    progress+=1

	    tmp=line.split()
	    l_start=int(tmp[0])
	    l_end=int(tmp[1])
	    l_seq=tmp[2]
	    r_start=int(tmp[3])
	    r_end=int(tmp[4])
	    r_seq=tmp[5]
            id=tmp[7]

	    buffer=str(l_start)+'\t'
	    buffer+=str(l_end)+'\t'
	    buffer+=l_seq+'\t'
	    buffer+=str(r_start)+'\t'
	    buffer+=str(r_end)+'\t'                                                                                                                                      
	    buffer+=reverse_complement(r_seq.upper())+'\t'
	    buffer+='208\t'
	    buffer+=id+'\n'
	    #print line
	    fp_w.write(buffer)
	    line=fp.readline()
	fp.close()
	fp_w.close()

#>2_6_19_R3
#G3203203231310012222133232     	
#53301_1_72/2    chr1    4404    -       -221    18      99      82      82      0       0       1       0       36      aagccagccttccgctccttgaagctggtctccaca88;;;;;;;7;;;;;;;;;;;;3;;;;;;;;;;;;;
#186361_1_66     chr1    181546345       181552489       0       1       -6056.0 0.0     1e-15   36416968


def generate_heatmap_code():
    FILE=open('./modil_heatmap2.txt','r')
    FILE_out=open('./modil_heatmap2_matlab.m','w')
    line=FILE.readline()
    x_axis='x=['
    y_axis='y=['
    while line:
	tmp=line.split()
	indel_size=abs(int(tmp[0]))
	clus_size=abs(int(tmp[1]))
	cardi=round(float(tmp[2])*20)
	#print tmp
	for i in range(cardi):
	    x_axis+=str(clus_size)+','
	    y_axis+=str(indel_size)+','
	
	print indel_size, clus_size, cardi
        line=FILE.readline()

    x_axis=x_axis[0:len(x_axis)-1]
    y_axis=y_axis[0:len(y_axis)-1]
    x_axis+='];\n'
    y_axis+='];\n'

    #print x_axis
    #print y_axis

    FILE_out.write(x_axis)
    FILE_out.write(y_axis)
	
    FILE.close()	
    FILE_out.close()	

    #######################################################
    ## 12. extract final results with FDR
    #create_formatted_extracted_results_with_FDR(0)
    #create_formatted_extracted_results_with_FDR(1)


def clustering_demo():
    print "........................................."
    print "... Now MoDIL is generating clusters ...."
    print "........................................."

    EXE_DIR = props["EXE_DIR"]
    INPUT_DATA_DIR = props["INPUT_DATA_DIR"]
    clus_size=0
    step_size=10**9
    dbname="%s/%s" % (INPUT_DATA_DIR,target_chr)
    db = open(dbname, "r")

    line=db.readline()
    while line:
        clus_size+=1
        line=db.readline()
    db.close()
    print 'size of data: ', clus_size

    step=0
    command="python %s/MoDIL_simple.py %s %d %d" %(EXE_DIR, target_chr, step, step_size)
    get_sys_command_result(command)
    print command

def inference_demo():
    print "........................................."
    print "......... INFERENCE INDELS .............."
    print "........................................."

    chr='chr1'
    exe_dir=props["EXE_DIR"]
    results_dir=props["RESULTS_DIR"]

    #prefix="mod_"

    #grep_word="%sCluster_Indel_%s.candidates" % (prefix,chr)
    ##############################
    ##  submit jobs to qsub
    #fname='%s/CLUSTERING_RESULTS/%s/%sCluster_Indel_%s.candidates.0.gz' % (results_dir,chr,prefix,chr)
    command='%s/qsub_MoDIL_inference.py %s %s' % (exe_dir, chr, '0')
    print command
    os.system(command)

def gen_simulated_indels():
    exe_dir=props["EXE_DIR"]
    fname=exe_dir+'/data/simul_indels_input.txt'
    FILE=open(fname, 'w')
   
    line='chromosome\tstart_pos\tend_pos\ttype\theterozygosity\tsequence\n'  
    FILE.write(line)
    #generate 100 indels
    num_of_indels=100
    start_pos=1
    for i in range(num_of_indels):
        start_pos=random.randrange(start_pos,start_pos+100)
        line='chr1\t'
        line+=str(start_pos)+'\t'
        size=random.randrange(20,100)
        end_pos=start_pos+size-1
        line+=str(end_pos)+'\t'
        start_pos+=1000 
        if random.random()>0.5:
            line+='INS\t'
        else:
            line+='DEL\t'
        if random.random()>0.5:
            line+='HOM\t'
        else:
            line+='HET\t'
        seq=''
        for j in range(size):
            seq+='A'
        line+=seq+'\n'
        FILE.write(line) 
    FILE.close() 
    
def post_processing_demo():
    exe_dir=props["EXE_DIR"]
    print "........................................."
    print ".......... POST PROCESSING .............."
    print "........................................."
    command=exe_dir+"/post_processing/modil_post_processing.py chr1"
    os.system(command)


def main(argv):
    #handle_command_line_args(argv)
    global target_chr
    initialize_global_vars()
    target_chr='chr1'

    input_data_dir=props["INPUT_DATA_DIR"]
    EXE_DIR=props["EXE_DIR"]

    #############################
    ## extract Mills indels - don't use it's too heavy for simple demo
    #extract_mills_indels() 
    #remove_dup_in_mill_indels()
    ###############################

    ##############################
    ## generate simulated indels
    #gen_simulated_indels()


    #################################
    ## 2.create simulated donor genome (diploid)
    create_simul_donor_genome(0) ## all SV
    create_simul_donor_genome(1) ## hetoro
    make_no_new_line_file()


    ##################################
    ## 3.generate chain for loc converter
    generate_chain(0)
    generate_chain(1)

    ##############################
    ## 4.1 make matepairs
    ## what is this step for?
    ## coverage coverage_per_run * step
    
    #step = int(sys.argv[1]) ## input argement
    coverage_per_run=30
    generate_matepairs(0,coverage_per_run,0) 
    generate_matepairs(1,coverage_per_run,0) 
    

    ################################################
    ## 4.2 reverse-complement for the right read
    #reverse_complement_matepairs() 

    ################################
    ## 5. make input data for MoDIL
    input_simul_db(0,0) 
    input_simul_db(1,0) 
   

    #########################################
    ## 6. merge interesting files  & sort
    command='cat %s/%s.0.* %s/%s.1.* > %s/%s_' % (input_data_dir, target_chr, input_data_dir,target_chr, input_data_dir,target_chr)
    os.system(command)
    # sorting interesting file
    command='sort -S 1G -k 1n,1 -k 2n,2 %s/%s_>%s/%s; rm %s/%s_' % (input_data_dir,target_chr,input_data_dir, target_chr, input_data_dir, target_chr)
    os.system(command)
    command='rm %s/%s.0.*; rm %s/%s.1.*;' % (input_data_dir, target_chr, input_data_dir, target_chr)
    os.system(command)

    ###########################################
    ## generate pofy
    command="python %s/setup.py 1" %(EXE_DIR)
    print command
    get_sys_command_result(command)

    ###########################################
    ## clustering
    clustering_demo()

    ###########################################
    ## inference
    inference_demo()

    ###########################################
    ## post-processing
    post_processing_demo() 



if __name__=="__main__":
    main(sys.argv[1:])
