#!/usr/bin/env python

from __future__ import division
import os
import sys
import datetime
import logging
import math
import MPProperties
import Statistics
from sets import Set
import random
import gzip

# logger configuration
mean_list={}
std_list={}

alpha={}
getprob={}
coverage_list={}
#---------
search=None
props = {}
chr_list=['chr1','chr2', 'chr3','chr4','chr5','chr6','chr7','chr8','chr9','chr10','chr11','chr12','chr13','chr14','chr15','chr16','chr17','chr18','chr19','chr20', 'chr21','chr22','chrX','chrY']

class Mem_Searcher:
    def __init__(self, filename):
        self.true_indel_list=[]
        self.f = open(filename, 'rb')
        line=self.f.readline() # first line
        line=self.f.readline()
        cnt=0
        while line:
            tmp=line.split()
            item_cnt=0
            tmp_=[]
            for item in tmp:
                item_cnt+=1
                if item_cnt==6:
                    tmp_.append(len(item))
                else:
                    tmp_.append(item)

            self.true_indel_list.append(tmp_)
            cnt+=1
            line=self.f.readline()
        self.length = cnt-1
        self.f.close()

    def find(self, chr, start, end):
        low = 0
        high = self.length
        while low < high:

            if abs(low-high)<1: # low == high
                break
            mid = (low+high)/2  # update position
            p = int(mid)
            #while p > 0:
            #    self.f.seek(p)
            #    if int(p)==0:
            #        break
            #    elif self.f.read(1) == '\n': break
            #    p -= 1
            tmp=self.true_indel_list[p]
            if tmp==[]:
                low=mid+1
                continue
            chr_=tmp[0]
            start_=int(tmp[1])

            if chr_<chr:
                low = mid+1
            elif chr_>chr:
                high = mid
            elif start_<start:
                low = mid+1
            else:
                high = mid
        p = max(int(low)-1, 0)
	#if start==245415724:
	#    print p
        #while p > 0:
        #    self.f.seek(p)
        #    if int(p)==0:
        #        break
        #    elif self.f.read(1) == '\n': break
        #    p -= 1

        result = []
        counter=0
        while True:
            tmp = self.true_indel_list[p]
            chr_=tmp[0]
            start_=int(tmp[1])
            end_=int(tmp[2])
	    het_=tmp[4]
            #if not line or chr_!=chr or start_>end:
            #print start_, end_
            if chr_!=chr or start_>end:
                break
            elif max(start_, start)<=min(end_, end):
                result.append([chr_,start_,end_, tmp[5], tmp[3],het_])
                counter+=1
            p+=1
            if p>self.length:
                break
        return result


def complement(seq):
    #complement = {'A': 'T', 'C': 'G', 'G': 'C', 'T': 'A'}
    complement = {'A': 'T', 'C': 'G', 'G': 'C', 'T': 'A', 'N':'N'}
    complseq = [complement[base] for base in seq]
    return complseq



def reverse_complement(seq):
    seq = list(seq)
    seq.reverse()
    return ''.join(complement(seq))

def initialize_global_vars():
    # set the properties hash
    global props
    # read the properties hash
    props = MPProperties.read_config()


def get_sys_command_result(command):
    result = []
    command_result_pipe = os.popen(command)
    while True:
        line = command_result_pipe.readline()
        if not line:
            break
        line = line.strip()
        result.append(line)
    command_result_pipe.close()
    return result

#---------

def print_message_and_current_time(message):
    curr_time = datetime.datetime.now().ctime()
    curr_time_msg = message + curr_time
    
    # log the time message
    log.info(curr_time_msg)
    
    # print message and current time
    print curr_time_msg
    print

#---------


# given a (one-liner) SQL statement, return the command needed
# at the command prompt to execute this on this system
#---------

def return_list_from_result_set_column(mysqldb_result_set, column_index):
    result = []
    for result_set_row in mysqldb_result_set:
        result.append(result_set_row[column_index])
    return result

#---------

def return_str_list_from_list(list):
    result = []
    for item in list:
        result.append(str(item))
    return result

#---------

class Searcher:
    def __init__(self, filename):
        self.f = open(filename, 'rb')
        self.f.seek(0,2)
        self.length = self.f.tell()

    def find(self, chr, start, end):
        low = 0
        high = self.length
        while low < high:
            
            if abs(low-high)<1: # low == high
                break
            mid = (low+high)/2  # update position
            p = mid
            while p > 0:
                self.f.seek(p)
                if int(p)==0:
                    break
                elif self.f.read(1) == '\n': break
                p -= 1
            line = self.f.readline()
            tmp=line.split()
            if tmp==[]:
                low=mid+1
                continue
            chr_=tmp[0]
            start_=int(tmp[1])

            if chr_<chr:
                low = mid+1
            elif chr_>chr:
                high = mid
            elif start_<start:
                low = mid+1
            else:
                high = mid
        p = low
        while p > 0:
            self.f.seek(p)
            if int(p)==0:
                break
            elif self.f.read(1) == '\n': break
            p -= 1

        result = []
        counter=0
        while True:
            line = self.f.readline()
            tmp=line.split()
            if tmp==[]:
                break
            chr_=tmp[0]
            start_=int(tmp[1])
            end_=int(tmp[2])
            if not line or chr_!=chr or start_>end:
                break
            result.append([chr_,start_,end_, len(tmp[5]), tmp[3]])
            counter+=1
        return result

#---------    

class Searcher_print:
    def __init__(self, filename):
        self.f = open(filename, 'rb')
        self.f.seek(0,2)
        self.length = self.f.tell()

    def find(self, chr, start, end):

        low = 0
        high = self.length
        while low < high:
            if abs(low-high)<1:
                break
            mid = (low+high)/2
            p = mid
            while p > 0:
                self.f.seek(p)
                if self.f.read(1) == '\n': break
                p -= 1
            line = self.f.readline()
            tmp=line.split()
            if tmp==[]:
                low=mid+1
                continue
            chr_=tmp[1]
            start_=int(tmp[2])

            if chr_<chr:
                low = mid+1
            elif chr_>chr:
                high = mid
            elif start_<start:
                low = mid+1
            else:
                high = mid
        p = low
        while p > 0:
            self.f.seek(p)
            if self.f.read(1) == '\n': break
            p -= 1

        result = []
        counter=0
        prev_start_=0
        while True:
            line = self.f.readline()
            tmp=line.split()
            if tmp==[]:
                break
            chr_=tmp[1]
            start_=int(tmp[2])
            end_=int(tmp[3])
            if not line or chr_!=chr or start_>end:
                break
            result.append([chr_,start_,end_, tmp[0]])
            counter+=1
        return result



#######################################################################################
#######################################################################################
#######################################################################################

#89811_1_19      chr1    179451492       179451601       0       1       -25.0   0.0     2.84129998435e-08       18002662
def evaluate_results():
    chr='chr1'
    exe_dir=props["EXE_DIR"]
    result_dir=props["RESULTS_DIR"]
    f_input="%s/POST_PROCESSING/%s/%s_all_results.txt.final.final.FDR" % (result_dir,chr,chr)
    modil_res_list=[]
    #FILE_RES=open(result_dir+'/chr1_MoDIL_hg17_final.results','r')
    FILE_RES=open(f_input,'r')
    #FILE_RES=gzip.open(result_dir+'/res_all.txt.formatted.gz','r')
    #FILE_RES=open(result_dir+'/chr1_MoDIL_hg17_final.results.no_factor','r')
    #FILE_RES=open(result_dir+'/chr1_MoDIL_hg17_final.results.realmapping','r')

    print '-----------------------------------------------------'
    #print '#############################'
    #print '1. READ MoDIL RESULTS'
    progress=0
    line=FILE_RES.readline()
    while line:
        #if progress % 10000 ==0:
        #    print 'read progress: ', progress
	#print line
        progress+=1

        tmp=line.split()
	map_id=tmp[0]
	chrom=tmp[1]
	start_pos=int(tmp[2])
	end_pos=int(tmp[3])
	indel_size=float(tmp[6])
	phat=float(tmp[7])
	pval=float(tmp[8])
	clusid=tmp[9]
            
	if indel_size>0:
	    type='INS'
            #if abs(indel_size)>100:# or abs(indel_size)>10000:
            #    line=FILE_RES.readline()
            #    continue
	else:
	    type='DEL'
            if abs(indel_size)>100000:
                line=FILE_RES.readline()
                continue

	l=[clusid, start_pos, end_pos, abs(indel_size), type, phat, pval]
	modil_res_list.append(l)
	#print l
        line=FILE_RES.readline()
    FILE_RES.close()
    modil_res_list.sort()
	

    #print '#####################################'
    #print '2. READ ground truth indel results'
    indel_list=[]
    FILE=open(exe_dir+'/data/simul_indels_input.txt','r')
    line=FILE.readline()
    line=FILE.readline()
    indel_id=1
    while line:
	tmp=line.split()
	chrom=tmp[0]
	start_pos=int(tmp[1])
	end_pos=int(tmp[2])
	type=tmp[3]
	hom=tmp[4]
	indel_length=len(tmp[5])
        #if type=='INS':
        #    if abs(indel_length)>100:
        #        line=FILE.readline()
        #        continue
        #elif type=='DEL':
        #    if abs(indel_length)<100:
        #        line=FILE.readline()
        #        continue
            
 	l=[indel_id, start_pos, end_pos, abs(indel_length), type, hom]
	indel_list.append(l)
	indel_id+=1
	#print l
        line=FILE.readline()
    FILE.close()
    indel_list.sort()
    
    search=Mem_Searcher(exe_dir+'/data/simul_indels_input.txt')

    #print '###########################'
    #print '3. Find overlapped indels'
    overlap_cnt=0
    overlap_cnt_range_ins=[0,0,0,0,0,0,0]
    overlap_cnt_range_del=[0,0,0,0,0,0,0]
    progress=0
    true_overlap_set=Set()

    true_overlap_range_ins=[Set(), Set(), Set(), Set(), Set(), Set(), Set()]
    true_overlap_range_del=[Set(), Set(), Set(), Set(), Set(), Set(), Set()]
    found_flag=[{},{},{},{},{},{},{}]
    found_flag_precision=[{},{},{},{},{},{},{}]
    het_stat=[Set(), Set()]
    undefined_het=Set()
    for indel_call in modil_res_list:
        #if progress % 10000 ==0:
        #    print 'find overlap: ', progress
	keygen_indel=''
	for indel in indel_call[0:3]:
	    keygen_indel+=str(indel)+'_'

        progress+=1
	call_start_pos=indel_call[1]
	call_end_pos=indel_call[2]
	call_phet=indel_call[5]
	if call_phet>0.8:
	    call_het_type='HET'
	elif call_phet<0.2:
	    call_het_type='HOM'
	else:
	    call_het_type='NA'

        ###################################
        res_list=search.find(chr, call_start_pos, call_end_pos)
	if indel_call[0]=='47710665':
	    print res_list
	already_counted=0
	for true_indel in res_list:
	    true_start_pos=true_indel[1]
	    true_end_pos=true_indel[2]
	    het_type=true_indel[5]
	    #print call_het_type, het_type
	    
            #if true_indel[4]=='INS':
            #    if abs(true_indel[3])>100:
            #        continue
            #elif true_indel[4]=='DEL':
            #    if abs(true_indel[3])<100:
            #        continue
	    cond=[0,0,0]
	    if indel_call[4]==true_indel[4]:
       	        cond[0]=1
	    if max(call_start_pos,true_start_pos)<=min(call_end_pos,true_end_pos):
       	        cond[1]=1
	    cond[2]=1
	    #if abs(indel_call[3]-true_indel[3])<20:
	#	cond[2]=1
		
	    
	    if cond[0]==1 and cond[1]==1 and cond[2]==1:
                if abs(indel_call[3])>=100:
		    if call_het_type=='NA':
		        undefined_het.add(keygen_indel)
		    elif call_het_type==het_type:
		        het_stat[0].add(keygen_indel)
	    	    else:
		        het_stat[1].add(keygen_indel)
			
	   	overlap_cnt+=1
                keygen=''
                for k_true in true_indel:
                    keygen+=str(k_true)+'_'
                true_overlap_set.add(keygen)

                #if (indel_call[3]>=0 and indel_call[3]<=4) and (true_indel[3]>=0 and true_indel[3]<=4):
                #    index=0
                #elif (indel_call[3]>=5 and indel_call[3]<=9) and (true_indel[3]>=5 and true_indel[3]<=9):
                #    index=1
                #elif (indel_call[3]>=10 and indel_call[3]<=14) and (true_indel[3]>=10 and true_indel[3]<=14):
                #    index=2
                #elif (indel_call[3]>=15 and indel_call[3]<=19) and (true_indel[3]>=15 and true_indel[3]<=19):
                #    index=3
                #elif (indel_call[3]>=20 and indel_call[3]<=29) and (true_indel[3]>=20 and true_indel[3]<=29):
                #    index=4
                #elif (indel_call[3]>=30 and indel_call[3]<=39) and (true_indel[3]>=30 and true_indel[3]<=39):
                #    index=5
                #elif (indel_call[3]>=40) and (true_indel[3]>=40):
                #    index=6
                #else:
                #    continue

		###########################
		## predicted indel calls
                if (indel_call[3]>=1 and indel_call[3]<=4):
                    index=0
                elif (indel_call[3]>=5 and indel_call[3]<=9):
                    index=1
                elif (indel_call[3]>=10 and indel_call[3]<=14):
                    index=2
                elif (indel_call[3]>=15 and indel_call[3]<=19):
                    index=3
                elif (indel_call[3]>=20 and indel_call[3]<=39):
                    index=4
                elif (indel_call[3]>=40 and indel_call[3]<=99):
                    index=5
                elif (indel_call[3]>=100):
                    index=6

		###########################
		## true indel calls
                if true_indel[3]>=1 and true_indel[3]<=4:
                    index2=0
                elif true_indel[3]>=5 and true_indel[3]<=9:
                    index2=1
                elif true_indel[3]>=10 and true_indel[3]<=14:
                    index2=2
                elif true_indel[3]>=15 and true_indel[3]<=19:
                    index2=3
                elif true_indel[3]>=20 and true_indel[3]<=39:
                    index2=4
                elif true_indel[3]>=40 and true_indel[3]<=99:
                    index2=5
                elif true_indel[3]>=100:
                    index2=6
	
                if true_indel[4]=='INS': ## insertion
		    if already_counted==0:
                        overlap_cnt_range_ins[index]+=1
			already_counted=1
                    true_overlap_range_ins[index2].add(keygen)  
                    #if index==4:
                    #    print true_indel[3], indel_call[3]
                else: ## deletion
		    if already_counted==0:
                        overlap_cnt_range_del[index]+=1
			already_counted=1
                    true_overlap_range_del[index2].add(keygen)  

	        found_flag[index2][keygen]=1
                found_flag_precision[index][keygen_indel]=1

	        if indel_call[0]=='810089':
		    print indel_call, cond, index

    ###############################################################
    ## checking - precision
    #l=[clusid, start_pos, end_pos, abs(indel_size), type, phat, pval]
    #print found_flag_precision[4].keys()
    for indel_call in modil_res_list:
	keygen_indel=''
	for indel in indel_call[0:3]:
	    keygen_indel+=str(indel)+'_'

        found=0
        if keygen_indel in found_flag_precision[4].keys():
	    found=1

        if found==0 and indel_call[4]=='INS' and indel_call[3]>=100:# and indel_call[3]<=39:
            print 'false pos: ', indel_call


    ##################################################################
    ## checking purposes - recall
    for true_indel in indel_list:
	start_pos=true_indel[1]
	end_pos=true_indel[2]
	indl_size=true_indel[3]
	type=true_indel[4]
	true_indel_=['chr1', start_pos, end_pos, indl_size, type]
	keygen=''
	for k_true in true_indel_:
	    keygen+=str(k_true)+'_'
	#print '2 ', keygen
        found=0
        for i in found_flag[4].keys():
            if i==keygen:
                found=1
                break
        #if found==0 and true_indel[3]>=20 and true_indel[3]<=39:
        #    print 'false neg: ', true_indel
        		
    ##################################################################
    #print true_overlap_range_ins
    #print 
    #print '#####################'
    #print '# 4. Statistics  '
    #print 

    ################################
    ## get range of indels
    true_indel_range_ins=[10**10,-1]
    call_indel_range_ins=[10**10,-1]
    true_indel_range_del=[10**10,-1]
    call_indel_range_del=[10**10,-1]

    t_range_ins=[0,0,0,0,0,0,0]
    t_range_del=[0,0,0,0,0,0,0]
    c_range_ins=[0,0,0,0,0,0,0]
    c_range_del=[0,0,0,0,0,0,0]

    for true_indel in indel_list:
        if true_indel[4]=='INS':
            true_indel_range_ins[0]=min(true_indel_range_ins[0],true_indel[3])
            true_indel_range_ins[1]=max(true_indel_range_ins[1],true_indel[3])
        elif true_indel[4]=='DEL':
            true_indel_range_del[0]=min(true_indel_range_del[0],true_indel[3])
            true_indel_range_del[1]=max(true_indel_range_del[1],true_indel[3])
            
        if true_indel[3]>=1 and true_indel[3]<=4:
            index=0
        elif true_indel[3]>=5 and true_indel[3]<=9:
            index=1
        elif true_indel[3]>=10 and true_indel[3]<=14:
            index=2
        elif true_indel[3]>=15 and true_indel[3]<=19:
            index=3
        elif true_indel[3]>=20 and true_indel[3]<=39:
            index=4
        elif true_indel[3]>=40 and true_indel[3]<=99:
            index=5
        elif true_indel[3]>=100:
            index=6
        else:
            print 'ERROR - check it out!!!'

        if true_indel[4]=='INS':
            t_range_ins[index]+=1
        else:
            t_range_del[index]+=1

    #print t_range_ins
    call_tot_num=0

    span_sum=[0,0]
    indel_tot_num=[0,0]
    for call_indel in modil_res_list:
        if call_indel[4]=='INS':
            call_indel_range_ins[0]=min(call_indel_range_ins[0],call_indel[3])
            call_indel_range_ins[1]=max(call_indel_range_ins[1],call_indel[3])
	    span_sum[0]+=call_indel[2]-call_indel[1]
	    indel_tot_num[0]+=1

        elif call_indel[4]=='DEL':
            call_indel_range_del[0]=min(call_indel_range_del[0],call_indel[3])
            call_indel_range_del[1]=max(call_indel_range_del[1],call_indel[3])
	    span_sum[1]+=call_indel[2]-call_indel[1]
	    indel_tot_num[1]+=1

	call_tot_num+=1

        if call_indel[3]>=1 and call_indel[3]<=4:
            index=0
        elif call_indel[3]>=5 and call_indel[3]<=9:
            index=1
        elif call_indel[3]>=10 and call_indel[3]<=14:
            index=2
        elif call_indel[3]>=15 and call_indel[3]<=19:
            index=3
        elif call_indel[3]>=20 and call_indel[3]<=39:
            index=4
        elif call_indel[3]>=40 and call_indel[3]<=99:
            index=5
        elif call_indel[3]>=100:
            index=6
        else:
            print 'EERRor'

        if call_indel[4]=='INS':
            c_range_ins[index]+=1
        else:
            c_range_del[index]+=1


    print 'undefined heterozygosity', len(undefined_het)
    print 'correct het:', len(het_stat[0]), 'wrong het', len(het_stat[1])

    if indel_tot_num[0]!=0 and indel_tot_num[1]!=0:
        print 'Span avg, [insertions:', span_sum[0]/indel_tot_num[0], ', deletions:', span_sum[1]/indel_tot_num[1],']'
    elif indel_tot_num[0]!=0:
        print 'Span avg, [insertions:', span_sum[0]/indel_tot_num[0],']'
    elif indel_tot_num[1]!=0:
        print 'Span avg, [deletions:', span_sum[1]/indel_tot_num[1],']'

	
    print 'INS - true indel range:', true_indel_range_ins
    print 'DEL - true indel range:', true_indel_range_del
    print 'INS - prediction indel range:', call_indel_range_ins
    print 'DEL - prediction indel range:', call_indel_range_del
    print 
    for i in range(7):
	if i==0:
            ss='['+str(1)+'-'+str(4)+']'
        elif i==4:
            ss='['+str(20)+'-'+str(39)+']'
        elif i==5:
            ss='['+str(40)+'-'+str(99)+']'
        elif i==6:
            ss='['+str(100)+'-]'
        else:
            ss='['+str(i*5)+'-'+str(i*5+4)+']'
        print 'INS - TRUE & PREDICTED ', ss, t_range_ins[i], c_range_ins[i]
    print 
    for i in range(7):
	if i==0:
            ss='['+str(1)+'-'+str(4)+']'
        elif i==4:
            ss='['+str(20)+'-'+str(39)+']'
        elif i==5:
            ss='['+str(40)+'-'+str(99)+']'
        elif i==6:
            ss='['+str(100)+'-]'
        else:
            ss='['+str(i*5)+'-'+str(i*5+4)+']'
        print 'DEL - TRUE & PREDICTED ', ss, t_range_del[i], c_range_del[i]


    ##################################
    ## compute recall & precision

    #true_tot_num=0
    #for true_indel in indel_list:
#	if true_indel[3]>=threshold_indel_size:	
#	    true_tot_num+=1
#    call_tot_num=0
#    for call_indel in modil_res_list:
#        if call_indel[3]>=threshold_indel_size:
#            call_tot_num+=1

    recall_ins=[0,0,0,0,0,0,0]
    recall_del=[0,0,0,0,0,0,0]
    precision_ins=[0,0,0,0,0,0,0]
    precision_del=[0,0,0,0,0,0,0]
    f_measure_ins=[0,0,0,0,0,0,0]
    f_measure_del=[0,0,0,0,0,0,0]
  

    #print 'tot ', c_range_ins
    #print 'overlap', overlap_cnt_range_ins 
    for i in range(7):
        if t_range_ins[i]==0:
            recall_ins[i]=-1
        else:
            recall_ins[i]=len(true_overlap_range_ins[i])/t_range_ins[i]

        if t_range_del[i]==0:
            recall_del[i]=-1
        else:
            recall_del[i]=len(true_overlap_range_del[i])/t_range_del[i]

        if c_range_ins[i]==0:
            precision_ins[i]=-1
        else:
            precision_ins[i]=overlap_cnt_range_ins[i]/c_range_ins[i]

        if c_range_del[i]==0:
            precision_del[i]=-1
        else:
            precision_del[i]=overlap_cnt_range_del[i]/c_range_del[i]



	################################################################
        if (precision_ins[i]+recall_ins[i]==0) or precision_ins[i]<0 or recall_ins[i]<0:
            f_measure_ins[i]=0
        else:
            f_measure_ins[i]=(2*precision_ins[i]*recall_ins[i])/(precision_ins[i]+recall_ins[i])

        if (precision_del[i]+recall_del[i]==0) or precision_del[i]<0 or recall_del[i]<0:
            f_measure_del[i]=0
        else:
            f_measure_del[i]=(2*precision_del[i]*recall_del[i])/(precision_del[i]+recall_del[i])
	################################################################

    #print 'tot num of true indels (>=', threshold_indel_size,')', true_tot_num
    #print 'tot num of predicted indels (>=', threshold_indel_size,')', call_tot_num
    print
    print '( tot overlapped indels: ', overlap_cnt, ')'
    print '( tot indels found (non-redundant): ', len(true_overlap_set), ')'
    print
    print 'INS - Recall: ', recall_ins
    print 'DEL - Recall: ', recall_del
    print
    print 'INS - Precision: ', precision_ins
    print 'DEL - Precision: ', precision_del
    print
    #print 'INS - F-measure: ', f_measure_ins
    #print 'DEL - F-measure: ', f_measure_del
    #print
    print '----------------------------------------------------------------------------'
    print

    #outfile_name='can_results_stats.%s.txt' % s_std
    #FILE_output=open(outfile_name,'w')

    #buf='INS - true indel range:'+str(true_indel_range_ins[0])+','+str(true_indel_range_ins[1])+'\n'
    #FILE_output.write(buf)
    #buf='DEL - true indel range:'+str(true_indel_range_del[0])+','+str(true_indel_range_del[1])+'\n'
    #FILE_output.write(buf)
    #buf='INS - prediction indel range:'+str(call_indel_range_ins[0])+','+str(call_indel_range_ins[1])+'\n'
    #FILE_output.write(buf)
    #buf='DEL - prediction indel range:'+str(call_indel_range_del[0])+','+str(call_indel_range_del[1])+'\n'
    #FILE_output.write(buf)

    #buf='tot num of true indels (>='+str(threshold_indel_size)+') : '+str(true_tot_num)+'\n'
    #FILE_output.write(buf)
    #buf='tot num of predicted indels (>='+str(threshold_indel_size)+') : '+str(call_tot_num)+'\n'
    #FILE_output.write(buf)
    #buf='tot overlapped indels: '+str(overlap_cnt)+'\n'
    #FILE_output.write(buf)
    #buf='tot true indels found: '+str(len(true_overlap_set))+'\n'
    #FILE_output.write(buf)
    #buf='Recall: '+str(recall)+'\n'
    #FILE_output.write(buf)
    #buf='Precision: '+str(precision)+'\n'
    #FILE_output.write(buf)
    #buf='F-measure: '+str(f_measure)+'\n'
    #FILE_output.write(buf)
    
    #FILE_output.close()




    #############################################
    ## print results  with varing size of indels
    



def look_at_coverage(chr,start_pos,end_pos,print_f,type, indel_size, search_fwd, search_rev):
    ALL_DATA_SOURCES=props["ALL_DATA_SOURCES"]
    interest=[start_pos, end_pos]
    rr=300
    res=search_fwd.find(chr, interest[0]-rr, interest[1]+rr)
    res2=search_rev.find(chr, interest[0]-rr, interest[1]+rr)
    #print  "FROM:", interest[0]-rr, "TO:", interest[1]+rr
    #print res, res2

    result=res+res2
    result.sort()
    prev_start=0
    ins_size={}
    pos={}

    print len(result)
    #if len(result)>3000: # >80 coverage
    #    print "!!! skip because of too large coverage !!!", len(result)
    #    return [100,50]

    for r1 in result:
        start=r1[1]
        end=r1[2]
        template_id=r1[3]
        tid=template_id.split('/')

        for r2 in result:
            start2=r2[1]
            end2=r2[2]
            template_id2=r2[3]
            tid2=template_id2.split('/')
            if tid[0]==tid2[0] and tid[1]!=tid2[1]:
                if start<start2:
                    isize=end2-start
                    pos[str(start)+str(end)+tid[0]]=[end,start2]
                else:
                    isize=end-start2
                    pos[str(start)+str(end)+tid[0]]=[end2,start]
                ins_size[str(start)+str(end)+tid[0]]=isize
                #print tid, tid2, isize
                break
            else:
                ins_size[str(start)+str(end)+tid[0]]=0
                pos[str(start)+str(end)+tid[0]]=[0,0]


    putative_size=Set()
    struct_cnt=0
    read_cnt=0
    for r in result:
        start=r[1]
        end=r[2]
        template_id=r[3]
        tid=template_id.split('/')
        span=208-ins_size[str(start)+str(end)+tid[0]]
        look=0
        if span<208:
            look=1

        if look==1:
            #print r[1],r[2], span
            arrow=''
            addi=''
            addi2=''
            curr_loc_start=pos[str(start)+str(end)+tid[0]][0]
            curr_loc_end=pos[str(start)+str(end)+tid[0]][1]
	    #if max(pos[str(start)+str(end)+template_id][0],interest[0])<min(pos[str(start)+str(end)+template_id][1],interest[1]):
            if int(start)>interest[0] and int(start)<interest[1]:
                arrow=" <=== PUTATIVE READ ==="
                read_cnt+=1

            if max(curr_loc_start,interest[0]) <= min(curr_loc_end,interest[0]):
                #arrow=" <=== PUTATIVE MATEPAIR ==="
                putative_size.add(tid[0])
                if type=='D' and span<0:
                    if abs(span)>abs(indel_size):
                        struct_cnt+=1
                elif type=='I' and span>0:
                    if abs(span)>abs(indel_size):
                        struct_cnt+=1

            if curr_loc_start!=0:
                if end==curr_loc_start:
                    addi2="left"
                else:
                    addi2="right"
                if print_f==1:
                    print "[read:", start, end,"] [matepair:", pos[str(start)+str(end)+tid[0]][0], pos[str(start)+str(end)+tid[0]][1], "]",span, tid, arrow, addi, addi2
            else:
                if print_f==1:
                    print "no pairs:",start, end, span, tid, arrow
        else:
            continue

        if abs(start-prev_start)>10:
            if print_f==1:
                print "...check jump...", abs(start-prev_start)
                print
        prev_start=r[1]

    if print_f==1:
        print "cluster_size:", len(putative_size), "structvar size", struct_cnt, "read coverage", read_cnt

    return [len(putative_size), struct_cnt/2]




def main(argv):
    #handle_command_line_args(argv)
    initialize_global_vars()
    target_chr='chr1'

    ###############################################
    ## 8. evalute can's results
    #std_level=1.5
    #std_level_list=[1, 1.5, 2, 2.5, 3, 3.5, 4]

    evaluate_results()

if __name__=="__main__":
    main(sys.argv[1:])

